@@ -97,7 +97,7 @@ read_freebsd_apm_data()
 #define	PROC_APM_FILE	"/proc/apm"
 
 static void
-read_linux_apm_data()
+read_linux_apm_data(void)
 	{
 	FILE	*f;
 	gchar	buf[128];
@@ -164,26 +164,38 @@ read_netbsd_apm_data()
   /* Themers need to be able to see the apm monitor.
   */
 static void
-read_apm_demo()
+read_apm_demo(void)
 	{
-	static gint	bump;
+	static gint	bump = 60;
 
 	sys_apm.battery_is_available = TRUE;
 
-	if (GK.two_second_tick)
-		bump = ++bump % 20;
+	if (!GK.five_second_tick)
+		return;
 
-	sys_apm.ac_is_on_line = bump & 2;
-	sys_apm.battery_is_charging = FALSE;
-	sys_apm.battery_percentage = sys_apm.ac_is_on_line ? 90 : 10;
-	sys_apm.battery_time_left = sys_apm.ac_is_on_line ? 225 : 1;
+	if (bump <= 5)
+		bump = 60;
+	bump -= 5;
+	sys_apm.ac_is_on_line = bump > 45;
+	if (sys_apm.ac_is_on_line)
+		{
+		sys_apm.battery_is_charging = TRUE;
+		sys_apm.battery_time_left = 200 + (60 - bump) * 20;
+		sys_apm.battery_percentage = sys_apm.battery_time_left / 5;
+		}
+	else
+		{
+		sys_apm.battery_is_charging = FALSE;
+		sys_apm.battery_time_left = bump;
+		sys_apm.battery_percentage = 1 + bump;
+		}
 	}
 
 
 /* ----- Pick a system interface ----------------------------------------- */
 
 static gint
-setup_apm_interface()
+setup_apm_interface(void)
 	{
 	gint	available	= FALSE;
 
@@ -215,15 +227,17 @@ setup_apm_interface()
 
 /* ======================================================================== */
 
+Monitor			*mon_apm;
+
 static Panel	*apm;
 
 static Krell	*apm_krell;
 static Decal	*power_source_decal;
 static Decal	*minutes_left_decal;
 
-static Launcher	launch;
+static Alert	*apm_alert;
 
-static gint		apm_visible;
+static Launcher	launch;
 
 static gint		style_id;
 
@@ -261,24 +275,30 @@ draw_time_left_decal(gint force)
 
 
 static void
-update_apm()
+update_apm(void)
 	{
 	gint			t, decal;
 	static gboolean	warning;
 
-	if (enable_apm == FALSE)
+	if (!enable_apm)
 		return;
-	if (GK.two_second_tick || warning)
+	if (GK.five_second_tick || warning)
 		{
 		(*read_apm_data)();
 
 		warning = FALSE;
+		gkrellm_check_alert(apm_alert, sys_apm.battery_time_left);
 		t = sys_apm.battery_display_percent ? sys_apm.battery_percentage
 							: sys_apm.battery_time_left;
 		if (sys_apm.ac_is_on_line)
+			{
 			decal = D_MISC_AC;
+			gkrellm_reset_alert(apm_alert);
+			gkrellm_freeze_alert(apm_alert);
+			}
 		else
 			{
+			gkrellm_thaw_alert(apm_alert);
 			decal = D_MISC_BATTERY;
 			if (t < 20)
 				{
@@ -293,7 +313,7 @@ update_apm()
 		apm_krell->full_scale = 100;
 		apm_krell->previous = 0;
 		gkrellm_update_krell(apm, apm_krell, sys_apm.battery_percentage);
-		gkrellm_draw_layers(apm);
+		gkrellm_draw_panel_layers(apm);
 		}
 	}
 
@@ -312,13 +332,15 @@ apm_expose_event(GtkWidget *widget, GdkEventExpose *ev)
 	}
 
 static gint
-apm_display_change (GtkWidget *widget, GdkEventButton *event)
+cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
 	{
-	if (event->button == 2 || event->button == 3)
+	if (ev->button == 3)
+		gkrellm_open_config_window(mon_apm);
+	else if (ev->button == 2)
 		{
-		sys_apm.battery_display_percent = 1 - sys_apm.battery_display_percent;
+		sys_apm.battery_display_percent = !sys_apm.battery_display_percent;
 		draw_time_left_decal(TRUE);
-		gkrellm_draw_layers(apm);
+		gkrellm_draw_panel_layers(apm);
 		}
 	return TRUE;
 	}
@@ -326,7 +348,6 @@ apm_display_change (GtkWidget *widget, GdkEventButton *event)
 static void
 create_apm(GtkWidget *vbox, gint first_create)
 	{
-	TextStyle	*ts;
 	Style		*style;
 	gint		x, w;
 
@@ -334,15 +355,8 @@ create_apm(GtkWidget *vbox, gint first_create)
 		{
 		apm = gkrellm_panel_new0();
 		}
-	else
-		{
-		gkrellm_destroy_decal_list(apm);
-		gkrellm_destroy_krell_list(apm);
-		}
-
 	style = gkrellm_meter_style(style_id);
 
-
 	power_source_decal = gkrellm_create_decal_pixmap(apm, GK.decal_misc_pixmap,
 			GK.decal_misc_mask, N_MISC_DECALS, style, style->margin + 2, -1);
 
@@ -350,17 +364,17 @@ create_apm(GtkWidget *vbox, gint first_create)
 	|  x position at left margin, so calculate the override values.
 	|  Cannot override w after the create.
 	*/
-	ts = gkrellm_meter_textstyle(style_id);
 	x = style->margin + power_source_decal->w + 6;
-	w = UC.chart_width - x - style->margin;
-	minutes_left_decal = gkrellm_create_decal_text(apm, "100", ts,
+	w = gkrellm_chart_width() - x - style->margin;
+	minutes_left_decal = gkrellm_create_decal_text(apm, "100",
+						gkrellm_meter_textstyle(style_id),
 						style, x, -1, w);	/* -1 means use y default */
 
 	apm_krell = gkrellm_create_krell(apm, gkrellm_krell_meter_image(style_id),
 								style);
 
-	gkrellm_configure_panel(apm, NULL, style);
-	gkrellm_create_panel(vbox, apm, gkrellm_bg_meter_image(style_id));
+	gkrellm_panel_configure(apm, NULL, style);
+	gkrellm_panel_create(vbox, mon_apm, apm);
 
 	/* Center the decals with respect to each other.
 	*/
@@ -380,27 +394,47 @@ create_apm(GtkWidget *vbox, gint first_create)
 		{
 		(*(read_apm_data))();
 		if (! sys_apm.battery_is_available)
-			gtk_widget_hide(apm->hbox);
-		else
-			{
-			apm_visible = TRUE;
-			gkrellm_monitor_height_adjust(apm->h);
-			}
+			gkrellm_panel_hide(apm);
 		}
 	else
-		gtk_widget_hide(apm->hbox);
+		gkrellm_panel_hide(apm);
+
 
 	if (first_create)
 		{
 		gtk_signal_connect(GTK_OBJECT (apm->drawing_area), "expose_event",
 				(GtkSignalFunc) apm_expose_event, NULL);
 		gtk_signal_connect(GTK_OBJECT (apm->drawing_area), "button_press_event",
-				(GtkSignalFunc) apm_display_change, NULL);
+				(GtkSignalFunc) cb_panel_press, NULL);
          }
 	gkrellm_setup_launcher(apm, &launch, METER_PANEL_TYPE, 0);
 	}
 
+static void
+cb_apm_alert_trigger(Alert *alert, gpointer data)
+	{
+	AlertDecal	*ad;
+	Decal		*d;
 
+	alert->panel = apm;
+	ad = &alert->ad;
+	d = minutes_left_decal;
+	ad->x = d->x + 1;
+	ad->y = d->y - 2;
+	ad->w = d->w - 1;
+	ad->h = d->h + 4;
+	gkrellm_render_default_alert_decal(alert);
+	}
+
+static void
+create_apm_alert(void)
+	{
+	if (!apm_alert)
+		apm_alert = gkrellm_alert_create(NULL, _("APM"),
+				_("Battery Minutes Remaining Limits"),
+				FALSE, TRUE, TRUE, 500, 0, 1, 10, 0);
+	gkrellm_alert_trigger_connect(apm_alert, cb_apm_alert_trigger, NULL);
+	}
 
 
 #define	APM_CONFIG_KEYWORD	"apm"
@@ -412,6 +446,7 @@ save_apm_config(FILE *f)
 	fprintf(f, "%s launch1 %s\n", APM_CONFIG_KEYWORD, launch.command);
 	fprintf(f, "%s tooltip_comment %s\n",
 			APM_CONFIG_KEYWORD, launch.tooltip_comment);
+	gkrellm_save_alertconfig(f, apm_alert, APM_CONFIG_KEYWORD, NULL);
 	}
 
 static void
@@ -427,15 +462,29 @@ load_apm_config(gchar *arg)
 			launch.command = g_strdup(item);
 		else if (strcmp(config, "tooltip_comment") == 0)
 			launch.tooltip_comment = g_strdup(item);
+		else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
+			{
+			create_apm_alert();
+			gkrellm_load_alertconfig(&apm_alert, item);
+			}
 		}
 	}
 
 static GtkWidget	*enable_apm_button;
 static GtkWidget	*launch_entry,
 					*tooltip_entry;
+static GtkWidget	*alert_button;
+
+
+static void
+cb_apm_set_alert(GtkWidget *entry, gpointer data)
+	{
+	create_apm_alert();
+	gkrellm_alert_config_window(&apm_alert);
+	}
 
 static void 
-apply_apm_config()
+apply_apm_config(void)
 	{
 	gint	new;
 
@@ -453,7 +502,10 @@ apply_apm_config()
 			}
 		}
 	enable_apm = new;
-	gkrellm_enable_visibility(enable_apm, &apm_visible, apm->hbox, apm->h);
+	if (enable_apm)
+		gkrellm_panel_show(apm);
+	else
+		gkrellm_panel_hide(apm);
 	gkrellm_apply_launcher(&launch_entry, &tooltip_entry, apm,
 				&launch, gkrellm_launch_button_cb);
 	}
@@ -461,12 +513,27 @@ apply_apm_config()
 static void
 create_apm_tab(GtkWidget *tab_vbox)
 	{
-	GtkWidget	*table;
+	GtkWidget	*tabs, *table, *vbox, *vbox1, *hbox;
 
-	gkrellm_check_button(tab_vbox, &enable_apm_button, enable_apm, FALSE, 10,
+	tabs = gtk_notebook_new();
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
+	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);
+
+/* -- Setup tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Setup"));
+
+	vbox1 = gkrellm_framed_vbox(vbox, _("Options"),
+			4, FALSE, 0, 2);
+	gkrellm_check_button(vbox1, &enable_apm_button, enable_apm, FALSE, 10,
 			_("Enable APM"));
-	insert_expanded_filler(tab_vbox);
-	table = gkrellm_launcher_table_new(tab_vbox, 1);
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+	gkrellm_button_connected(hbox, &alert_button, FALSE, FALSE, 0,
+			cb_apm_set_alert, NULL, _("Alerts"));
+
+	vbox1 = gkrellm_framed_vbox_end(vbox, _("Launch Commands"),
+			4, FALSE, 0, 2);
+	table = gkrellm_launcher_table_new(vbox1, 1);
 	gkrellm_config_launcher(table, 0,  &launch_entry, &tooltip_entry, 
 					_("APM"), &launch);
 	}
@@ -500,6 +567,7 @@ init_apm_monitor(void)
 	enable_apm = FALSE;			/* Some people have probs reading /proc/apm */
 	monitor_apm.name=_(monitor_apm.name);
 	style_id = gkrellm_add_meter_style(&monitor_apm, APM_STYLE_NAME);
+	mon_apm = &monitor_apm;
     if (setup_apm_interface())
 		return &monitor_apm;
 	return NULL;
