@@ -19,8 +19,9 @@
 */
 
 /*
-|  1/23/2001 Hajimu UMEMOTO improvements with read_freebsd_cpu() and
-|			register_freebsd_cpus().
+|  4/22/2001  Solaris code contributed by Daisuke Yabuki <dxy@acm.org>
+|  1/23/2001  Hajimu UMEMOTO improvements with read_freebsd_cpu() and
+|               register_freebsd_cpus().
 | 10/12/2000  NetBSD code contributed by Anthony Mallet <metall@ficus.yi.org>
 |  2/25/2000  FreeBSD code contributed by Hajimu UMEMOTO ume@mahoroba.org
 */
@@ -35,8 +36,7 @@ GList	*cpu_mon_list;
 void	(*read_system_cpu_info)();
 
 static gint		n_cpus;
-static gint		smp_mode;
-gint			smp_cpus;
+gint			n_smp_cpus;
 
 
 /* ====== System dependent interface ====================================== */
@@ -168,7 +168,7 @@ static gulong	swapin,
   |  and ticks for cpu[i] are jiffies (1/CLK_TCK)
   */
 void
-read_proc_stat()
+read_proc_stat(void)
 	{
 	FILE		*f;
 	GList		*list;
@@ -237,11 +237,11 @@ read_proc_stat()
 			item = strtok(arg, " \t\n");
 			while (item)
 				{
-				/* It may evolve - won't know till the 2.4 dust settles */
+				/* disk_io lines in 2.4.x kernels have had 2 formats */
 				n = sscanf(item, "(%d,%d):(%*d,%lu,%lu,%lu,%lu)",
 						&major, &i_disk, &rb1, &rb2, &wb1, &wb2);
 				if (n == 6)	/* patched as of 2.4.0-test1-ac9 */
-					{		/* (major,disk):(total,rio,rblk,wio,wblk) */
+					{		/* (major,disk):(total_io,rio,rblk,wio,wblk) */
 					rblk = rb2;
 					wblk = wb2;
 					}
@@ -273,7 +273,7 @@ read_stat_swap(gulong *sin, gulong *sout)
 	}
 
 static void
-register_stat_cpus()
+register_stat_cpus(void)
 	{
 	FILE	*f;
 	CpuMon	*cpu;
@@ -298,10 +298,10 @@ register_stat_cpus()
 	/* There can be cpu and cpu0 only (single cpu machines with kernel 2.4
 	|  or 2.2 compiled for SMP), and in this case, eliminate cpu0.
 	*/
-	smp_cpus = n_cpus - 1;
-	if (smp_cpus == 1)	/* cpu and cpu0 but no cpu1 (not really SMP)	*/
+	n_smp_cpus = n_cpus - 1;
+	if (n_smp_cpus == 1)	/* cpu and cpu0 but no cpu1 (not really SMP)	*/
 		{
-		smp_cpus = 0;
+		n_smp_cpus = 0;
 		n_cpus = 1;
 		list = cpu_mon_list->next;	/* cpu0 */
 		cpu = (CpuMon *) list->data;
@@ -309,7 +309,7 @@ register_stat_cpus()
 		g_free(cpu);
 		cpu_mon_list = g_list_remove_link(cpu_mon_list, list);
 		}
-	if (smp_cpus)			/* The first one is the composite cpu	*/
+	if (n_smp_cpus)			/* The first one is the composite cpu	*/
 		((CpuMon *) cpu_mon_list->data)->is_composite = TRUE;
 	}
 #endif	/* __linux__ */
@@ -349,13 +349,133 @@ read_netbsd_cpu()
 #endif /* __NetBSD__ */
 
 
+/* ----- Solaris ------------------------------------------------------ */
+
+#if defined(__solaris__)
+
+#include <kstat.h>
+#include <sys/sysinfo.h>
+
+void
+read_solaris_cpu()
+{
+    CpuMon *cpu;
+    GList *list;
+    extern kstat_ctl_t *kc;
+    kstat_t *ksp;
+    cpu_stat_t cs;
+    gulong composite_user, composite_nice, composite_sys, composite_idle;
+    gchar *cpuid;
+#if 0
+    static gint data_read_tick = -1;
+
+    if (data_read_tick == GK.timer_ticks)        /* Only one read per tick */
+        return;
+    data_read_tick = GK.timer_ticks;
+#else
+    /* one read per tick seems to make older Ultra machines too busy */
+    if (!GK.second_tick)
+        return;
+#endif
+
+    composite_user = composite_nice = composite_sys = composite_idle = 0;
+
+    if (kstat_chain_update(kc) == -1) {
+        perror("kstat_chain_update");
+        return;
+    }
+
+    for (ksp = kc->kc_chain; ksp; ksp = ksp->ks_next) {
+        if (strcmp(ksp->ks_module, "cpu_stat")) 
+            continue;
+        if (kstat_read(kc, ksp, &cs) == -1) {
+            perror("kstat_read");
+            continue;
+        }
+        for (list = cpu_mon_list; list; list = list->next) {
+            cpu = (CpuMon *) list->data;
+            cpuid = g_strdup_printf("cpu%d", ksp->ks_instance);
+            if (strcmp(cpu->name, cpuid)) {
+                g_free(cpuid);
+                continue;
+            }
+            g_free(cpuid);
+
+            cpu->user = cs.cpu_sysinfo.cpu[CPU_USER];
+            cpu->nice = cs.cpu_sysinfo.cpu[CPU_WAIT];
+            cpu->sys  = cs.cpu_sysinfo.cpu[CPU_KERNEL];
+            cpu->idle = cs.cpu_sysinfo.cpu[CPU_IDLE];
+
+            if (n_smp_cpus) {
+                composite_user += cpu->user;
+                composite_nice += cpu->nice;
+                composite_sys  += cpu->sys;
+                composite_idle += cpu->idle;
+            }
+        } 
+    }
+
+    /* dump composite data onto the first element (composite element) */
+    if (n_smp_cpus) {
+        ((CpuMon *) cpu_mon_list->data)->user = composite_user;
+        ((CpuMon *) cpu_mon_list->data)->nice = composite_nice;
+        ((CpuMon *) cpu_mon_list->data)->sys  = composite_sys;
+        ((CpuMon *) cpu_mon_list->data)->idle = composite_idle;
+    }
+
+}
+
+/*
+ * note: on some SPARC systems, you can monitor temperature of CPUs 
+ * with kstat (unix::temperature:[min/max/state/trend...])
+ */
+
+static void 
+register_solaris_cpus() {
+    CpuMon *cpu;
+    extern kstat_ctl_t *kc;
+    kstat_t *ksp;
+
+    n_cpus = n_smp_cpus = 0;
+
+    if(kstat_chain_update(kc) == -1) {
+        perror("kstat_chain_update");
+        return;
+    } 
+
+    for (ksp = kc->kc_chain; ksp; ksp = ksp->ks_next) {
+        if (strcmp(ksp->ks_module, "cpu_stat")) 
+            continue;
+        if (kstat_read(kc, ksp, NULL) != -1) {
+            cpu = g_new0(CpuMon, 1);
+            cpu->name = g_strdup_printf("cpu%d", ksp->ks_instance);
+            cpu_mon_list = g_list_append(cpu_mon_list, cpu);
+            n_cpus++;
+        }
+    }
+
+    if (n_cpus > 1) 
+	n_smp_cpus++;
+
+    /* create composite cpu entry and add it at the head of the list */
+    if (n_smp_cpus) {
+        cpu = g_new0(CpuMon, 1);
+        cpu->name = g_strdup("cpu"); 
+        cpu->is_composite = TRUE;
+        cpu_mon_list = g_list_insert(cpu_mon_list, cpu, 0);
+    }
+
+}
+
+#endif /* __solaris__ */
+
 /* ----- Others ------------------------------------------------------- */
 #if defined(USE_LIBGTOP)
 
 #include <glibtop/cpu.h>
 
 static void
-read_glibtop_cpu()
+read_glibtop_cpu(void)
 	{
 	CpuMon		*cpu;
 	GList		*list;
@@ -374,9 +494,9 @@ read_glibtop_cpu()
 #endif
 
 
-#if !defined(__linux__)
+#if !defined(__linux__) && !defined(__solaris__)
 static void
-register_glibtop_cpus()
+register_glibtop_cpus(void)
 	{
 	CpuMon	*cpu;
 
@@ -384,14 +504,14 @@ register_glibtop_cpus()
 	cpu->name = g_strdup("cpu");
 	cpu_mon_list = g_list_append(cpu_mon_list, cpu);
 	n_cpus = 1;
-	smp_cpus = 0;
+	n_smp_cpus = 0;
 	}
 #endif
 
 /* ----- Pick a system interface ----------------------------------------- */
 
 static gint
-setup_cpu_interface()
+setup_cpu_interface(void)
 	{
 #if defined(__FreeBSD__)
 	register_freebsd_cpus();
@@ -405,6 +525,9 @@ setup_cpu_interface()
 #elif defined(__OpenBSD__)
 	register_glibtop_cpus();
 	read_system_cpu_info = read_openbsd_cpu;
+#elif defined(__solaris__)
+	register_solaris_cpus();   
+	read_system_cpu_info = read_solaris_cpu;
 #else
 	register_glibtop_cpus();
 	read_system_cpu_info = read_glibtop_cpu;
@@ -413,66 +536,142 @@ setup_cpu_interface()
 	}
 
 /* ======================================================================== */
+/* Exporting CPU data for plugins */
+
+gint
+gkrellm_smp_cpus(void)
+	{
+	return n_smp_cpus;
+	}
+
+gboolean
+gkrellm_cpu_stats(gint n, gulong *user, gulong *nice,
+			gulong *sys, gulong *idle)
+	{
+	GList	*list;
+	CpuMon	*cpu;
+
+	list = g_list_nth(cpu_mon_list, n);
+	if (!list)
+		return FALSE;
+	cpu = (CpuMon *) list->data;
+	if (user)
+		*user = cpu->user;
+	if (nice)
+		*nice = cpu->nice;
+	if (sys)
+		*sys = cpu->sys;
+	if (idle)
+		*idle = cpu->idle;
+	return TRUE;
+	}
+
+/* ======================================================================== */
+
+  /* Values for smp_mode - must be same order as buttons in config */
+#define	SMP_REAL_MODE				0
+#define	SMP_COMPOSITE_MODE			1
+#define	SMP_COMPOSITE_AND_REAL_MODE	2
+
+static gint		smp_mode;
+static Monitor	*mon_cpu;
 
 static gboolean	cpu_enabled  = TRUE;
-static gint		add_nice_mode;
+static gint		omit_nice_mode;
 static gint		ascent;
 static gint		style_id;
 static gint		mapped_sensors;
 static gint		sensor_separate_mode;
 
 
-static void
-draw_cpu_extra(Chart *cp, unsigned long l, unsigned long total)
-	{
-	TextStyle	*ts;
-	gint		n, y;
-	gchar		buf[32];
+static gchar    *text_format;
 
-	if (!cp || !total)
-		return;
-	ts = gkrellm_chart_textstyle(style_id);
-	if (ascent == 0)
-		ascent = gdk_char_height(ts->font, '8');
-	y = 4;
-	if (   cp->panel && cp->panel->label && sensor_separate_mode
-		&& (mapped_sensors & SENSOR_TEMPERATURE)
-		&& (mapped_sensors & SENSOR_FAN)
-	   )
+static void
+format_chart_text(CpuMon *cpu, gchar *buf, gint size)
+	{
+	Chart	*cp;
+	gchar	c, *s;
+	gint	len, sys, user, nice, total, t;
+
+	--size;
+	*buf = '\0';
+	cp = cpu->chart;
+	sys = gkrellm_get_current_chartdata(cpu->sys_cd);
+	user = gkrellm_get_current_chartdata(cpu->user_cd);
+	nice = gkrellm_get_current_chartdata(cpu->nice_cd);
+	total = sys + user;
+	if (!omit_nice_mode && !gkrellm_get_chartdata_hide(cpu->nice_cd))
+		total += nice;
+	for (s = text_format; *s != '\0' && size > 0; ++s)
 		{
-		y += ascent;
-		gkrellm_draw_chart_label(cp, ts, 4, y, cp->panel->label->string);
-		y += 2;
+		len = 1;
+		if (*s == '$' && *(s + 1) != '\0')
+			{
+			t = -1;
+			if ((c = *(s + 1)) == 'T')
+				t = total;
+			else if (c == 's')
+				t = sys;
+			else if (c == 'u')
+				t = user;
+			else if (c == 'n')
+				t = nice;
+			else if (c == 'L')
+				len = snprintf(buf, size, "%s", cp->panel->label->string);
+			else
+				{
+				*buf = *s;
+				if (size > 1)
+					{
+					*(buf + 1) = *(s + 1);
+					++len;
+					}
+				}
+			if (t >= 0)
+				{
+				t = ((200 * t / cpu->total_diff) + 1) / 2;
+				if (t > 100)
+					t = 100;
+				len = snprintf(buf, size, "%d%%", t);
+				}
+			++s;
+			}
+		else
+			*buf = *s;
+		size -= len;
+		buf += len;
 		}
-	y += ascent;
-	n = ((200 * l / total) + 1) / 2;
-	if (n > 100)
-		n = 100;
-	snprintf(buf, sizeof(buf), "%d%%", n);
-	gkrellm_draw_chart_label(cp, ts, 4, y, buf);
+	*buf = '\0';	
 	}
 
+static void
+draw_cpu_extra(CpuMon *cpu)
+	{
+	Chart	*cp  = cpu->chart;
+	gchar	buf[128];
+
+	if (!cp || cpu->total_diff <= 0)
+		return;
+	format_chart_text(cpu, buf, sizeof(buf));
+	gkrellm_draw_chart_text(cp, style_id, buf);
+	}
 
 static void
 refresh_cpu_chart(CpuMon *cpu)
 	{
 	Chart	*cp		= cpu->chart;
 
-	gkrellm_draw_chart(cp);
-
-	/* prevTotal is really current total.  It just will be prevTotal next
-	|  time store_chart_data() is called.
-	*/
+	gkrellm_draw_chartdata(cp);
 	if (cpu->extra_info)
-		draw_cpu_extra(cp, cpu->sys_user,cp->prevTotal - cpu->prev_total_save);
-	cp->need_redraw = FALSE;
+		draw_cpu_extra(cpu);
+	gkrellm_draw_chart_to_screen(cp);
 	}
 
 static void
 draw_sensor_decals(CpuMon *cpu)
 	{
 	Panel	*p		= cpu->chart->panel;
-	gchar	*name	= cpu->chart->name;
+	gchar	*name	= cpu->name;
 	gchar	units;
 	gfloat	t, f;
 	gint	toggle;
@@ -509,14 +708,14 @@ draw_sensor_decals(CpuMon *cpu)
 	}
 
 static void
-update_cpu()
+update_cpu(void)
 	{
 	GList		*list;
 	CpuMon		*cpu;
 	Chart		*cp;
 	Panel		*p;
 	Krell		*krell;
-	gulong		total;
+	gulong		total, krell_value;
 
 	GK.cpu_sys_activity = 0;
 	(*read_system_cpu_info)();
@@ -528,37 +727,35 @@ update_cpu()
 			continue;
 		cp = cpu->chart;
 		p = cp->panel;
-		if (smp_mode == 0)
-			GK.cpu_sys_activity += (int)(cpu->sys - cp->prevOut);
-		else if (list == cpu_mon_list)
-			GK.cpu_sys_activity = (int)(cpu->sys - cp->prevOut);
+		if (smp_mode == SMP_REAL_MODE)
+			GK.cpu_sys_activity += (int)(cpu->sys - cpu->sys_cd->previous);
+		else if (list == cpu_mon_list)	/* Use composite cpu values */
+			GK.cpu_sys_activity = (int)(cpu->sys - cpu->sys_cd->previous);
 		total = cpu->user + cpu->nice + cpu->sys + cpu->idle;
-		if (add_nice_mode)
-			cpu->user += cpu->nice;
-		krell = KRELL(cp->panel);
 		if (GK.second_tick)
 			{
-			cpu->sys_user = (cpu->sys - cp->prevOut) + (cpu->user - cp->prevIn);
-			cpu->prev_total_save = cp->prevTotal;
-			gkrellm_store_chart_data(cp, cpu->sys, cpu->user, total);
+			cpu->total_diff = (gint) (total - cpu->previous_total_sec);
+			cpu->previous_total_sec = total;
+			gkrellm_store_chartdata(cp, total, cpu->sys, cpu->user, cpu->nice);
 			refresh_cpu_chart(cpu);
 			}
-
 		if (   (GK.two_second_tick && !sensor_separate_mode)
 			|| (GK.five_second_tick && sensor_separate_mode)
 		   )
 			draw_sensor_decals(cpu);
 
-		if (cp->need_redraw)
-			refresh_cpu_chart(cpu);
-		krell->full_scale = (gint) (total - krell->previous_total);
-		krell->full_scale *= krell->full_scale_expand;
-		if (krell->previous_total > 0)
+		krell = KRELL(cp->panel);
+		gkrellm_set_krell_full_scale(krell,
+					(gint) (total - cpu->previous_total), 10);
+		if (cpu->previous_total > 0)
 			{
-			gkrellm_update_krell(p, krell, cpu->sys + cpu->user);
-			gkrellm_draw_layers(p);
+			krell_value = cpu->sys + cpu->user;
+			if (!omit_nice_mode && !gkrellm_get_chartdata_hide(cpu->nice_cd))
+				krell_value += cpu->nice;
+			gkrellm_update_krell(p, krell, krell_value);
+			gkrellm_draw_panel_layers(p);
 			}
-		krell->previous_total = total;
+		cpu->previous_total = total;
 		}
 	}
 
@@ -590,44 +787,44 @@ cpu_expose_event(GtkWidget *widget, GdkEventExpose *ev)
 
 
 static gint
-cb_cpu_extra(GtkWidget *widget, GdkEventButton *event)
+cb_cpu_extra(GtkWidget *widget, GdkEventButton *ev)
 	{
 	GList	*list;
 	CpuMon	*cpu;
 	Chart	*cp;
 
-	if (event->button == 1)
-		for (list = cpu_mon_list; list; list = list->next)
+	for (list = cpu_mon_list; list; list = list->next)
+		{
+		cpu = (CpuMon *) list->data;
+		cp = cpu->chart;
+		if (widget != cpu->chart->drawing_area)
+			continue;
+		if (ev->button == 1 && ev->type == GDK_BUTTON_PRESS)
 			{
-			cpu = (CpuMon *) list->data;
-			cp = cpu->chart;
-			if (widget == cpu->chart->drawing_area)
-				{
-				cpu->extra_info = 1 - cpu->extra_info;
-				gkrellm_config_modified();
-				}
+			cpu->extra_info = !cpu->extra_info;
 			refresh_cpu_chart(cpu);
+			gkrellm_config_modified();
 			}
+		else if (   ev->button == 3
+				 || (ev->button == 1 && ev->type == GDK_2BUTTON_PRESS)
+				)
+			gkrellm_chartconfig_window_create(cpu->chart);
+		break;
+		}
 	return TRUE;
 	}
 
 static void
-setup_cpu_scaling(Chart *cp)
+setup_cpu_scaling(ChartConfig *cf)
 	{
-	Krell	*krell;
 	gint	grids;
 
-	krell = KRELL(cp->panel);
-	grids = UC.fixed_scale ? UC.fixed_scale : FULL_SCALE_GRIDS;
+	grids = gkrellm_get_chartconfig_fixed_grids(cf);
+	if (!grids)
+		grids = FULL_SCALE_GRIDS;
 
-	cp->scale_min = CPU_TICKS_PER_SECOND / grids;
-	cp->scale_max = 0;	/* Force a chart rescale */
-
-	/* krell->full_scale can be small (as low as 10), so map it to a bigger
-	|  number so ema calc will be smooth.   This is accounted for in
-	|  update_krell()
-	*/
-	krell->full_scale_expand = 10;
+	gkrellm_set_chartconfig_grid_resolution(cf,
+				CPU_TICKS_PER_SECOND / grids);
 	}
 
 static gboolean
@@ -635,19 +832,64 @@ enable_cpu_visibility(CpuMon *cpu)
 	{
 	gint	enabled = cpu_enabled;
 
-	if (smp_cpus > 0)
+	if (n_smp_cpus > 0)
 		{
-		if (   (cpu->is_composite && smp_mode == 0)		/* real cpu's only */
-			|| (! cpu->is_composite && smp_mode == 1)	/* Compos cpu only */
+		if (   (cpu->is_composite && smp_mode == SMP_REAL_MODE)
+			|| (! cpu->is_composite && smp_mode == SMP_COMPOSITE_MODE)
 		   )
 			enabled = FALSE;
 		}
-	return
-		gkrellm_enable_visibility(enabled, &cpu->enabled, cpu->vbox,
-				cpu->height);
+	return gkrellm_chart_enable_visibility(cpu->chart, enabled, &cpu->enabled);
 	}
 
 
+static void
+cb_cpu_temp_alert_trigger(Alert *alert, CpuMon *cpu)
+	{
+	AlertDecal	*ad;
+	Decal		*d;
+
+	if (alert && cpu && cpu->chart)
+		{
+		ad = &alert->ad;
+		d = cpu->sensor_decal;
+		if (d)
+			{
+			ad->x = d->x - 1;
+			ad->y = d->y - 1;
+			ad->w = d->w + 2;
+			ad->h = d->h + 2;
+			gkrellm_render_default_alert_decal(alert);
+			}
+		alert->panel = cpu->chart->panel;
+		}
+	}
+
+static void
+cb_cpu_fan_alert_trigger(Alert *alert, CpuMon *cpu)
+	{
+	AlertDecal	*ad;
+	Decal		*d;
+
+	if (alert && cpu && cpu->chart)
+		{
+		ad = &alert->ad;
+		if (sensor_separate_mode)
+			d = cpu->fan_decal;
+		else
+			d = cpu->sensor_decal;
+		if (d)
+			{
+			ad->x = d->x - 1;
+			ad->y = d->y - 1;
+			ad->w = d->w + 2;
+			ad->h = d->h + 2;
+			gkrellm_render_default_alert_decal(alert);
+			}
+		alert->panel = cpu->chart->panel;
+		}
+	}
+
   /* How to decide when to make the sensor_decal and fan_decal visible.
   |  The sensor_decal can show temp values, fan values, or alternating
   |  temp/fan values.  The fan_decal only shows fan values when non
@@ -672,6 +914,7 @@ adjust_sensors_display(CpuMon *cpu, gint force)
 	{
 	Panel	*p;
 	Decal	*ds, *df;
+	Alert	*alert;
 	gint	position	= 0;
 
 	ds = cpu->sensor_decal;
@@ -683,9 +926,23 @@ adjust_sensors_display(CpuMon *cpu, gint force)
 	*/
 	p = cpu->chart->panel;
 	mapped_sensors = 0;
-	if (get_mapped_sensor(cpu->chart->name, SENSOR_TEMPERATURE) || GK.demo)
+	if (!GK.demo)
+		{
+		gkrellm_mapped_sensor_alert_connect(cpu->name,
+				SENSOR_TEMPERATURE, cb_cpu_temp_alert_trigger, cpu);
+		gkrellm_mapped_sensor_alert_connect(cpu->name,
+				SENSOR_FAN, cb_cpu_fan_alert_trigger, cpu);
+		}
+
+	/* If a fan alert is triggered, turn it off in case fan decal being used
+	|  is changed.  The alert will just retrigger at next fan update.
+	*/
+	alert = gkrellm_mapped_sensor_alert(cpu->name, SENSOR_FAN);
+	gkrellm_reset_alert(alert);
+
+	if (get_mapped_sensor(cpu->name, SENSOR_TEMPERATURE) || GK.demo)
 		mapped_sensors |= SENSOR_TEMPERATURE;
-	if (get_mapped_sensor(cpu->chart->name, SENSOR_FAN) || GK.demo)
+	if (get_mapped_sensor(cpu->name, SENSOR_FAN) || GK.demo)
 		mapped_sensors |= SENSOR_FAN;
 
 	if (mapped_sensors & (SENSOR_TEMPERATURE | SENSOR_FAN))
@@ -720,25 +977,89 @@ adjust_sensors_display(CpuMon *cpu, gint force)
 			p->label->position = position;	/* original label was visible.   */
 		gkrellm_draw_panel_label(p, gkrellm_bg_panel_image(style_id));
 		draw_sensor_decals(cpu);
-		gkrellm_draw_layers(p);
+		gkrellm_draw_panel_layers(p);
 		return TRUE;
 		}
 	return FALSE;
 	}
 
+static gint
+cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
+	{
+	if (ev->button == 3)
+		gkrellm_open_config_window(mon_cpu);
+	return TRUE;
+	}
+
+static GdkImlibImage	*nice_data_image,
+						*nice_data_grid_image;
+static GdkPixmap		*nice_data_pixmap,
+						*nice_data_grid_pixmap;
+static GdkColor			nice_color,
+						nice_grid_color;
+static gchar			*nice_color_string,
+						*nice_grid_color_string;
+
+static void
+load_nice_data_images(void)
+	{
+	g_free(nice_color_string);
+	g_free(nice_grid_color_string);
+	nice_color_string = gkrellm_get_gkrellmrc_string("cpu_nice_color");
+	nice_grid_color_string =
+				gkrellm_get_gkrellmrc_string("cpu_nice_grid_color");
+	map_color_string(nice_color_string, &nice_color);
+	map_color_string(nice_grid_color_string, &nice_grid_color);
+		
+	if (nice_data_image)
+		gdk_imlib_kill_image(nice_data_image);
+	if (nice_data_grid_image)
+		gdk_imlib_kill_image(nice_data_grid_image);
+	nice_data_image = nice_data_grid_image = NULL;
+
+	gkrellm_free_pixmap(&nice_data_pixmap);
+	gkrellm_free_pixmap(&nice_data_grid_pixmap);
+
+	gkrellm_load_image("nice", NULL, &nice_data_image, CPU_STYLE_NAME);
+	gkrellm_load_image("nice_grid", NULL, &nice_data_grid_image,
+					CPU_STYLE_NAME);
+	}
+
+static void
+render_nice_data_pixmaps(void)
+	{
+	GList		*list;
+	CpuMon		*cpu;
+	gint		h_max;
+
+	gkrellm_render_data_grid_pixmap(nice_data_grid_image,
+				&nice_data_grid_pixmap, &nice_grid_color);
+
+	h_max = 2;
+	for (list = cpu_mon_list; list; list = list->next)
+		{
+		cpu = (CpuMon *) list->data;
+		if (cpu->chart && (cpu->chart->h > h_max))
+			h_max = cpu->chart->h;
+		}
+	gkrellm_render_data_pixmap(nice_data_image,
+				&nice_data_pixmap, &nice_color, h_max);
+	}
 
   /* I want cpu labels in upper case.
   */
 static gchar	mapname[5] ;
+
 static void
 create_cpu(GtkWidget *vbox, gint first_create)
 	{
-	CpuMon	*cpu;
-	Chart	*cp;
-	Panel	*p;
-	Style	*style;
-	GList	*list;
+	CpuMon		*cpu;
+	Chart		*cp;
+	Panel		*p;
+	Style		*style;
+	GList		*list;
 
+	load_nice_data_images();
 	ascent = 0;
 	for (list = cpu_mon_list; list; list = list->next)
 		{
@@ -748,55 +1069,61 @@ create_cpu(GtkWidget *vbox, gint first_create)
 		*/
 		if (first_create)
 			{
+			/* don't really need the cpu->vbox unless I start destroying...
+			*/
 			cpu->vbox = gtk_vbox_new(FALSE, 0);
 			gtk_container_add(GTK_CONTAINER(vbox), cpu->vbox);
 			gtk_widget_show(cpu->vbox);
 			cpu->chart = gkrellm_chart_new0();
 			cpu->chart->panel = gkrellm_panel_new0();
-			cpu->chart->name = g_strdup(cpu->name);
 			cpu->enabled = TRUE;
 			}
-		else
-			{
-			gkrellm_destroy_decal_list(cpu->chart->panel);
-			gkrellm_destroy_krell_list(cpu->chart->panel);
-			}
 		cp = cpu->chart;
 		p = cp->panel;
 
 		style = gkrellm_panel_style(style_id);
 		gkrellm_create_krell(p, gkrellm_krell_panel_image(style_id), style);
 
-		setup_cpu_scaling(cp);
+		gkrellm_chart_create(cpu->vbox, mon_cpu, cp, &cpu->chart_config);
+		gkrellm_set_draw_chart_function(cp, refresh_cpu_chart, cpu);
+		cpu->sys_cd = gkrellm_add_default_chartdata(cp, _("sys time"));
+		cpu->user_cd = gkrellm_add_default_chartdata(cp, _("user time"));
+		if (   (nice_data_image && nice_data_grid_image)
+			|| (nice_color_string && nice_grid_color_string)
+		   )
+			{
+			render_nice_data_pixmaps();
+			cpu->nice_cd = gkrellm_add_chartdata(cp, &nice_data_pixmap,
+						nice_data_grid_pixmap, _("nice time"));
+			}
+		else
+			cpu->nice_cd = gkrellm_add_default_chartdata(cp, _("nice time"));
+		gkrellm_set_chartdata_flags(cpu->nice_cd, CHARTDATA_ALLOW_HIDE);
 
-		if (GK.debug_level & DEBUG_MISC)
-			printf(_("creating cpu <%s>, scale_min=%d krell->full_scale=%d\n"),
-					cp->name, cp->scale_min, KRELL(p)->full_scale);
+		/* Since there is a constant Max value of the chart (CPU_TICKS/SEC)
+		|  I control the grid resolution when fixed grids change.
+		|  So don't call gkrellm_grid_resolution_adjustment() so there won't
+		|  be a grid resolution part in the chart config.  Also, make sure
+		|  auto grid res is off.
+		*/
+		gkrellm_set_chartconfig_auto_grid_resolution(cp->config, FALSE);
+		gkrellm_chartconfig_fixed_grids_connect(cp->config,
+									setup_cpu_scaling, NULL);
+		setup_cpu_scaling(cp->config);
 
-		cp->h = UC.chart_height[MON_CPU];
-		gkrellm_create_chart(cpu->vbox, cp, style_id);
-
-		sprintf( mapname, _("CPU%c"), (cp->name)[3] );
-		p->textstyle = gkrellm_panel_textstyle(style_id);
-
-		if (p->label->string)
-			g_free(p->label->string);
-
-		if (! (!strcmp(cp->name, "cpu") && cpu->is_composite))
+		if (! (!strcmp(cpu->name, "cpu") && cpu->is_composite))
 			sensor_create_decals(p, style_id,
 					&cpu->sensor_decal, &cpu->fan_decal);
 
-		gkrellm_configure_panel(p, g_strdup(mapname), style);
-		gkrellm_create_panel(cpu->vbox, p, gkrellm_bg_panel_image(style_id));
-		cpu->height = cp->h + p->h;
-		if (cpu->enabled)
-			gkrellm_monitor_height_adjust(cpu->height);
+		sprintf( mapname, _("CPU%c"), (cpu->name)[3] );
+		gkrellm_panel_configure(p, mapname, style);
+		gkrellm_panel_create(cpu->vbox, mon_cpu, p);
 
 		cpu->save_label_position = p->label->position;
 		if (cpu->sensor_decal)
 			adjust_sensors_display(cpu, TRUE);
 
-		gkrellm_alloc_chart_data(cp);
+		gkrellm_alloc_chartdata(cp);
 		enable_cpu_visibility(cpu);
 
 		if (first_create)
@@ -808,6 +1135,8 @@ create_cpu(GtkWidget *vbox, gint first_create)
 
 			gtk_signal_connect(GTK_OBJECT(cp->drawing_area),
 					"button_press_event", (GtkSignalFunc) cb_cpu_extra, NULL);
+			gtk_signal_connect(GTK_OBJECT(p->drawing_area),
+					"button_press_event", (GtkSignalFunc) cb_panel_press,NULL);
 			}
 		else
 			refresh_cpu_chart(cpu);
@@ -816,13 +1145,13 @@ create_cpu(GtkWidget *vbox, gint first_create)
 	}
 
 
-
 /* ------------------------------------------------------------------ */
 #define	CPU_CONFIG_KEYWORD	"cpu"
 
-static GtkWidget	*cpu_enabled_button;
+static GtkWidget	*cpu_enabled_button,
+					*text_format_combo;
 static GtkWidget	*smp_button[3];
-static GtkWidget	*add_nice_button;
+static GtkWidget	*omit_nice_button;
 static GtkWidget	*sensor_separate_button;
 
 
@@ -843,13 +1172,15 @@ save_cpu_config(FILE *f)
 						cpu->name, cpu->launch.tooltip_comment);
 		fprintf(f, "%s extra_info %s %d\n", CPU_CONFIG_KEYWORD,
 					cpu->name, cpu->extra_info);
-
+		gkrellm_save_chartconfig(f, cpu->chart_config,
+					CPU_CONFIG_KEYWORD, cpu->name);
 		}
 	fprintf(f, "%s enable %d\n", CPU_CONFIG_KEYWORD, cpu_enabled);
 	fprintf(f, "%s smp_mode %d\n", CPU_CONFIG_KEYWORD, smp_mode);
-	fprintf(f, "%s nice_mode %d\n", CPU_CONFIG_KEYWORD, add_nice_mode);
+	fprintf(f, "%s omit_nice_mode %d\n", CPU_CONFIG_KEYWORD, omit_nice_mode);
 	fprintf(f, "%s sensor_mode %d\n", CPU_CONFIG_KEYWORD,
 			sensor_separate_mode);
+	fprintf(f, "%s text_format %s\n", CPU_CONFIG_KEYWORD, text_format);
 	}
 
 static void
@@ -864,15 +1195,27 @@ load_cpu_config(gchar *arg)
 	n = sscanf(arg, "%31s %[^\n]", config, item);
 	if (n == 2)
 		{
-		if (strcmp(config, "enable") == 0)
+		if (!strcmp(config, "enable"))
 			sscanf(item, "%d", &cpu_enabled);
-		else if (strcmp(config, "smp_mode") == 0)
+		else if (!strcmp(config, "smp_mode"))
 			sscanf(item, "%d\n", &smp_mode);
-		else if (strcmp(config, "nice_mode") == 0)
-			sscanf(item, "%d\n", &add_nice_mode);
-		else if (strcmp(config, "sensor_mode") == 0)
+		else if (!strcmp(config, "omit_nice_mode"))
+			sscanf(item, "%d\n", &omit_nice_mode);
+		else if (!strcmp(config, "sensor_mode"))
 			sscanf(item, "%d\n", &sensor_separate_mode);
-		else if (strcmp(config, "extra_info") == 0)
+		else if (!strcmp(config, "text_format"))
+			gkrellm_dup_string(&text_format, item);
+		else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
+			{
+			sscanf(item, "%31s %[^\n]", cpu_name, command);
+			for (list = cpu_mon_list; list; list = list->next)
+				{
+				cpu = (CpuMon *) list->data;
+				if (strcmp(cpu->name, cpu_name) == 0)
+					gkrellm_load_chartconfig(&cpu->chart_config, command, 3);
+				}
+			}
+		else if (!strcmp(config, "extra_info"))
 			{
 			sscanf(item, "%31s %[^\n]", cpu_name, command);
 			for (list = cpu_mon_list; list; list = list->next)
@@ -882,7 +1225,7 @@ load_cpu_config(gchar *arg)
 					sscanf(command, "%d\n", &cpu->extra_info);
 				}
 			}
-		else if (strcmp(config, "launch") == 0)
+		else if (!strcmp(config, "launch"))
 			{
 			sscanf(item, "%31s %[^\n]", cpu_name, command);
 			for (list = cpu_mon_list; list; list = list->next)
@@ -892,7 +1235,7 @@ load_cpu_config(gchar *arg)
 					cpu->launch.command = g_strdup(command);
 				}
 			}
-		else if (strcmp(config, "tooltip_comment") == 0)
+		else if (!strcmp(config, "tooltip_comment"))
 			{
 			sscanf(item, "%31s %[^\n]", cpu_name, command);
 			for (list = cpu_mon_list; list; list = list->next)
@@ -907,18 +1250,22 @@ load_cpu_config(gchar *arg)
 
 
 static void
-apply_cpu_config()
+apply_cpu_config(void)
 	{
 	GList	*list;
 	CpuMon	*cpu;
+	gchar	*s;
 	gint	i;
 
+	s = gkrellm_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
+	gkrellm_dup_string(&text_format, s);
+
 	cpu_enabled = GTK_TOGGLE_BUTTON(cpu_enabled_button)->active;
-	if (smp_cpus > 0)
+	if (n_smp_cpus > 0)
 		for (i = 0; i < 3; ++i)
 			if (GTK_TOGGLE_BUTTON(smp_button[i])->active)
 				smp_mode = i;
-	add_nice_mode = GTK_TOGGLE_BUTTON(add_nice_button)->active;
+	omit_nice_mode = GTK_TOGGLE_BUTTON(omit_nice_button)->active;
 	if (sensor_separate_button)
 		sensor_separate_mode =
 				GTK_TOGGLE_BUTTON(sensor_separate_button)->active;
@@ -927,8 +1274,7 @@ apply_cpu_config()
 		{
 		cpu = (CpuMon *) list->data;
 		if (enable_cpu_visibility(cpu) && cpu->enabled)
-			gkrellm_reset_chart(cpu->chart);
-		setup_cpu_scaling(cpu->chart);
+			gkrellm_reset_and_draw_chart(cpu->chart);
 		gkrellm_apply_launcher(&cpu->launch_entry, &cpu->tooltip_entry,
 					cpu->chart->panel, &cpu->launch, gkrellm_launch_button_cb);
 		if (adjust_sensors_display(cpu, FALSE) && cpu->launch.button)
@@ -941,11 +1287,27 @@ apply_cpu_config()
 		}
 	}
 
+
+#define	DEFAULT_TEXT_FORMAT	"$T"
+
+static gchar	*cpu_info_text[] =
+{
+N_("<b>Chart Labels\n"),
+N_("Substitution variables for the format string for chart labels:\n"),
+N_("\t$L    the CPU label\n"),
+N_("\t$T    total sys + user {+ nice} time percent usage\n"),
+N_("\t$s    sys time percent usage\n"),
+N_("\t$u    user time percent usage\n"),
+N_("\t$n    nice time percent usage\n"),
+};
+
 static void
 create_cpu_tab(GtkWidget *tab_vbox)
 	{
+	GtkWidget	*tabs;
 	GtkWidget	*button;
-	GtkWidget	*hbox;
+	GtkWidget	*hbox, *vbox, *vbox1;
+	GtkWidget	*text;
 	GtkWidget	*table;
 	GSList		*group;
 	GList		*list;
@@ -953,14 +1315,27 @@ create_cpu_tab(GtkWidget *tab_vbox)
 	gchar		buf[128];
 	gint		i;
 
-	gkrellm_check_button(tab_vbox, &cpu_enabled_button, cpu_enabled,
+
+	tabs = gtk_notebook_new();
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
+	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);
+
+/* -- Options tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Options"));
+	gkrellm_check_button(vbox, &cpu_enabled_button, cpu_enabled,
 			FALSE, 10, _("Enable CPU"));
-	if (smp_cpus > 0)
+	gkrellm_check_button(vbox, &omit_nice_button, omit_nice_mode, FALSE, 0,
+		_("Exclude nice CPU time from krell even if nice is shown on chart"));
+	if (gkrellm_sensors_available())
+		gkrellm_check_button(vbox, &sensor_separate_button,
+					sensor_separate_mode, FALSE, 0,
+		_("Draw fan and temperature values separately (not alternating)."));
+	if (n_smp_cpus > 0)
 		{
-		insert_expanded_filler(tab_vbox);
+		vbox1 = gkrellm_framed_vbox(vbox, _("SMP Charts Select"), 4, FALSE,
+				0, 2);
 		hbox = gtk_hbox_new (FALSE, 3);
-		gtk_container_set_border_width(GTK_CONTAINER(hbox), 3);
-		gtk_container_add(GTK_CONTAINER(tab_vbox), hbox);
+		gtk_container_add(GTK_CONTAINER(vbox1), hbox);
 
 		button = gtk_radio_button_new_with_label(NULL, _("Real CPUs."));
 		gtk_box_pack_start(GTK_BOX (hbox), button, TRUE, TRUE, 0);
@@ -979,16 +1354,31 @@ create_cpu_tab(GtkWidget *tab_vbox)
 		button = smp_button[smp_mode];
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 		}
-	insert_expanded_filler(tab_vbox);
-	gkrellm_check_button(tab_vbox, &add_nice_button, add_nice_mode, FALSE, 0,
-		_("Add nice CPU time to user time."));
-	if (gkrellm_sensors_available())
-		gkrellm_check_button(tab_vbox, &sensor_separate_button,
-					sensor_separate_mode, FALSE, 0,
-		_("Draw fan and temperature values separately (not alternating)."));
-	insert_expanded_filler(tab_vbox);
 
-	table = gkrellm_launcher_table_new(tab_vbox, n_cpus);
+/* -- Setup tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Setup"));
+	vbox1 = gkrellm_framed_vbox(vbox, _("Format String for Chart Labels"),
+			 4, FALSE, 0, 2);
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+	text_format_combo = gtk_combo_new();
+	gtk_widget_set_usize (GTK_WIDGET(text_format_combo), 350, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
+	list = NULL;
+	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
+	list = g_list_append(list, _("\\fu \\.$u\\n\\fs \\.$s"));
+	list = g_list_append(list, _("\\ww\\D2\\f\\au\\.$u\\D1\\f\\as\\.$s"));
+	list = g_list_append(list, _("\\ww\\D3\\f\\au\\.$u\\D0\\f\\as\\.$s"));
+	list = g_list_append(list,
+		"\\ww\\C\\f$L\\D5\\f\\an\\.$n\\D2\\f\\au\\.$u\\D1\\f\\as\\.$s");
+	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
+	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
+			text_format);
+	
+	vbox1 = gkrellm_framed_vbox_end(vbox, _("Launch Commands"), 4, TRUE, 0,2);
+	vbox1 = gkrellm_scrolled_vbox(vbox1, NULL,
+			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	table = gkrellm_launcher_table_new(vbox1, n_cpus);
 	for (i = 0, list = cpu_mon_list; list; list = list->next, ++i)
 		{
 		cpu = (CpuMon *) list->data;
@@ -996,6 +1386,14 @@ create_cpu_tab(GtkWidget *tab_vbox)
 		gkrellm_config_launcher(table, i,  &cpu->launch_entry,
 				&cpu->tooltip_entry, buf, &cpu->launch);
 		}
+
+/* --Info tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Info"));
+	text = gkrellm_scrolled_text(vbox, NULL,
+				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	for (i = 0; i < sizeof(cpu_info_text)/sizeof(gchar *); ++i)
+		gkrellm_add_info_text_string(text, _(cpu_info_text[i]));
+
 	}
 
 static Monitor	monitor_cpu =
@@ -1024,10 +1422,20 @@ static Monitor	monitor_cpu =
 Monitor *
 init_cpu_monitor(void)
 	{
+	GList	*list;
+
 	monitor_cpu.name = _(monitor_cpu.name);
 	style_id = gkrellm_add_chart_style(&monitor_cpu, CPU_STYLE_NAME);
+	text_format = g_strdup(DEFAULT_TEXT_FORMAT);
+
+	mon_cpu = &monitor_cpu;
+
 	if (setup_cpu_interface())
+		{
+		for (list = cpu_mon_list; list; list = list->next)
+			((CpuMon *)(list->data))->extra_info = TRUE;
 		return &monitor_cpu;
+		}
 	return NULL;
 	}
 
