@@ -18,9 +18,10 @@
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* 10/12/2000	NetBSD code contributed by Anthony Mallet
-|			<metall@ficus.yi.org>
-| 2/25/2000	FreeBSD code contributed by Hajimu UMEMOTO ume@mahoroba.org
+/*
+|  4/22/2001  Solaris code contributed by Daisuke Yabuki <dxy@acm.org>
+| 10/12/2000  NetBSD code contributed by Anthony Mallet <metall@ficus.yi.org>
+|  2/25/2000  FreeBSD code contributed by Hajimu UMEMOTO ume@mahoroba.org
 */
 
 
@@ -28,7 +29,9 @@
 #include "gkrellm_private_proto.h"
 #include "cpu_disk.h"
 
-#define	DEFAULT_RESOLUTION	200
+#define	MIN_GRID_RES		10
+#define	MAX_GRID_RES		500000
+#define	DEFAULT_GRID_RES	2000
 
 void	(*read_system_disk_info)();
 
@@ -132,7 +135,6 @@ register_disks(gboolean re_checking)
 		disk = g_new0(DiskMon, 1);
 		disk_mon_list = g_list_append(disk_mon_list, disk);
 		disk->name = g_strdup_printf("%s%c", _("Disk"), 'A' + i);
-		disk->resolution = DEFAULT_RESOLUTION;
 		}
 	using_DiskN_names = TRUE;
 	}
@@ -233,7 +235,6 @@ register_disks(gboolean re_checking)
 		disk = g_new0(DiskMon, 1);
 		disk->name = g_strdup_printf("%s%d", dev->device_name,
 					     dev->unit_number);
-		disk->resolution = DEFAULT_RESOLUTION;
 		disk->major = dev->device_number;
 		disk->minor = 0;
 		disk->order = dev->device_type + 1;	/* Skip the composite disk */
@@ -321,7 +322,6 @@ add_disk_device(gint major, gint i_disk)
 			{
 			disk = g_new0(DiskMon, 1);
 			disk->name = g_strdup(buf);
-			disk->resolution = DEFAULT_RESOLUTION;
 			disk->major = major;
 			disk->minor = i_disk;
 			disk->order = i + 1;	/* Skip the composite disk */
@@ -373,7 +373,6 @@ register_disks(gboolean re_checking)
 				disk = g_new0(DiskMon, 1);
 				disk_mon_list = g_list_append(disk_mon_list, disk);
 				disk->name = g_strdup_printf("%s%c", _("Disk"), c);
-				disk->resolution = DEFAULT_RESOLUTION;
 				++c;
 				++n_disks;
 				}
@@ -494,8 +493,6 @@ register_disks(gboolean re_checking)
       else
 	 disk->name = strdup(buf);
 
-      disk->resolution = DEFAULT_RESOLUTION;
-
       p = d.dk_link.tqe_next;
    }
 
@@ -504,6 +501,389 @@ register_disks(gboolean re_checking)
 
 #endif /* __NetBSD__ || __OpenBSD__ */
 
+/* ----- Solaris --------------------------------------------------------- */
+
+#if defined(__solaris__)
+
+#include <sys/types.h>
+#include <sys/sysinfo.h>
+#include <unistd.h>
+#include <kstat.h>
+#include <libdevinfo.h>
+#include <errno.h>
+#include <sys/dkio.h>
+
+#define UNIT_SHIFT 3
+
+#define NAME2MAJOR 0
+#define MAJOR2NAME 1
+
+typedef struct {
+    gint major;
+    gchar name[32];
+} name_to_major_t;
+
+static gint check_media_type(kstat_t *);
+static gint isharddisk(kstat_t *);
+void solaris_list_harddisks(void);                   /* called from main.c */
+
+static gint lookup_name_to_major(name_to_major_t *, int);
+static gint get_major(gchar *);
+static gint get_devname(gint, gchar *);
+static gint get_minor(gint);
+static gint get_instance(gint); 
+
+typedef struct {
+    char name[8];
+} probed_harddisk;
+
+GList *hard_disk_list;
+
+static DiskMon *
+add_disk_device(gint major, gint minor) {
+    DiskMon *disk;
+    gchar *devid = NULL;
+    gchar devname[32];
+    gint instance;
+
+    if ((disk = lookup_disk_by_device(major, minor)) == NULL) {
+        get_devname(major, devname);
+        instance = get_instance(minor);
+        if (devname && instance)
+            devid = g_strdup_printf("%s%d", devname, instance);
+
+        disk = g_new0(DiskMon, 1);
+        disk->name = g_strdup(devid);
+        disk->major = major;
+        disk->minor = minor;
+        disk_mon_list = g_list_append(disk_mon_list, disk);
+        n_disks++;
+
+        g_free(devid);
+    }  
+    return disk;
+}
+
+static void
+read_solaris_disk() 
+{
+    DiskMon *disk;
+    GList *list;
+
+    extern kstat_ctl_t *kc;
+    kstat_t *ksp;
+    kstat_io_t kios;
+
+#if 1
+    gint block_size = 512;
+#else
+    gint block_size = 8192; /* default ufs logical blocksize for sun4u arch */
+#endif
+    guint blocks_read, blocks_written;
+
+    if (!GK.second_tick)
+        return;
+
+    composite_disk->rblk = composite_disk->wblk = 0;
+
+    if (kstat_chain_update(kc) == -1) {
+        perror("kstat_chain_update");
+        return;
+    }
+    for (ksp = kc->kc_chain; ksp; ksp = ksp->ks_next) {
+        for (list = disk_mon_list; list; list = list->next) {
+            disk = (DiskMon *) list->data;
+
+            if(strcmp(disk->name, ksp->ks_name))
+                continue;
+
+            blocks_read = blocks_written = 0;
+
+            memset((void *)&kios, 0, sizeof(kstat_io_t));
+            kstat_read(kc, ksp, &kios);
+
+            if (kios.nread)   
+                blocks_read = kios.nread / block_size;
+            if (kios.nwritten)
+                blocks_written = kios.nwritten / block_size;
+
+            disk->rblk = blocks_read;
+            disk->wblk = blocks_written;
+
+            composite_disk->rblk += blocks_read;
+            composite_disk->wblk += blocks_written;
+        }
+    }
+}
+
+static void
+register_disks(gboolean re_checking)
+{
+    gint i;
+    DiskMon *disk;
+    GList *list;
+    gint major, minor;
+
+    extern kstat_ctl_t *kc;
+    kstat_t *ksp;
+
+    probed_harddisk *drive;
+
+    if (re_checking)
+        return; 
+
+    n_disks = 0;
+
+    /* Solaris port would not use 'DiskN' type of disk naming. 
+     * Instead, use sd0, sd1, ... just like you would see on iostat,
+     * in the hope that this could be more scalable.
+     */
+    using_DiskN_names = FALSE; 
+
+    if (kstat_chain_update(kc) == -1) {
+        perror("kstat_chain_update");
+        return;
+    }
+    for (ksp = kc->kc_chain, i = 0; ksp; ksp = ksp->ks_next) {
+        for (list = hard_disk_list; list; list = list->next) {
+            drive = (probed_harddisk *)list->data;
+
+            if(strcmp(drive->name, ksp->ks_name)) 
+                continue;
+            disk = g_new0(DiskMon, 1);
+            if (using_DiskN_names) 
+                disk->name = g_strdup_printf("%s%c", _("Disk"), 'A' + i);  
+	    else 
+                disk->name = g_strdup(ksp->ks_name);
+
+            major = get_major(ksp->ks_module);
+            minor = get_minor(ksp->ks_instance);
+
+            disk->major = major; 
+            disk->minor = minor;
+            disk_mon_list = g_list_append(disk_mon_list, disk);
+            n_disks++;
+            i++;
+        }
+    } 
+}
+
+static gint
+lookup_name_to_major(name_to_major_t *name_to_major, gint type) {
+    FILE *fp;
+    char line[80];
+    char *name, *maj;
+    gint name2major, major2name;
+    gint majnum;
+
+    name2major = major2name = 0;
+    switch (type) {
+        case NAME2MAJOR:
+            name2major = 1;
+            break;
+        case MAJOR2NAME:
+            major2name = 1;
+            break;
+        default:
+            break;
+    }
+
+    if ((fp = fopen("/etc/name_to_major", "r")) == NULL) {
+        perror("fopen");
+        return -1;
+    }
+
+    while (fgets(line, sizeof(line), fp) != NULL) {
+        name = strtok(line, " \t");
+        if (name == NULL)
+            continue;
+
+        maj = strtok(NULL, "\n");
+        if (maj == NULL)
+            continue;
+        majnum = (gint) atol(maj); 
+
+        if (name2major) {
+            if (strcmp(name_to_major->name, name) == 0) {
+                name_to_major->major = majnum;
+                fclose(fp);
+                return 0;
+            }
+        } else if (major2name) {
+            if (name_to_major->major == majnum) {
+                strcpy(name_to_major->name, name);
+                fclose(fp);
+                return 0; 
+            }
+        }
+    }
+    fclose(fp);
+    return -1;
+}
+
+static gint
+get_major(gchar *devname) {
+    /* xlation from device name to major (e.g. sd -> 32) */
+    name_to_major_t name_to_major;
+
+    strcpy(name_to_major.name, devname);
+    if (lookup_name_to_major(&name_to_major, NAME2MAJOR) < 0)
+        return -1;
+    return name_to_major.major;
+}
+
+static gint
+get_devname(gint major, gchar *devname) {
+    /* xlation from major to device name (e.g. 118 -> ssd) */
+    name_to_major_t name_to_major;
+
+    name_to_major.major = major;
+    if (lookup_name_to_major(&name_to_major, MAJOR2NAME) < 0)
+        return -1;
+    strcpy(devname, name_to_major.name);
+    return 0;
+}
+
+static gint
+get_minor(gint instance) {
+    return instance << UNIT_SHIFT;
+}
+
+static gint
+get_instance(gint minor) {
+    return minor >> UNIT_SHIFT;
+}
+
+/* 
+ * An sd instance could be a cdrom or a harddrive. You can't simply tell,
+ * from contents of kstat, which type of device an sd device is 
+ * (well, maybe you could, but at least i can't.)
+ * It, however, doesn't make much sense to count cdrom read/write as 
+ * "Disk" activity. So I'd like to exclude removable media's from 
+ * monitoring target. In order to do this, I try to open a physical 
+ * device of a corresponding sd instance. If it's succeeded, I assume
+ * it's a hard drive. If I get ENXIO or EBUSY, I'll guess it's CDROM.
+ * If you come up with a better (simpler or safer) way to tell it's 
+ * a removable media or a hard drive, please drop me an e-mail at 
+ * Daisuke Yabuki <dxy@acm.org>. 
+ * I don't know any other driver which handle both hard drive and 
+ * removable media, by the way. I hope it wouldn't do any harm on
+ * other type of devices, i.e. ssd, or IDE drivers. 
+ */ 
+static gint
+check_media_type(kstat_t *ksp) {
+    gint fd;
+    char *phys_path, devices_path[256]; /* or OBP_MAXPATHLEN? */
+    di_node_t node;
+    static di_node_t root_node = NULL;
+#if 0
+    /* Not supported on Solaris 7 */
+    struct dk_minfo dk;
+#else
+    int dkRemovable;
+#endif
+
+    if (root_node == NULL) {
+        if ((root_node = di_init("/", DINFOCPYALL)) == DI_NODE_NIL) {
+                perror("di_init");
+                return -1;
+        }
+    }
+
+    node = di_drv_first_node(ksp->ks_module, root_node);
+    while (node != DI_NODE_NIL) {
+        if (di_instance(node) != ksp->ks_instance) {
+            node = di_drv_next_node(node);
+            continue;
+        }
+        if ((phys_path = di_devfs_path(node)) == NULL) {
+            perror("di_devfs_path");
+            return -1;
+        }
+        if (sprintf(devices_path, "/devices%s:c,raw", phys_path) <= 0) {
+            di_devfs_path_free(phys_path);
+            return -1;
+        }
+        if ((fd = open(devices_path, O_RDONLY)) == -1) {
+            if (errno == ENXIO || errno == EBUSY) {
+                close(fd);
+                di_devfs_path_free(phys_path);
+                return 0; /* guess it's removable media */
+            } else {
+#ifdef DEBUG
+                printf("opening %s\n", devices_path);
+                printf("unexpected errno: %d\n", errno);
+                printf("disabled auto-detection/exclusion of removable media\n");
+#endif
+                close(fd);
+                di_devfs_path_free(phys_path);
+                return -1; /* EACCESS (unless setgid sys) or suchlike */
+            }
+        }
+#if 0
+	/* Not supported on Solaris 7 */
+        if (ioctl(fd, DKIOCGMEDIAINFO, &dk) < 0)
+#else
+	if (ioctl(fd, DKIOCREMOVABLE, &dkRemovable) < 0)
+#endif
+	{
+            close(fd);
+            di_devfs_path_free(phys_path);
+            return -1;
+        }
+#if 0
+        if (dk.dki_media_type == DK_FIXED_DISK)
+#else
+        if (!dkRemovable)
+#endif
+	{
+	   close(fd);
+	   di_devfs_path_free(phys_path);
+	   return 1;
+        }
+    }
+    return -1; /* shouldn't be reached */
+} 
+
+static gint
+isharddisk(kstat_t *ksp) {
+    if (ksp->ks_type != KSTAT_TYPE_IO) 
+        return 0;
+    if (strncmp(ksp->ks_class, "disk", 4)) 
+        return 0; /* excluding nfs etc. */
+    if (!strcmp(ksp->ks_module, "fd"))
+        return 0; /* excluding fd */
+    if (check_media_type(ksp) == 0)
+        return 0; /* guess it's removable media (e.g. CD-ROM, CD-R/W etc) */
+    return 1;
+}
+
+/* 
+ * creating a preliminary list of drives, which should be a complete
+ * list of drives available on the system. the list is not supposed to
+ * contain nfs, fd, cdrom, cdrw etc.
+ */
+void
+solaris_list_harddisks(void) {
+    extern kstat_ctl_t *kc;
+    kstat_t *ksp;
+    probed_harddisk *drive;
+
+    if (kstat_chain_update(kc) == -1) {
+        perror("kstat_chain_update");
+        return;
+    }
+    
+    for (ksp = kc->kc_chain; ksp; ksp = ksp->ks_next) {
+        if(isharddisk(ksp)) {
+             drive = g_new0(probed_harddisk, 1);
+             hard_disk_list = g_list_append(hard_disk_list, drive);
+             strcpy(drive->name, ksp->ks_name);
+        }
+    }
+}
+
+#endif /* __solaris__ */
 
 /* ----- Other Systems -------------- */
 
@@ -525,7 +905,7 @@ register_disks(gint x)
 
 /* ----- Pick a system interface ----------------------------------------- */
 static gint
-setup_disk_interface()
+setup_disk_interface(void)
 	{
 	gint	available = FALSE;
 
@@ -537,7 +917,7 @@ setup_disk_interface()
 
 #if defined(__linux__)
 	register_disks(FALSE);
-	read_system_disk_info = read_proc_stat;		/* In stat_cpu.c */
+	read_system_disk_info = read_proc_stat;		/* In cpu.c */
 	available = TRUE;
 #endif
 
@@ -547,10 +927,17 @@ setup_disk_interface()
 	available = TRUE;
 #endif
 
+#if defined(__solaris__)
+	register_disks(FALSE);
+	read_system_disk_info = read_solaris_disk;      /* above */
+	available = TRUE; 
+#endif
+
 	return available;
 	}
 
 /* ======================================================================== */
+static Monitor	*mon_disk;
 
 DiskMon			*composite_disk;
 static gint		ascent;
@@ -587,32 +974,88 @@ lookup_disk_by_name(gchar *name)
 	}
 
 
-static void
-draw_disk_extra(Chart *cp, unsigned long l)
+static size_abbrev_table	disk_blocks_abbrev[]	=
 	{
-	TextStyle	*ts;
-	gchar		buf[32];
+	{ KB_SIZE(1),		1,				"%.0f" },
+	{ KB_SIZE(20),		KB_SIZE(1),		"%.1fK" },
+	{ MB_SIZE(1),		KB_SIZE(1),		"%.0fK" },
+	{ MB_SIZE(20),		MB_SIZE(1),		"%.1fM" }
+	};
 
-	ts = gkrellm_chart_textstyle(style_id);
-	if (ascent == 0)
-		ascent = gdk_char_height(ts->font, '8');
-	snprintf(buf, sizeof(buf), "%d", (gint) l);
-	gkrellm_draw_chart_label(cp, ts, 4, 4 + ascent, buf);
+
+static gchar    *text_format;
+
+static void
+format_chart_text(DiskMon *disk, gchar *buf, gint size)
+	{
+	Chart	*cp;
+	gchar	c, *s;
+	size_t	tbl_size;
+	gint	len, r_blocks, w_blocks, blocks;
+
+	--size;
+	*buf = '\0';
+	cp = disk->chart;
+	r_blocks = gkrellm_get_current_chartdata(disk->rb_cd);
+	w_blocks = gkrellm_get_current_chartdata(disk->wb_cd);
+	tbl_size = sizeof(disk_blocks_abbrev) / sizeof(size_abbrev_table);
+	for (s = text_format; *s != '\0' && size > 0; ++s)
+		{
+		len = 1;
+		if (*s == '$' && *(s + 1) != '\0')
+			{
+			blocks = -1;
+			if ((c = *(s + 1)) == 'T')
+				blocks = r_blocks + w_blocks;
+			else if (c == 'M')
+				blocks = gkrellm_get_chart_scalemax(cp);
+			else if (c == 'r')
+				blocks = r_blocks;
+			else if (c == 'w')
+				blocks = w_blocks;
+			else
+				{
+				*buf = *s;
+				if (size > 1)
+					{
+					*(buf + 1) = *(s + 1);
+					++len;
+					}
+				}
+			if (blocks >= 0)
+				len = format_size_abbrev(buf, size, (gfloat) blocks,
+						&disk_blocks_abbrev[0], tbl_size);
+			++s;
+			}
+		else
+			*buf = *s;
+		size -= len;
+		buf += len;
+		}
+	*buf = '\0';	
 	}
 
 static void
-refresh_disk_chart(DiskMon *disk)
+draw_disk_extra(DiskMon *disk)
 	{
-	Chart	*cp		= disk->chart;
+	gchar		buf[128];
 
-	gkrellm_draw_chart(cp);
-	if (disk->extra_info)
-		draw_disk_extra(cp, disk->cur);
-	cp->need_redraw = FALSE;
+	if (!disk->extra_info)
+		return;
+	format_chart_text(disk, buf, sizeof(buf));
+	gkrellm_draw_chart_text(disk->chart, style_id, buf);
 	}
 
 static void
-update_disk()
+draw_disk_chart(DiskMon *disk)
+	{
+	gkrellm_draw_chartdata(disk->chart);
+	draw_disk_extra(disk);
+	gkrellm_draw_chart_to_screen(disk->chart);
+	}
+
+static void
+update_disk(void)
 	{
 	GList		*list;
 	DiskMon		*disk;
@@ -629,17 +1072,12 @@ update_disk()
 			continue;
 		if (GK.second_tick)
 			{
-			if (cp->primed)
-				disk->cur = (disk->rblk - cp->prevIn)
-							+ (disk->wblk - cp->prevOut);
-			gkrellm_store_chart_data(cp, disk->wblk, disk->rblk, 0);
-			refresh_disk_chart(disk);
+			gkrellm_store_chartdata(cp, 0, disk->wblk, disk->rblk);
+			draw_disk_chart(disk);
 			}
-		if (cp->need_redraw)
-			refresh_disk_chart(disk);
 		gkrellm_update_krell(cp->panel, KRELL(cp->panel),
 										disk->wblk + disk->rblk);
-		gkrellm_draw_layers(cp->panel);
+		gkrellm_draw_panel_layers(cp->panel);
 		}
 	}
 
@@ -671,37 +1109,42 @@ disk_expose_event(GtkWidget *widget, GdkEventExpose *ev)
 	}
 
 static gint
-cb_disk_extra(GtkWidget *widget, GdkEventButton *event)
+cb_disk_extra(GtkWidget *widget, GdkEventButton *ev)
 	{
 	GList	*list;
 	DiskMon	*disk;
 
-	if (event->button == 1)
+	for (list = disk_mon_list; list; list = list->next)
 		{
-		for (list = disk_mon_list; list; list = list->next)
+		disk = (DiskMon *) list->data;
+		if (!disk->enabled || widget != disk->chart->drawing_area)
+			continue;
+		if (ev->button == 1 && ev->type == GDK_BUTTON_PRESS)
 			{
-			disk = (DiskMon *) list->data;
-			if (disk->enabled && widget == disk->chart->drawing_area)
-				{
-				disk->extra_info = 1 - disk->extra_info;
-				refresh_disk_chart(disk);
-				gkrellm_config_modified();
-				}
+			disk->extra_info = !disk->extra_info;
+			draw_disk_chart(disk);
+			gkrellm_config_modified();
 			}
+		else if (   ev->button == 3
+				 || (ev->button == 1 && ev->type == GDK_2BUTTON_PRESS)
+				)
+			gkrellm_chartconfig_window_create(disk->chart);
+		break;
 		}
 	return TRUE;
 	}
 
 static void
-setup_disk_scaling(DiskMon *disk)
+setup_disk_scaling(ChartConfig *cf, Chart *cp)
 	{
-	Chart	*cp		= disk->chart;
-	gint	grids;
+	gint	grids, res;
 
-	grids = UC.fixed_scale ? UC.fixed_scale : FULL_SCALE_GRIDS;
-	cp->scale_min = disk->resolution;
-	KRELL(cp->panel)->full_scale = cp->scale_min * grids / UC.update_HZ;
-	cp->scale_max = 0;
+	grids = gkrellm_get_chartconfig_fixed_grids(cf);
+	if (!grids)
+		grids = FULL_SCALE_GRIDS;
+	res = gkrellm_get_chartconfig_grid_resolution(cf);
+
+	KRELL(cp->panel)->full_scale = res * grids / gkrellm_update_HZ();
 	}
 
   /* Destroy everything in a DiskMon structure except for the vbox which
@@ -712,25 +1155,25 @@ setup_disk_scaling(DiskMon *disk)
 static void
 destroy_disk_monitor(DiskMon *disk)
 	{
-	Chart	*cp;
-
 	if (disk->launch.button)
 		gkrellm_destroy_button(disk->launch.button);
 	disk->launch.button = NULL;
+	disk->launch.tooltip = NULL;
 	gkrellm_dup_string(&disk->launch.command, "");
 	gkrellm_dup_string(&disk->launch.tooltip_comment, "");
-	cp = disk->chart;
-	g_free(cp->name);
-	gkrellm_monitor_height_adjust( - disk->height);
-	gkrellm_destroy_panel(cp->panel);
-	disk->launch.tooltip = NULL;
-	g_free(cp->panel);
-	gkrellm_destroy_chart(cp);
-	g_free(cp);
+	gkrellm_chart_destroy(disk->chart);
 	disk->chart = NULL;
 	disk->enabled = FALSE;
 	}
 
+static gint
+cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
+	{
+	if (ev->button == 3)
+		gkrellm_open_config_window(mon_disk);
+	return TRUE;
+	}
+
 static void
 create_disk_monitor(DiskMon *disk, gint first_create)
 	{
@@ -743,45 +1186,49 @@ create_disk_monitor(DiskMon *disk, gint first_create)
 		disk->chart = gkrellm_chart_new0();
 		disk->chart->panel = gkrellm_panel_new0();
 		}
-	else
-		{
-		gkrellm_destroy_decal_list(disk->chart->panel);
-		gkrellm_destroy_krell_list(disk->chart->panel);
-		}
 	cp = disk->chart;
 	p = cp->panel;
 
-	gkrellm_dup_string(&cp->name, disk->name);
-
 	style = gkrellm_panel_style(style_id);
 	gkrellm_create_krell(p, gkrellm_krell_panel_image(style_id), style);
-	setup_disk_scaling(disk);
 
-	cp->h = UC.chart_height[MON_DISK]; 
-	gkrellm_create_chart(disk->vbox, cp, style_id);
+	gkrellm_chart_create(disk->vbox, mon_disk, cp, &disk->chart_config);
+	disk->wb_cd = gkrellm_add_default_chartdata(cp, _("Write Blocks"));
+	disk->rb_cd = gkrellm_add_default_chartdata(cp, _("Read Blocks"));
+	gkrellm_set_draw_chart_function(cp, draw_disk_chart, disk);
 
-	p->textstyle = gkrellm_panel_textstyle(style_id);
+	gkrellm_chartconfig_fixed_grids_connect(cp->config,
+				setup_disk_scaling, cp);
+	gkrellm_chartconfig_grid_resolution_connect(cp->config,
+				setup_disk_scaling, cp);
+	gkrellm_chartconfig_grid_resolution_adjustment(cp->config, TRUE,
+				0, (gfloat) MIN_GRID_RES, (gfloat) MAX_GRID_RES, 0, 0, 0, 70);
+	gkrellm_chartconfig_grid_resolution_label(cp->config,
+				_("Disk I/O blocks per sec"));
+	if (gkrellm_get_chartconfig_grid_resolution(cp->config) < MIN_GRID_RES)
+		gkrellm_set_chartconfig_grid_resolution(cp->config, DEFAULT_GRID_RES);
+	gkrellm_alloc_chartdata(cp);
 
-	gkrellm_configure_panel(p, cp->name, style);
-	gkrellm_create_panel(disk->vbox, p, gkrellm_bg_panel_image(style_id));
-	disk->height = cp->h + p->h;
-	gkrellm_monitor_height_adjust(disk->height);
+	setup_disk_scaling(cp->config, cp);
+
+	gkrellm_panel_configure(p, disk->name, style);
+	gkrellm_panel_create(disk->vbox, mon_disk, p);
 	disk->enabled = TRUE;
 
-	gkrellm_alloc_chart_data(cp);
-
 	if (first_create)
 		{
 		gtk_signal_connect(GTK_OBJECT(cp->drawing_area), "expose_event",
-					(GtkSignalFunc) disk_expose_event, NULL);
+				(GtkSignalFunc) disk_expose_event, NULL);
 		gtk_signal_connect(GTK_OBJECT (p->drawing_area), "expose_event",
-					(GtkSignalFunc) disk_expose_event, NULL);
+				(GtkSignalFunc) disk_expose_event, NULL);
 
 		gtk_signal_connect(GTK_OBJECT(cp->drawing_area), "button_press_event",
-					(GtkSignalFunc) cb_disk_extra, NULL);
+				(GtkSignalFunc) cb_disk_extra, NULL);
+		gtk_signal_connect(GTK_OBJECT(p->drawing_area),
+				"button_press_event", (GtkSignalFunc) cb_panel_press, NULL);
 		}
 	else
-		refresh_disk_chart(disk);
+		draw_disk_chart(disk);
 
 	gkrellm_configure_tooltip(p, &disk->launch);
 	if (*(disk->launch.command) != '\0')
@@ -806,9 +1253,8 @@ create_disk(GtkWidget *vbox, gint first_create)
 		if (first_create)
 			{
 			disk->vbox = gtk_vbox_new(FALSE, 0);
-			gtk_container_add(GTK_CONTAINER(vbox), disk->vbox);
-			if (disk->enabled)
-				gtk_widget_show(disk->vbox);
+			gtk_box_pack_start(GTK_BOX(vbox), disk->vbox, FALSE, FALSE, 0);
+			gtk_widget_show(disk->vbox);
 			}
 		gkrellm_setup_launcher(NULL, &disk->launch, CHART_PANEL_TYPE, 4);
 		if (disk->enabled)
@@ -820,7 +1266,7 @@ create_disk(GtkWidget *vbox, gint first_create)
   |  may dynamically add a drive.
   */
 static void
-check_for_new_disks()
+check_for_new_disks(void)
 	{
 	GList	*list;
 	DiskMon	*disk;
@@ -833,7 +1279,8 @@ check_for_new_disks()
 		if (disk->vbox == NULL)
 			{
 			disk->vbox = gtk_vbox_new(FALSE, 0);
-			gtk_container_add(GTK_CONTAINER(disk_vbox), disk->vbox);
+			gtk_box_pack_start(GTK_BOX(disk_vbox), disk->vbox, FALSE, FALSE,0);
+			gtk_widget_show(disk->vbox);
 			gkrellm_setup_launcher(NULL, &disk->launch, CHART_PANEL_TYPE, 4);
 			}
 		}
@@ -842,27 +1289,8 @@ check_for_new_disks()
 /* --------------------------------------------------------------------	*/
 #define	DISK_CONFIG_KEYWORD	"disk"
 
-static GtkWidget	*launch_vbox;
-
-static gint     disk_map[] =
-	{ 10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000 };
-
-static gint		disk_temp;
-
-static void
-cb_disk_resolution(GtkWidget *widget, GtkSpinButton *spin)
-	{
-	gint    value;
-
-	value = gtk_spin_button_get_value_as_int(spin);
-	if (value != disk_temp) /* Avoid recursion */
-		{
-		value = map_1_2_5(value, disk_map, sizeof(disk_map) / sizeof(int));
-		disk_temp = value;
-		gtk_spin_button_set_value(spin, (gfloat) value);
-		}
-	}
-
+static GtkWidget	*launch_vbox,
+					*text_format_combo;
 
 static void
 save_disk_config(FILE *f)
@@ -876,20 +1304,23 @@ save_disk_config(FILE *f)
 		{
 		disk = (DiskMon *) list->data;
 		if (!using_DiskN_names && disk != composite_disk)
-			fprintf(f, "%s enable (%d,%d) %d %d %d\n", DISK_CONFIG_KEYWORD,
+			fprintf(f, "%s enable (%d,%d) %d %d\n", DISK_CONFIG_KEYWORD,
 					disk->major, disk->minor, disk->enabled,
-					disk->resolution, disk->extra_info);
+					disk->extra_info);
 		else
-			fprintf(f, "%s enable %s %d %d %d\n", DISK_CONFIG_KEYWORD,
+			fprintf(f, "%s enable %s %d %d\n", DISK_CONFIG_KEYWORD,
 					disk->name, disk->enabled,
-					disk->resolution, disk->extra_info);
+					disk->extra_info);
 		if (*(disk->launch.command) != '\0')
 			fprintf(f, "%s launch %s %s\n", DISK_CONFIG_KEYWORD,
 						disk->name, disk->launch.command);
 		if (*(disk->launch.tooltip_comment) != '\0')
 			fprintf(f, "%s tooltip_comment %s %s\n", DISK_CONFIG_KEYWORD,
 						disk->name, disk->launch.tooltip_comment);
+		gkrellm_save_chartconfig(f, disk->chart_config,
+					DISK_CONFIG_KEYWORD, disk->name);
 		}
+	fprintf(f, "%s text_format %s\n", DISK_CONFIG_KEYWORD, text_format);
 	}
 
 static void
@@ -898,42 +1329,45 @@ load_disk_config(gchar *arg)
 	DiskMon	*disk;
 	gchar	config[32], item[CFG_BUFSIZE],
 			disk_name[32], command[CFG_BUFSIZE];
-	gint	major, minor, res, n, extra;
+	gint	major, minor, n, extra;
 
 	if (n_disks == 0)
 		return;
 	n = sscanf(arg, "%31s %[^\n]", config, item);
 	if (n == 2)
 		{
-		if (strcmp(config, "composite") == 0)	/* XXX Obsolete >= 1.0.8 */
-			sscanf(item, "%d %d %d", &composite_disk->enabled,
-					&composite_disk->resolution, &composite_disk->extra_info);
+		if (!strcmp(config, "text_format"))
+			gkrellm_dup_string(&text_format, item);
 		else if (strcmp(config, "enable") == 0)
 			{
 			n = FALSE;
-			res = DEFAULT_RESOLUTION;
 			extra = 0;
 			disk = NULL;
 			if (item[0] == '(')
 				{	/* Avoid user_config pathology (eg kernel 2.4 -> 2.2) */
 				if (   !using_DiskN_names
-					&& sscanf(item, "(%d,%d) %d %d %d",
-						&major, &minor, &n, &res, &extra) > 1
+					&& sscanf(item, "(%d,%d) %d %d",
+						&major, &minor, &n, &extra) > 1
 				   )
 					disk = add_disk_device(major, minor);
 				}
 			else
 				{
-				sscanf(item, "%31s %d %d %d", disk_name, &n, &res, &extra);
+				sscanf(item, "%31s %d %d", disk_name, &n, &extra);
 				disk = lookup_disk_by_name(disk_name);
 				}
 			if (disk)
 				{
 				disk->enabled = n;
-				disk->resolution = res;
 				disk->extra_info = extra;
 				}
 			}
+		else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
+			{
+			sscanf(item, "%31s %[^\n]", disk_name, command);
+			if ((disk = lookup_disk_by_name(disk_name)) != NULL)
+				gkrellm_load_chartconfig(&disk->chart_config, command, 2);
+			}
 		else if (strcmp(config, "launch") == 0)
 			{
 			sscanf(item, "%31s %[^\n]", disk_name, command);
@@ -963,20 +1397,21 @@ add_launch_entry(GtkWidget *vbox, DiskMon *disk)
     }
 
 static void
-apply_disk_config()
+apply_disk_config(void)
 	{
-	GtkSpinButton	*spin;
-	GList			*list;
-	DiskMon			*disk;
+	GList		*list;
+	DiskMon		*disk;
+	gchar		*s;
 
 	if (n_disks == 0)
 		return;
 
+	s = gkrellm_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
+	gkrellm_dup_string(&text_format, s);
+
 	for (list = disk_mon_list; list; list = list->next)
 		{
 		disk = (DiskMon *) list->data;
-		spin = GTK_SPIN_BUTTON(disk->spin_button);
-		disk->resolution = gtk_spin_button_get_value_as_int(spin);
 		
 		if (disk->launch_table)		/* Not NULL only if enabled */
 			{
@@ -993,60 +1428,35 @@ apply_disk_config()
 		if (disk->new_disk)
 			add_launch_entry(launch_vbox, disk);
 		if (disk->new_disk && ! disk->enabled)
-			{
-			gtk_widget_show(disk->vbox);
 			create_disk_monitor(disk, TRUE);
-			disk->chart->primed = FALSE;
-			}
 		else if (! disk->new_disk && disk->enabled)
-			{
-			gtk_widget_hide(disk->vbox);
 			destroy_disk_monitor(disk);
-			}
-		if (disk->chart)
-			setup_disk_scaling(disk);
 		disk->enabled = disk->new_disk;
 		}
 	}
 
 
+#define	DEFAULT_TEXT_FORMAT	"$T"
+
 static gchar	*disk_info_text[] =
 {
-#if defined (__linux__)
-N_("<b>Kernel 2.2.x or 2.0.x\n"),
-N_("The kernel reports accumulated disk I/O stats in /proc/stat for\n"
-	"disk drives /dev/hda through /dev/hdd and /dev/sda through /dev/sdd.\n"
-	"The letter suffix a-d for these devices correspond to charts DiskA\n"
-	"through DiskD.\n"
-	"When both an IDE and SCSI drive have a common minor number (eg hda\n"
-	"and sda), the disk I/O stats for the two drives are combined.  So, if\n"
-	"you have an IDE /dev/hda and a SCSI drive which you wish to monitor\n"
-	"separately, then the SCSI drive should not be configured as /dev/sda.\n"
-	"SCSI drives with minor numbers greater than 3 (eg sde, sdf, ...) do\n"
-	"not have I/O stats reported in /proc/stat\n\n"),
-
-N_("<b>Kernel 2.4.x\n"),
-N_("Data for IDE and SCSI drives is not combined as in kernel 2.2, and\n"
-	"disk charts may be selected by device name instead of generic DiskN\n"
-	"names.")
-#else
-""
-#endif
+N_("<b>Chart Labels\n"),
+N_("Substitution variables for the format string for chart labels:\n"),
+N_("\t$M    maximum chart value\n"),
+N_("\t$T    total read blocks + write blocks\n"),
+N_("\t$r    read blocks\n"),
+N_("\t$w    write blocks\n"),
 };
 
-
 static void
 create_disk_tab(GtkWidget *tab_vbox)
 	{
-	GtkWidget		*table;
 	GtkWidget		*tabs;
 	GtkWidget		*vbox, *vbox1, *hbox;
-	GtkWidget		*label;
-	GtkWidget		*scrolled;
 	GtkWidget		*text;
-	GtkWidget		*sep;
 	GList			*list;
 	DiskMon			*disk;
+	gchar			*s;
 	gint			i;
 
 	check_for_new_disks();
@@ -1057,69 +1467,48 @@ create_disk_tab(GtkWidget *tab_vbox)
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
 	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);
 
-/* --Setup tab */
-	vbox = gkrellm_create_tab(tabs, _("Setup"));
-
-	scrolled = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
+/* -- Options tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Options"));
+	vbox1 = gkrellm_scrolled_vbox(vbox, NULL,
 			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
 
-	vbox1 = gtk_vbox_new(FALSE, 3);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled),vbox1);
-	table = gtk_table_new (n_disks + 2, 4, FALSE /*homogeneous*/);
-	gtk_box_pack_start(GTK_BOX(vbox1), table, FALSE, FALSE, 2);
-
-	label = gtk_label_new(_("Disk Name"));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
-	label = gtk_label_new(_("Chart Resolution"));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
-	sep = gtk_hseparator_new();
-	gtk_table_attach_defaults(GTK_TABLE(table), sep, 0, 3, 1, 2);
-
-	for (i = 2, list = disk_mon_list; list; list = list->next, ++i)
+	for (list = disk_mon_list; list; list = list->next)
 		{
 		disk = (DiskMon *) list->data;
-		disk->enable_button = gtk_check_button_new_with_label(disk->name);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(disk->enable_button),
-				disk->enabled);
-		gtk_table_attach_defaults(GTK_TABLE(table), disk->enable_button,
-				0, 1, i, i + 1);
 
-		hbox = gtk_hbox_new(TRUE, 0);
-		gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 2, i, i+1);
-		gkrellm_spin_button(hbox, &disk->spin_button,
-			(gfloat) disk->resolution, 10.0, 50000.0, 1.0, 1.0, 0, 80,
-			cb_disk_resolution, NULL, FALSE, NULL);
-
-		if (i == 2)
-			{
-			label = gtk_label_new(
+		if (list == disk_mon_list)
+			s = g_strdup_printf("%s - %s", disk->name,
 					_("Composite chart combines data for all disks"));
-			gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-			gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 2, 3);
-			}
-
+		else
+			s = g_strdup(disk->name);
+		gkrellm_check_button(vbox1, &disk->enable_button, disk->enabled,
+				FALSE, 0, s);
+		g_free(s);
 		}
 
-	sep = gtk_hseparator_new();
-	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 2);
+/* -- Setup tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Setup"));
+	vbox1 = gkrellm_framed_vbox(vbox, _("Format String for Chart Labels"),
+			 4, FALSE, 0, 2);
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+	text_format_combo = gtk_combo_new();
+	gtk_widget_set_usize (GTK_WIDGET(text_format_combo), 350, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
+	list = NULL;
+	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
+	list = g_list_append(list, "$T\\C\\f$M");
+	list = g_list_append(list, "\\c\\f$M\\b$T");
+	list = g_list_append(list,
+					_("\\f\\ww\\C\\f$M\\D2\\f\\ar\\. $r\\D1\\f\\aw\\. $w"));
+	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
+	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
+			text_format);
 
-	label = gtk_label_new(
-		_("Chart resolutions are disk I/O blocks/sec per grid"));
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 5);
+	vbox = gkrellm_framed_vbox_end(vbox, _("Launch Commands"), 4, TRUE, 0, 2);
+	launch_vbox = gkrellm_scrolled_vbox(vbox, NULL,
+						GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
-/* --Launch tab */
-	vbox = gkrellm_create_tab(tabs, _("Launch"));
-/*	insert_expanded_filler(vbox); */
-
-	scrolled = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
-			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
-	launch_vbox = gtk_vbox_new(FALSE, 0);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled),
-			launch_vbox);
 	for (i = 0, list = disk_mon_list; list; list = list->next, ++i)
 		{
 		disk = (DiskMon *) list->data;
@@ -1128,19 +1517,11 @@ create_disk_tab(GtkWidget *tab_vbox)
 		}
 
 /* --Info tab */
-	if (*disk_info_text[0] != '\0')
-		{
-		vbox = gkrellm_create_tab(tabs, _("Info"));
-		scrolled = gtk_scrolled_window_new(NULL, NULL);
-		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
+	vbox = gkrellm_create_framed_tab(tabs, _("Info"));
+	text = gkrellm_scrolled_text(vbox, NULL,
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-		gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
-		text = gtk_text_new(NULL, NULL);
-		for (i = 0; i < sizeof(disk_info_text)/sizeof(gchar *); ++i)
-			gkrellm_add_info_text_string(text, _(disk_info_text[i]));
-		gtk_text_set_editable(GTK_TEXT(text), FALSE);
-		gtk_container_add(GTK_CONTAINER(scrolled), text);
-		}
+	for (i = 0; i < sizeof(disk_info_text)/sizeof(gchar *); ++i)
+		gkrellm_add_info_text_string(text, _(disk_info_text[i]));
 	}
 
 static Monitor	monitor_disk =
@@ -1177,13 +1558,18 @@ init_disk_monitor(void)
 	disk = g_new0(DiskMon, 1);
 	disk_mon_list = g_list_append(disk_mon_list, disk);
 	disk->name = g_strdup(_("Disk"));
-	disk->resolution = DEFAULT_RESOLUTION;
 	composite_disk = disk;
+	text_format = g_strdup(DEFAULT_TEXT_FORMAT);
+	mon_disk = &monitor_disk;
 
 	if (setup_disk_interface())
 		{
 		for (list = disk_mon_list->next; list; list = list->next)
-			((DiskMon *) list->data)->enabled = FALSE;
+			{
+			disk = (DiskMon *) list->data;
+			disk->enabled = FALSE;
+			disk->extra_info = TRUE;
+			}
 		if (n_disks > 0)
 			composite_disk->enabled = TRUE;
 		style_id = gkrellm_add_chart_style(&monitor_disk, DISK_STYLE_NAME);
