@@ -18,9 +18,10 @@
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-/* 10/12/2000	NetBSD code contributed by Anthony Mallet
-|			<metall@ficus.yi.org>
-| 2/25/2000	FreeBSD code contributed by Hajimu UMEMOTO ume@mahoroba.org
+/*
+|  4/22/2001  Solaris code contributed by Daisuke Yabuki <dxy@acm.org>
+| 10/12/2000  NetBSD code contributed by Anthony Mallet <metall@ficus.yi.org>
+|  2/25/2000  FreeBSD code contributed by Hajimu UMEMOTO ume@mahoroba.org
 */
 
 #include "gkrellm.h"
@@ -38,15 +39,15 @@ typedef struct
 
 typedef struct
 	{
-	GtkWidget		*vbox;
 	gint			idx;
-	Panel			panel;
+	Panel			*panel;
 	DecalButton		*md_button;
 	Decal			*mount_decal,
 					*text_decal;
 	gchar			*label;
-	gint			label_is_data;
-	gint			restore_label;
+	gboolean		label_is_data,
+					restore_label,
+					mouse_entered;
 
 	Mount			mount;
 	gint			enable_mounting;
@@ -60,13 +61,14 @@ typedef struct
 	*/
 	gint			in_drawer;
 	gint			show_if_mounted;	/* for in_drawer monitors */
-	gint			is_visible;
 
 	gint			is_mounted;
 	gint			is_nfs_fs;
 	gint			eject_pending;
 	GString			*pipe_gstring;		/* output of mount commands */
 
+	gulong			krell_factor;		/* avoid krell math overflow */
+
 	gulong			blocks,
 					bfree,
 					bavail,
@@ -185,7 +187,7 @@ getmntent_fallback(Mount *m)
 	}
 
 static void
-get_linux_proc_mounts_list()
+get_linux_proc_mounts_list(void)
 	{
 	FILE		*f;
 	Mount		*m;
@@ -256,6 +258,132 @@ get_statfs_fsusage(FSmon *fs)
 	}
 #endif
 
+/* ----- Solaris ------------------------------------------------------- */
+#if defined(__solaris__)
+
+#include <sys/mnttab.h>
+#include <sys/vfstab.h>
+#include <sys/statvfs.h>
+
+static void 
+get_solaris_mounts_list(){
+    FILE *fp;
+    Mount *m;
+    struct mnttab mntbuf;
+
+    while (proc_mounts_list) {
+        m = (Mount *) proc_mounts_list->data;
+        if (m->directory)
+            g_free(m->directory);
+        if (m->device)
+            g_free(m->device);
+        if (m->type)
+            g_free(m->type);
+        if (proc_mounts_list->data)
+            g_free(proc_mounts_list->data);
+        proc_mounts_list = g_list_remove(proc_mounts_list, proc_mounts_list->data);
+    }
+
+    if ((fp = fopen(MNTTAB, "r")) == NULL)
+        return;
+
+    while (getmntent(fp, &mntbuf) == 0) {
+        if (strcmp(mntbuf.mnt_fstype, "ufs") && 
+                                  strcmp(mntbuf.mnt_fstype, "nfs"))
+            continue;
+        m = g_new0(Mount, 1);
+        if (!mntbuf.mnt_special) {
+            m->device = g_strdup("");
+        } else {
+            m->device = g_strdup(mntbuf.mnt_special);
+        }
+        if (!mntbuf.mnt_mountp) {
+            m->directory = g_strdup("");
+        } else {
+            m->directory = g_strdup(mntbuf.mnt_mountp);
+        }
+        if (!mntbuf.mnt_fstype) {
+            m->type = g_strdup("");
+        } else { 
+            m->type = g_strdup(mntbuf.mnt_fstype);
+        }
+        proc_mounts_list = g_list_append(proc_mounts_list, m);
+    }
+    fclose(fp);
+}
+
+static void
+get_solaris_statvfs_fsusage(FSmon *fs){
+    struct statvfs st;
+
+    if (fs->mount.directory && statvfs(fs->mount.directory, &st) == 0) {
+        fs->blocks = (gulong) st.f_blocks;
+        fs->bavail = (gulong) st.f_bavail;
+        fs->bfree  = (gulong) st.f_bfree;
+        fs->bsize  = (gulong) st.f_frsize;
+    } else {
+        fs->blocks = 0;
+        fs->bavail = 0;
+        fs->bfree  = 0;
+        fs->bsize  = 0;
+    } 
+}
+
+static void
+get_solaris_vfstab_list(){
+    Mount *m;
+    FILE *fp;
+    struct vfstab vfsbuf;
+
+    while (fstab_list) {
+        m = (Mount *) fstab_list->data;
+        if (m->device)
+            g_free(m->device);
+        if (m->directory)
+            g_free(m->directory);
+        if (m->type)
+            g_free(m->type);
+        if (m->options)
+            g_free(m->options);
+        if (m)
+            g_free(m);
+        fstab_list = g_list_remove(fstab_list, fstab_list->data);
+    }
+
+    if ((fp = fopen(VFSTAB, "r")) == NULL)
+        return;
+
+    while (getvfsent(fp, &vfsbuf) == 0) {
+        if (strcmp(vfsbuf.vfs_fstype, "ufs"))
+            continue;
+        m = g_new0(Mount, 1);
+        if(!vfsbuf.vfs_special) {
+            m->device = g_strdup("");
+        } else {
+            m->device = g_strdup(vfsbuf.vfs_special);
+        }
+        if(!vfsbuf.vfs_mountp) {
+            m->directory = g_strdup("");
+        } else {
+            m->directory = g_strdup(vfsbuf.vfs_mountp); 
+        }
+        if(!vfsbuf.vfs_fstype) {
+            m->type = g_strdup("");
+        } else {
+            m->type = g_strdup(vfsbuf.vfs_fstype);
+        }
+        if(!vfsbuf.vfs_mntopts) { 
+            m->options = g_strdup("");
+        } else {
+            m->options = g_strdup(vfsbuf.vfs_mntopts);
+        }
+        fstab_list = g_list_append(fstab_list, m);
+    }  
+
+    fclose(fp);
+}
+
+#endif /* __solaris__ */
 
 /* ----- Generic /etc/fstab -------------------------------------------- */
 static gchar *
@@ -292,7 +420,7 @@ fix_fstab_name(gchar *buf)
 	}
 
 static void
-get_etc_fstab_list()
+get_etc_fstab_list(void)
 	{
 	FILE			*f;
 	Mount			*m;
@@ -350,7 +478,7 @@ get_etc_fstab_list()
 
 
 static void
-get_glibtop_mountlist()
+get_glibtop_mountlist(void)
 	{
 	glibtop_mountlist	mount_list;
 	glibtop_mountentry	*mount_entries;
@@ -396,7 +524,7 @@ get_glibtop_fsusage(FSmon *fs)
 
 /* ----- Pick a system interface ----------------------------------------- */
 static gint
-setup_fs_interface()
+setup_fs_interface(void)
     {
 	uid = getuid();	/* only real root is allowed to mount/umount always */
 
@@ -404,16 +532,18 @@ setup_fs_interface()
     get_mountlist = get_bsd_mounts_list;
     get_fsusage = get_statfs_fsusage;
     get_fstab_list = get_etc_fstab_list;
-#else
-#if defined(__linux__)
+#elif defined(__linux__)
 	get_mountlist = get_linux_proc_mounts_list;
 	get_fsusage = get_statfs_fsusage;
 	get_fstab_list = get_etc_fstab_list;
+#elif defined(__solaris__) 
+        get_mountlist = get_solaris_mounts_list;
+        get_fsusage = get_solaris_statvfs_fsusage;
+        get_fstab_list = get_solaris_vfstab_list;
 #else
 	get_mountlist = get_glibtop_mountlist;;
 	get_fsusage = get_glibtop_fsusage;
 	get_fstab_list = get_etc_fstab_list;
-#endif
 #endif
     return TRUE;
     }
@@ -430,6 +560,8 @@ setup_fs_interface()
 #define	FORCE_REDRAW	1
 #define	FORCE_UPDATE	2
 
+
+static Monitor		*mon_fs;
 static GtkWidget	*fs_main_vbox,
 					*fs_drawer_vbox;
 
@@ -651,7 +783,7 @@ draw_fs_text_decal(FSmon *fs, gint value)
 	if (value == 0)
 		{
 		d->x_off = 0;
-		gkrellm_draw_decal_text(&fs->panel, d, fs->label, 0);
+		gkrellm_draw_decal_text(fs->panel, d, fs->label, 0);
 		}
 	else
 		{
@@ -672,7 +804,7 @@ draw_fs_text_decal(FSmon *fs, gint value)
 		|  from last call.  If scrolling, will be so each time, if not
 		|  will be so as format_fs_data() return value changes.
 		*/
-		gkrellm_draw_decal_text(&fs->panel, d, buf, changed);
+		gkrellm_draw_decal_text(fs->panel, d, buf, changed);
 		d->text_style = ts_save;
 		}
 	return w;
@@ -784,55 +916,35 @@ mount_command(FSmon *fs)
 	}
 
 static void
-hide_fs_monitor(FSmon *fs)
-	{
-	if (fs->is_visible && !(fs_is_mounted(fs) && fs->show_if_mounted))
-		{
-		gtk_widget_hide(fs->vbox);
-		gkrellm_monitor_height_adjust( - fs->panel.h);
-		fs->is_visible = FALSE;
-		}
-	}
-
-static void
-hide_monitors_in_drawer()
+hide_monitors_in_drawer(void)
 	{
 	FSmon	*fs;
 	GList	*list;
 
+	gkrellm_freeze_side_frame_packing();
 	for (list = fs_mon_list; list; list = list->next)
 		{
 		fs = (FSmon *) list->data;
-		if (fs->in_drawer)
-			hide_fs_monitor(fs);
+		if (fs->in_drawer && (!fs_is_mounted(fs) || !fs->show_if_mounted))
+			gkrellm_panel_hide(fs->panel);
 		}
-	gkrellm_pack_side_frames();
+	gkrellm_thaw_side_frame_packing();
 	}
 
 static void
-show_fs_monitor(FSmon *fs)
-	{
-	if (!fs->is_visible)
-		{
-		gtk_widget_show(fs->vbox);
-		gkrellm_monitor_height_adjust(fs->panel.h);
-		fs->is_visible = TRUE;
-		}
-	}
-
-static void
-show_monitors_in_drawer()
+show_monitors_in_drawer(void)
 	{
 	FSmon	*fs;
 	GList	*list;
 
+	gkrellm_freeze_side_frame_packing();
 	for (list = fs_mon_list; list; list = list->next)
 		{
 		fs = (FSmon *) list->data;
 		if (fs->in_drawer)
-			show_fs_monitor(fs);
+			gkrellm_panel_show(fs->panel);
 		}
-	gkrellm_pack_side_frames();
+	gkrellm_thaw_side_frame_packing();
 	}
 
 static void
@@ -857,13 +969,13 @@ update_fs(void)
 			w = draw_fs_text_decal(fs, 1);
 			if (w > w_scroll)
 				w_scroll = w;
-			gkrellm_draw_layers(&fs->panel);
+			gkrellm_draw_panel_layers(fs->panel);
 			}
 		}
 	if (!fs_in_motion)
 		{
 		if (w_scroll > text_decal_width)
-			x_scroll = (x_scroll + ((UC.update_HZ < 7) ? 2 : 1))
+			x_scroll = (x_scroll + ((gkrellm_update_HZ() < 7) ? 2 : 1))
 					% (w_scroll - text_decal_width / 3);
 		else
 			x_scroll = 0;
@@ -886,7 +998,7 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 	for (list = fs_mon_list; list; list = list->next)
 		{
 		fs = (FSmon *) list->data;
-		p  = &fs->panel;
+		p  = fs->panel;
 		KRELL(p)->previous = 0;
 		if (fs_is_mounted(fs))
 			{
@@ -906,17 +1018,19 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 				|| (fs_check && !fs->is_nfs_fs)
 				|| (nfs_check && fs->is_nfs_fs)
 			   )
+				{
 				(*get_fsusage)(fs);
-			KRELL(p)->full_scale = fs->blocks;
-			l = fs->blocks - fs->bavail;
-			if (fs_in_motion && fs->label_is_data && x_moved)
+				fs->krell_factor = fs->blocks > 2097152 ? 1024 : 1;
+				}			
+			KRELL(p)->full_scale = fs->blocks / fs->krell_factor;
+			l = (fs->blocks - fs->bavail) / fs->krell_factor;
+			if (   (fs_in_motion && fs->label_is_data && x_moved)
+				|| fs->mouse_entered
+			   )
 				l = 0;
 			gkrellm_update_krell(p, KRELL(p), l);
-			if (fs->in_drawer && !fs->is_visible && fs->show_if_mounted)
-				{
-				show_fs_monitor(fs);
-				gkrellm_pack_side_frames();
-				}
+			if (fs->in_drawer && fs->show_if_mounted)
+				gkrellm_panel_show(fs->panel);
 			}
 		else
 			{
@@ -934,17 +1048,14 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 				gkrellm_set_decal_button_index(fs->md_button, D_MISC_LED0);
 			KRELL(p)->full_scale = 100;		/* Arbitrary > 0 */
 			gkrellm_update_krell(p, KRELL(p), 0);
-			if (!fs_drawer_is_shown && fs->in_drawer && fs->is_visible)
-				{
-				hide_fs_monitor(fs);
-				gkrellm_pack_side_frames();
-				}
+			if (!fs_drawer_is_shown && fs->in_drawer)
+				gkrellm_panel_hide(fs->panel);
 			}
 		fs->md_button->sensitive = fs->enable_mounting ? TRUE : FALSE;
 		if (!fs->label_is_data && fs != fs_in_motion)
 			draw_fs_text_decal(fs, 0);
 
-		gkrellm_draw_layers(p);
+		gkrellm_draw_panel_layers(p);
 		if (fs->eject_pending && fs->eject_pending < GK.timer_ticks)
 			{
 			eject_cdrom(fs);
@@ -958,7 +1069,7 @@ printf("fs update %d nfs %d force %d\n", fs_check, nfs_check, force_fs_check);
 				if (fs->pipe_gstring)
 					{
 					gkrellm_message_window(_("GKrellM Mount Error"),
-							fs->pipe_gstring->str, fs->panel.drawing_area);
+							fs->pipe_gstring->str, fs->panel->drawing_area);
 					g_string_free(fs->pipe_gstring, 1);
 					fs->pipe_gstring = NULL;
 					}
@@ -983,9 +1094,9 @@ fs_expose_event(GtkWidget *widget, GdkEventExpose *ev)
 	for (list = fs_mon_list; list; list = list->next)
 		{
 		fs = (FSmon *) list->data;
-		if (widget == fs->panel.drawing_area)
+		if (widget == fs->panel->drawing_area)
 			{
-			gdk_draw_pixmap(widget->window, GK.draw1_GC, fs->panel.pixmap,
+			gdk_draw_pixmap(widget->window, GK.draw1_GC, fs->panel->pixmap,
 					ev->area.x, ev->area.y, ev->area.x, ev->area.y,
 					ev->area.width, ev->area.height);
 			break;
@@ -994,10 +1105,31 @@ fs_expose_event(GtkWidget *widget, GdkEventExpose *ev)
 	return FALSE;
 	}
 
+static gint
+cb_panel_enter(GtkWidget *w, GdkEventButton *ev, FSmon *fs)
+	{
+	if (fs->label_is_data)
+		{
+		fs->mouse_entered = TRUE;
+		force_fs_check = FORCE_REDRAW;
+		}
+	return TRUE;
+	}
+
+static gint
+cb_panel_leave(GtkWidget *w, GdkEventButton *ev, FSmon *fs)
+	{
+	if (fs->mouse_entered)
+		force_fs_check = FORCE_REDRAW;
+	fs->mouse_entered = FALSE;
+	return TRUE;
+	}
+
+
 static gint
 cb_fs_panel_release(GtkWidget *widget, GdkEventButton *ev)
 	{
-	if (ev->button == 1)
+	if (ev->button != 2)
 		return TRUE;
 	if (fs_in_motion /* && fs_in_motion->panel.drawing_area == widget */)
 		{
@@ -1007,7 +1139,7 @@ cb_fs_panel_release(GtkWidget *widget, GdkEventButton *ev)
 				gkrellm_config_modified();
 			fs_in_motion->label_is_data = FALSE;
 			draw_fs_text_decal(fs_in_motion, 0);
-			gkrellm_draw_layers(&fs_in_motion->panel);
+			gkrellm_draw_panel_layers(fs_in_motion->panel);
 			}
 		fs_in_motion->restore_label = TRUE;
 		}
@@ -1024,10 +1156,15 @@ cb_fs_panel_press(GtkWidget *widget, GdkEventButton *ev)
 	GList	*list;
 	Decal	*d;
 
+	if (ev->button == 3)
+		{
+		gkrellm_open_config_window(mon_fs);
+		return TRUE;
+		}
 	for (list = fs_mon_list; list; list = list->next)
 		{
 		fs = (FSmon *) list->data;
-		if (widget != fs->panel.drawing_area)
+		if (widget != fs->panel->drawing_area)
 			continue;
 		d = fs->mount_decal;
 		if (ev->button == 1 && (ev->x < d->x || ev->x > d->x + d->w))
@@ -1036,19 +1173,20 @@ cb_fs_panel_press(GtkWidget *widget, GdkEventButton *ev)
 				hide_monitors_in_drawer();
 			else
 				show_monitors_in_drawer();
-			fs_drawer_is_shown = 1 - fs_drawer_is_shown;
+			fs_drawer_is_shown = !fs_drawer_is_shown;
 			}
-		else if (fs->is_mounted && (ev->button == 2 || ev->button == 3))
+		else if (fs->is_mounted && ev->button == 2)
 			{
 			if (!fs->label_is_data)
 				{
 				fs->label_is_data = TRUE;
 				fs->restore_label = FALSE;
+				fs->mouse_entered = TRUE;
 				gkrellm_config_modified();
 				}
 			x_fs_motion = ev->x;
 			draw_fs_text_decal(fs, 1);
-			gkrellm_draw_layers(&fs->panel);
+			gkrellm_draw_panel_layers(fs->panel);
 			fs_in_motion = fs;
 			x_moved = 0;
 			}
@@ -1098,7 +1236,7 @@ cb_fs_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 			if (fs->label_is_data)
 				{
 				draw_fs_text_decal(fs, 1);
-				gkrellm_draw_layers(&fs->panel);
+				gkrellm_draw_panel_layers(fs->panel);
 				}
 			}
 		if (x_moved > 0)
@@ -1120,40 +1258,25 @@ cb_fs_mount_button(DecalButton *button)
 		mount_command((FSmon *) button->data);
 	}
 
-
 static void
-create_fs_monitor(GtkWidget *vbox1, FSmon *fs, gint index, gint first_create)
+create_fs_monitor(GtkWidget *vbox, FSmon *fs, gint index, gint first_create)
 	{
-	GtkWidget	*vbox;
 	Style		*style;
 	Panel		*p;
 	gint		x, w;
 
-	p = &fs->panel;
 	if (first_create)
-		{
-		vbox = gtk_vbox_new(FALSE, 0);
-		gtk_container_add(GTK_CONTAINER(vbox1), vbox);
-		fs->vbox = vbox;
-
-		p->textstyle = gkrellm_textstyle_new0();
-		p->label = gkrellm_label_new0();
-		}
-	else		/* Theme or size change, will recreate decals and krells */
-		{
-		vbox = fs->vbox;
-		gkrellm_destroy_decal_list(p);
-		gkrellm_destroy_krell_list(p);
-		}
-
+		fs->panel = gkrellm_panel_new0();
+	p = fs->panel;
 	fs->idx = index;
 	++n_fs_monitors;
+	fs->krell_factor = 1;
 
 	style = gkrellm_meter_style(style_id);
-	p->textstyle = gkrellm_meter_textstyle(style_id);
 	x = style->margin;
 
-	fs->text_decal = gkrellm_create_decal_text(p, "Ap0", p->textstyle,
+	fs->text_decal = gkrellm_create_decal_text(p, "Afp0",
+				gkrellm_meter_textstyle(style_id),
 				style, x, -1, -1);	/* -1 means use y default */
 
 	fs->mount_decal = gkrellm_create_decal_pixmap(p, GK.decal_misc_pixmap,
@@ -1165,20 +1288,17 @@ create_fs_monitor(GtkWidget *vbox1, FSmon *fs, gint index, gint first_create)
 	*/
 	if (style->label_position >= 50)
 		x += fs->mount_decal->w;
-	w = UC.chart_width - fs->mount_decal->w
+	w = gkrellm_chart_width() - fs->mount_decal->w
 			- 2 * style->margin;
 	text_decal_width = w;
 
 	gkrellm_create_krell(p, gkrellm_krell_meter_image(style_id), style);
 
-	gkrellm_configure_panel(p, NULL, style);
-
-	gkrellm_create_panel(vbox, p, gkrellm_bg_meter_image(style_id));
+	gkrellm_panel_configure(p, NULL, style);
+	gkrellm_panel_create(vbox, mon_fs, p);
 
 	/* Center the mount decal
 	*/
-	fs->mount_decal->y = (p->h - fs->mount_decal->h) / 2;
-
 	fs->md_button = gkrellm_make_decal_button(p, fs->mount_decal,
 			cb_fs_mount_button, fs, D_MISC_FS_UMOUNTED, D_MISC_FS_PRESSED);
 
@@ -1192,11 +1312,13 @@ create_fs_monitor(GtkWidget *vbox1, FSmon *fs, gint index, gint first_create)
 				(GtkSignalFunc) cb_fs_panel_release, NULL);
 		gtk_signal_connect(GTK_OBJECT(p->drawing_area),"motion_notify_event",
 				(GtkSignalFunc) cb_fs_panel_motion, NULL);
-		if (!fs->in_drawer || (fs_is_mounted(fs) && fs->show_if_mounted))
-			show_fs_monitor(fs);
+		gtk_signal_connect(GTK_OBJECT(p->drawing_area), "enter_notify_event",
+				(GtkSignalFunc) cb_panel_enter, fs);
+		gtk_signal_connect(GTK_OBJECT(p->drawing_area), "leave_notify_event",
+				(GtkSignalFunc) cb_panel_leave, fs);
+		if (fs->in_drawer && (!fs_is_mounted(fs) || !fs->show_if_mounted))
+			gkrellm_panel_hide(fs->panel);
 		}
-	else if (fs->is_visible)
-		gkrellm_monitor_height_adjust(p->h);
 
 	draw_fs_text_decal(fs, 0);
 	force_fs_check = FORCE_UPDATE;
@@ -1212,10 +1334,8 @@ destroy_fs_monitor(FSmon *fs)
 	{
 	g_free(fs->label);
 	g_free(fs->mount.directory);
-	if (fs->is_visible)
-		gkrellm_monitor_height_adjust( - fs->panel.h);
-	gkrellm_destroy_panel(&fs->panel);
-	gtk_widget_destroy(fs->vbox);
+	gkrellm_destroy_button(fs->md_button);
+	gkrellm_panel_destroy(fs->panel);
 	g_free(fs);
 	--n_fs_monitors;
 	}
@@ -1227,17 +1347,14 @@ create_fs(GtkWidget *vbox, gint first_create)
 	FSmon		*fs;
 	gint		i;
 
-	/* If first time called, create the container widgets.  Subsequent
-	|  calls will be for theme or size changes.
-	*/
 	if (fs_main_vbox == NULL)
 		{
 		fs_main_vbox = gtk_vbox_new(FALSE, 0);
-		gtk_container_add(GTK_CONTAINER(vbox), fs_main_vbox);
+		gtk_box_pack_start(GTK_BOX(vbox), fs_main_vbox, FALSE, FALSE, 0);
 		gtk_widget_show(fs_main_vbox);
 
 		fs_drawer_vbox = gtk_vbox_new(FALSE, 0);
-		gtk_container_add(GTK_CONTAINER(vbox), fs_drawer_vbox);
+		gtk_box_pack_start(GTK_BOX(vbox), fs_drawer_vbox, FALSE, FALSE, 0);
 		gtk_widget_show(fs_drawer_vbox);
 		fs_drawer_is_shown = FALSE;
 		}
@@ -1248,6 +1365,8 @@ create_fs(GtkWidget *vbox, gint first_create)
 		create_fs_monitor(fs->in_drawer ? fs_drawer_vbox : fs_main_vbox,fs,
 					i, first_create);
 		}
+	if (g_list_length(fs_mon_list) == 0)
+		hide_spacers(mon_fs);
 	}
 
 #define	FS_CONFIG_KEYWORD	"fs"
@@ -1281,13 +1400,21 @@ save_fs_config(FILE *f)
 	fprintf(f, "%s data_format %s\n", FS_CONFIG_KEYWORD, data_format);
 	}
 
+static gboolean
+fstab_user_permission(Mount *m)
+	{
+	if (strstr(m->options, "user") || strstr(m->options, "owner"))
+		return TRUE;
+	return FALSE;
+	}
+
 static gint
 fix_fstab_mountable_changed(FSmon *fs)
 	{
 	Mount	*m;
 
 	m = in_fstab_list(fs->mount.directory);
-	if (   (!m || (!strstr(m->options, "user") && uid != 0))
+	if (   (!m || (!fstab_user_permission(m) && uid != 0))
 		&& fs->enable_mounting == MT_FSTAB
 	   )
 		{
@@ -1345,6 +1472,7 @@ load_fs_config(gchar *arg)
 			fs->enable_mounting = mnt;
 			fix_fstab_mountable_changed(fs);
 			fs->in_drawer = drawer;
+			fs->krell_factor = 1;
 			fs->show_if_mounted = show;
 			if (is_data)
 				{
@@ -1459,7 +1587,7 @@ cb_combo_changed(GtkWidget *widget, gpointer drawer)
 
 	s = gkrellm_entry_get_text(&(GTK_COMBO(dir_combo)->entry));
 	m = in_fstab_list(s);
-	if (m != NULL && (strstr(m->options, "user") != NULL || uid == 0))
+	if (m && (fstab_user_permission(m) || uid == 0))
 		{
 		gtk_widget_set_sensitive(mounting_button, TRUE);
 		if (GTK_TOGGLE_BUTTON(mounting_button)->active)
@@ -1714,7 +1842,7 @@ cb_delete_fs(GtkWidget *widget, gpointer drawer)
   |  possible (overly complicated? - I could just destroy all then re-create).
   */
 static void
-apply_fs_config()
+apply_fs_config(void)
 	{
 	GList			*new_fs_list, *old_fs_list;
 	FSmon			*fs;
@@ -1819,10 +1947,12 @@ apply_fs_config()
 		new_fs_list = g_list_append(new_fs_list, fs);
 		if (need_new_monitor)
 			create_fs_monitor(fs_drawer_vbox, fs, n_fs_monitors, TRUE);
-		else if (fs_drawer_is_shown)
-			show_fs_monitor(fs);
+		else if (   fs_drawer_is_shown
+				 || (fs_is_mounted(fs) && fs->show_if_mounted)
+				)
+			gkrellm_panel_show(fs->panel);
 		else
-			hide_fs_monitor(fs);
+			gkrellm_panel_hide(fs->panel);
 		}
 	/* Destroy any remaining secondary monitors left in the old fs_mon_list.
 	*/
@@ -1836,9 +1966,12 @@ apply_fs_config()
 	while (fs_mon_list)
 		fs_mon_list = g_list_remove(fs_mon_list, fs_mon_list->data);
 	fs_mon_list = new_fs_list;
-	gkrellm_pack_side_frames();
 	fs_list_modified = FALSE;
 	force_fs_check = FORCE_UPDATE;
+	if (g_list_length(fs_mon_list) > 0)
+		show_spacers(mon_fs);
+	else
+		hide_spacers(mon_fs);
 	}
 
 static gchar *fs_title1[5] =
@@ -1877,6 +2010,7 @@ fill_fs_tab(GtkWidget *vbox, GtkWidget **label_entry, GtkWidget **dir_combo,
 
 	label = gtk_label_new(_("Mount Point"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 6);
+	gtk_misc_set_alignment(GTK_MISC(label), 1.00, 0.5);
 	*dir_combo = gtk_combo_new();
 	gtk_box_pack_start(GTK_BOX(hbox), *dir_combo, FALSE, FALSE,0);
 	combo_list = NULL;
@@ -2003,9 +2137,9 @@ N_("If a mount point is in your\n"
 "\tto be able to mount your floppy, your /etc/fstab could have\n"
 "\teither of:\n"),
 
-N_("<i>\t\t/dev/fd0  /mnt/floppy  ext2   user,noauto,rw,exec  0  0\n"),
+N_("<i>\t\t/dev/fd0 /mnt/floppy ext2 user,noauto,rw,exec 0 0\n"),
 N_("\tor\n"),
-N_("<i>\t\t/dev/fd0  /mnt/floppy  ext2   user,defaults  0  0\n\n"),
+N_("<i>\t\t/dev/fd0 /mnt/floppy ext2 user,defaults 0 0\n\n"),
 
 N_("<b>\t2) Mounting using custom commands: "),
 N_("If GKrellM is run as root or if\n"
@@ -2014,8 +2148,8 @@ N_("If GKrellM is run as root or if\n"
 "\tbox.  A umount command must also be entered if you choose this\n"
 "\tmethod.  Example mount and umount entries using sudo:\n"),
 
-N_("<i>\t\tsudo /bin/mount -t msdos /dev/fd0 /mnt/A\n"
-	"\t\tsudo /bin/umount /mnt/A\n"),
+N_("<i>\t\tsudo /bin/mount -t msdos /dev/fd0 /mnt/A\n"),
+N_("<i>\t\tsudo /bin/umount /mnt/A\n"),
 
 N_("\tNotes: the mount point specified in a custom mount command\n"
    "\t(/mnt/A in this example) must be the same as entered in the\n"
@@ -2037,13 +2171,22 @@ N_("File system monitors can be created as primary (always visible)\n"
 	"could be created with mounting enabled just to have the\n"
 	"mount/umount convenience.\n\n"),
 
+N_("<b>Panel Labels\n"),
+N_("Substitution variables for the format string for file system panels:\n"),
+N_("\t$t    total capacity\n"),
+N_("\t$u    used space\n"),
+N_("\t$f    free space\n"),
+N_("\t$U    used %,\n"),
+N_("\t$F    free %\n"),
+N_("\t$l    the panel label\n"),
+"\n",
 N_("<b>Mouse Button Actions:\n"),
 N_("<b>\tLeft "),
 N_("click on a fs monitor label to show/hide secondary fs monitors.\n"),
 N_("<b>\tLeft "),
 N_("click on a drive decal to apply mount/umount commands (if enabled)\n"
 	"\t\tto the mount point being monitored.\n"),
-N_("<b>\tMiddle or Right "),
+N_("<b>\tMiddle "),
 N_("click on a FS panel to scroll a programmable display\n"),
 N_("\t\tof file system capacities (default is total and free space).\n\n"),
 
@@ -2055,11 +2198,9 @@ static void
 create_fs_tab(GtkWidget *tab_vbox)
 	{
 	GtkWidget		*tabs;
-	GtkWidget		*vbox, *hbox;
-	GtkWidget		*scrolled;
+	GtkWidget		*vbox, *vbox1, *hbox;
 	GtkWidget		*text;
 	GtkWidget		*label;
-	GtkWidget		*sep;
 	GList			*list;
 	gint			i;
 
@@ -2070,33 +2211,42 @@ create_fs_tab(GtkWidget *tab_vbox)
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
 	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);
 
-	vbox = gkrellm_create_tab(tabs, _("Primary"));
+	vbox = gkrellm_create_framed_tab(tabs, _("Primary"));
 	fill_fs_tab(vbox, &primary_label_entry, &primary_dir_combo,
 			&primary_mounting_button, NULL,
 			&primary_mount_entry, &primary_umount_entry,
 			&primary_clist, &primary_drawer);
 
-	vbox = gkrellm_create_tab(tabs, _("Secondary"));
+	vbox = gkrellm_create_framed_tab(tabs, _("Secondary"));
 	fill_fs_tab(vbox, &secondary_label_entry, &secondary_dir_combo,
 			&secondary_mounting_button, &show_if_mounted_button,
 			&secondary_mount_entry, &secondary_umount_entry,
 			&secondary_clist, &secondary_drawer);
 
-/* --Options tab */
-	vbox = gkrellm_create_tab(tabs, _("Options"));
-	label = gtk_label_new(
-_("A middle or right click in a File System monitor panel toggles a display\n"
-"of capacities in a format controlled by a user supplied format string where\n"
-"$t is total capacity, $u is used space, $f is free space, $U is used %,\n"
-"$F is free %, and $l is the panel label."));
-	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 2);
-	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+/* --Setup tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Setup"));
 
+	vbox1 = gkrellm_framed_vbox(vbox, _("Options"), 4, FALSE, 0, 2);
+	gkrellm_check_button(vbox1, &binary_units_button, binary_units, FALSE, 0,
+	_("Use binary units (MiB, GiG) for reporting disk capacities."));
+
+	gkrellm_check_button(vbox1, &cdrom_auto_eject_button, cdrom_auto_eject,
+				FALSE, 0, _("Enable eject when CD-ROMs are unmounted"));
+
+	vbox1 = gkrellm_framed_vbox_end(vbox, NULL, 4, FALSE, 0, 2);
+	gkrellm_spin_button(vbox1, &fs_check_spin_button, (gfloat) fs_check_timeout,
+		1.0, 15.0, 1.0, 5.0, 0 /*digits*/, 0 /*width*/, NULL, NULL, FALSE,
+		_("Seconds between data updates for local mounted file systems"));
+	gkrellm_spin_button(vbox1, &nfs_check_spin_button,
+		(gfloat) nfs_check_timeout, 5.0, 60.0, 1.0, 5.0, 0, 0, NULL,NULL,FALSE,
+		_("Seconds between data updates for remote mounted file systems"));
+
+	vbox1 = gkrellm_framed_vbox_end(vbox,
+			_("Format String for Panel Labels"), 4, FALSE, 0, 2);
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 2);
 	data_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(hbox), data_format_combo, FALSE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), data_format_combo, FALSE, FALSE, 0);
 	list = NULL;
 	list = g_list_append(list, DEFAULT_DATA_FORMAT);
 	list = g_list_append(list, _("$t - $u used"));
@@ -2105,37 +2255,14 @@ _("A middle or right click in a File System monitor panel toggles a display\n"
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(data_format_combo)->entry),
 			data_format);
 	label = gtk_label_new(_("Format string for FS data display"));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
-
-	gkrellm_check_button(vbox, &binary_units_button, binary_units, FALSE, 4,
-	_("Use binary units (MiB, GiG) for reporting disk capacities."));
-	insert_expanded_filler(vbox);
-
-	sep = gtk_hseparator_new();
-	gtk_box_pack_start(GTK_BOX(vbox), sep, TRUE, TRUE, 0);
-
-	gkrellm_check_button(vbox, &cdrom_auto_eject_button, cdrom_auto_eject,
-				FALSE, 4, _("Enable eject when CD-ROMs are unmounted"));
-
-	gkrellm_spin_button(vbox, &fs_check_spin_button, (gfloat) fs_check_timeout,
-		1.0, 15.0, 1.0, 5.0, 0 /*digits*/, 0 /*width*/, NULL, NULL, FALSE,
-		_("Seconds between data updates for local mounted file systems"));
-
-	gkrellm_spin_button(vbox, &nfs_check_spin_button,
-		(gfloat) nfs_check_timeout, 5.0, 60.0, 1.0, 5.0, 0, 0, NULL,NULL,FALSE,
-		_("Seconds between data updates for remote mounted file systems"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 2);
 
 /* --Info tab */
-	vbox = gkrellm_create_tab(tabs, _("Info"));
-	scrolled = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
-			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
-	text = gtk_text_new(NULL, NULL);
+	vbox = gkrellm_create_framed_tab(tabs, _("Info"));
+	text = gkrellm_scrolled_text(vbox, NULL,
+				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	for (i = 0; i < sizeof(fs_info_text)/sizeof(gchar *); ++i)
 		gkrellm_add_info_text_string(text, _(fs_info_text[i]));
-	gtk_text_set_editable(GTK_TEXT(text), FALSE);
-	gtk_container_add(GTK_CONTAINER(scrolled), text);
 	}
 
 
@@ -2169,6 +2296,7 @@ init_fs_monitor(void)
 	monitor_fs.name = _(monitor_fs.name);
 	style_id = gkrellm_add_meter_style(&monitor_fs, FS_STYLE_NAME);
 	data_format = g_strdup(DEFAULT_DATA_FORMAT);
+	mon_fs = &monitor_fs;
 	if (setup_fs_interface())
 		{
 		(*get_fstab_list)();
