@@ -57,34 +57,39 @@ exp_MA(Krell *k)
   /* If the Krell has moved, redraw its layer on its stencil.
   */
 void
-gkrellm_update_krell(Panel *p, Krell *krell, unsigned long value)
+gkrellm_update_krell(Panel *p, Krell *k, gulong value)
 	{
 	gint	xnew, x_src, y_src, x_dst, w1, w2, w_overlap, d, frame;
 
-	if (!p || !krell || krell->full_scale == 0)
+	if (!p || !k || k->full_scale == 0)
 		return;
-	if (value < krell->previous)		/* unsigned long overflow? */
+	k->value = value;
+	if (value < k->previous)		/* unsigned long overflow? */
 		{
-		krell->previous = value;
+		k->previous = value;
 		return;
 		}
-	krell->reading = (gint) (value - krell->previous) * krell->full_scale_expand;
-	if (krell->reading > krell->full_scale)
-		krell->reading = krell->full_scale;
+	if (!k->monotonic)
+		k->previous = 0;
+	k->reading = (gint) (value - k->previous) * k->full_scale_expand;
+	if (k->reading > k->full_scale)
+		k->reading = k->full_scale;
 
-	krell->ema = (krell->period > 1) ? exp_MA(krell) : krell->reading;
-	krell->previous = value;
-	krell->last_reading = krell->reading;
+	k->ema = (k->period > 1) ? exp_MA(k) : k->reading;
+	if (k->monotonic)
+		k->previous = value;
+	k->last_reading = k->reading;
 
-	xnew = krell->x0 + krell->ema * krell->w_scale / krell->full_scale;
-	if (xnew == 0 && krell->reading)
+	xnew = k->x0 + k->ema * k->w_scale / k->full_scale;
+
+	if (xnew == k->x0 && k->reading)
 		{
-		xnew = 1;	/* Show something for all activity */
-		krell->ema = 1;
+		xnew = k->x0 + 1;	/* Show something for all activity */
+		k->ema = 1;
 		}
-	if (xnew == krell->x_position)
+	if (xnew == k->x_position)
 		return;
-	krell->x_position = xnew;
+	k->x_position = xnew;
 
 
 	/* If the krell has depth, pick the frame to display as function of
@@ -93,11 +98,11 @@ gkrellm_update_krell(Panel *p, Krell *krell, unsigned long value)
 	|  diplay frame depth - 1 at xnew == full_scale, and display other
 	|  frames in between at xnew proportional to ema/full_scale.
 	*/
-	d = krell->depth;
+	d = k->depth;
 
-	if (krell->ema == 0 || xnew == krell->x0) /* Krell can settle before ema*/
+	if (k->ema == 0 || xnew == k->x0) /* Krell can settle before ema*/
 		frame = 0;
-	else if (krell->ema >= krell->full_scale)
+	else if (k->ema >= k->full_scale)
 		frame = d - 1;
 	else
 		{
@@ -106,79 +111,96 @@ gkrellm_update_krell(Panel *p, Krell *krell, unsigned long value)
 		else if (d == 2)
 			frame = 1;
 		else
-			frame = 1 + ((d - 2) * krell->ema / krell->full_scale);
+			frame = 1 + ((d - 2) * k->ema / k->full_scale);
 		}
-	y_src = krell->h_frame * frame;
+	y_src = k->h_frame * frame;
 
 	/* Clear the krell stencil bitmap.
 	*/
-	gdk_draw_rectangle(krell->stencil, GK.bit0_GC, TRUE,
-				0,0,  krell->w, krell->h_frame);
+	gdk_draw_rectangle(k->stencil, GK.bit0_GC, TRUE,
+				0,0,  k->w, k->h_frame);
 
-	x_src = krell->x_hot - xnew;
+	x_src = k->x_hot - (xnew - k->x0);
 	if (x_src < 0)
 		x_src = 0;
 
-	x_dst = xnew - krell->x_hot;
-	if (x_dst < 0)
-		x_dst = 0;
+	x_dst = xnew - k->x_hot;
+	if (x_dst < k->x0)
+		x_dst = k->x0;
 
-	w1 = krell->w - x_src;
-	w2 = p->w - x_dst;
+	w1 = k->w - x_src;
+	w2 = (k->x0 + k->w_scale + 1) - x_dst;
 	w_overlap = (w2 > w1) ? w1 : w2;
 
-#if 0
-if (strcmp(krell->name, "ppp0") == 0)
-{
-printf(_("%s: ema=%d xnew=%d x0=%d full_scale=%d reading=%d\n"), krell->name,
-krell->ema, xnew, krell->x0, krell->full_scale, krell->reading);
-printf(_("    frame=%d xsrc=%d ysrc=%d xdst=%d ydst=%d w=%d h=%d hot=%d\n"),
-frame, x_src, y_src, x_dst, krell->y0, w_overlap, krell->h_frame, krell->x_hot);
-}
-#endif
-
-	if (krell->mask)
-		gdk_draw_pixmap(krell->stencil, GK.bit1_GC, krell->mask,
-				x_src, y_src, x_dst, 0, w_overlap, krell->h_frame);
+	if (k->mask)
+		gdk_draw_pixmap(k->stencil, GK.bit1_GC, k->mask,
+				x_src, y_src, x_dst, 0, w_overlap, k->h_frame);
 	else
-		gdk_draw_rectangle(krell->stencil, GK.bit1_GC,
-				TRUE, x_dst, 0, w_overlap, krell->h_frame);
+		gdk_draw_rectangle(k->stencil, GK.bit1_GC,
+				TRUE, x_dst, 0, w_overlap, k->h_frame);
 
-	krell->old_draw = krell->draw;
+	k->old_draw = k->draw;
 
-	krell->draw.x_src = x_src;
-	krell->draw.y_src = y_src;
-	krell->draw.x_dst = x_dst;
-	krell->draw.y_dst = krell->y0;
-	krell->draw.w = w_overlap;
-	krell->draw.h = krell->h_frame;
+	k->draw.x_src = x_src;
+	k->draw.y_src = y_src;
+	k->draw.x_dst = x_dst;
+	k->draw.y_dst = k->y0;
+	k->draw.w = w_overlap;
+	k->draw.h = k->h_frame;
 
-	krell->modified = TRUE;
+	k->modified = TRUE;
 	}
 
+void
+gkrellm_draw_decal_on_chart(Chart *cp, Decal *d, gint x, gint y)
+	{
+	if (!cp || !d || d->state != DS_VISIBLE)
+		return;
+	gdk_gc_set_clip_mask(GK.text_GC, d->stencil);
+	gdk_gc_set_clip_origin(GK.text_GC, x, y);
+	gdk_draw_pixmap(cp->pixmap, GK.text_GC, d->pixmap,
+				0, d->y_src, x, y, d->w, d->h);
+
+	gdk_gc_set_clip_mask(GK.text_GC, NULL);
+	gdk_gc_set_clip_origin(GK.text_GC, 0, 0);
+	}
 
   /* Push decal pixmaps through their stencils onto a Panel expose pixmap
   */
-static void
-push_decal_pixmaps(Panel *p)
+static gboolean
+push_decal_pixmaps(Panel *p, gboolean top_layer)
 	{
-	GList	*list;
-	Decal	*d;
+	GList		*list;
+	Decal		*d;
+	gboolean	on_top, do_top_layer = FALSE;
+	gboolean	restore_gc = FALSE;
 
 	if (!p)
-		return;
+		return FALSE;
 	for (list = p->decal; list; list = list->next)
 		{
 		d = (Decal *) list->data;
-		if (d->state != DS_VISIBLE)
+		on_top = (d->flags & DF_TOP_LAYER);
+		if (on_top && !top_layer)
+			do_top_layer = TRUE;
+
+		if (   d->state != DS_VISIBLE
+			|| (on_top && !top_layer) || (!on_top && top_layer)
+		   )
 			continue;
 		gdk_gc_set_clip_mask(GK.text_GC, d->stencil);
 		gdk_gc_set_clip_origin(GK.text_GC, d->x, d->y);
 		gdk_draw_pixmap(p->pixmap, GK.text_GC, d->pixmap,
 					0, d->y_src, d->x, d->y, d->w, d->h);
+		restore_gc = TRUE;
 		}
-	gdk_gc_set_clip_mask(GK.text_GC, NULL);
-	gdk_gc_set_clip_origin(GK.text_GC, 0, 0);
+	if (restore_gc)
+		{
+		gdk_gc_set_clip_mask(GK.text_GC, NULL);
+		gdk_gc_set_clip_origin(GK.text_GC, 0, 0);
+		}
+
+	return do_top_layer;
 	}
 
   /* Push krell pixmaps through their stencils onto a Panel expose pixmap
@@ -189,7 +211,8 @@ push_krell_pixmaps(Panel *p)
 	GList		*list;
 	Krell		*k;
 	Draw_rec	*dr;
-	gint		restore_clip	= FALSE;
+	gboolean	restore_clip_mask   = FALSE,
+				restore_clip_origin = FALSE;
 
 	if (!p)
 		return;
@@ -197,78 +220,39 @@ push_krell_pixmaps(Panel *p)
 		{
 		k = (Krell *) list->data;
 		gdk_gc_set_clip_mask(GK.text_GC, k->stencil);
-		if (k->y0 > 0 || restore_clip)
+		if (k->y0 != 0 || restore_clip_origin)
 			{
 			gdk_gc_set_clip_origin(GK.text_GC, 0, k->y0);
-			restore_clip = TRUE;
+			restore_clip_origin = TRUE;
 			}
 		dr = &k->draw;
 		gdk_draw_pixmap(p->pixmap, GK.text_GC, k->pixmap,
 					dr->x_src, dr->y_src, dr->x_dst, dr->y_dst, dr->w, dr->h);
+		restore_clip_mask = TRUE;
 		}
-	gdk_gc_set_clip_mask(GK.text_GC, NULL);
-	if (restore_clip)
+	if (restore_clip_mask)
+		gdk_gc_set_clip_mask(GK.text_GC, NULL);
+	if (restore_clip_origin)
 		gdk_gc_set_clip_origin(GK.text_GC, 0, 0);
 	}
 
 
-void
-gkrellm_draw_decal_pixmap(Panel *p, Decal *d, gint index)
-	{
-	gint	y_src;
-
-	if (!p || !d)
-		return;
-	if (d->value != index)
-		{
-		/* Write the new decal mask onto the panel stencil
-		*/
-		y_src = index * d->h;
-		d->y_src = y_src;
-		if (d->mask)		/* Can be NULL if no transparency	*/
-			gdk_draw_pixmap(d->stencil, GK.bit1_GC, d->mask,
-					0, y_src, 0, 0, d->w, d->h);
-		else	/* Fill decal extent with white	*/
-			gdk_draw_rectangle(d->stencil, GK.bit1_GC,
-					TRUE, 0, 0, d->w, d->h);
-		d->modified = TRUE;
-		}
-	d->value = index;
-	}
-
-void
-gkrellm_draw_decal_text(Panel *p, Decal *d, gchar *s, gint value)
-	{
-	TextStyle	*ts;
-	gint		x, y;
-
-	if (!p || !d || d->state == DS_INVISIBLE)
-		return;
-	if (d->value != value || value == -1)
-		{
-		ts = &d->text_style;
-		x = d->x_off;
-		y = d->y_baseline;
-		gdk_draw_pixmap(d->pixmap, GK.draw1_GC, p->bg_pixmap,
-				d->x, d->y,  0, 0,  d->w, d->h);
-		gkrellm_draw_string(d->pixmap, ts, x, y, s);
-		gdk_draw_rectangle(d->stencil, GK.bit0_GC, TRUE, 0, 0, d->w, d->h);
-		gdk_draw_string(d->stencil, ts->font, GK.bit1_GC, x, y, s);
-		if (ts->effect)
-			gdk_draw_string(d->stencil, ts->font, GK.bit1_GC, x + 1, y + 1, s);
-		d->modified = TRUE;
-		}
-	d->value = value;
-	}
-
-
+/* ---- Deprecated function - name change ---- */
 void
 gkrellm_draw_layers(Panel *p)
 	{
+	gkrellm_draw_panel_layers(p);
+	}
+
+void
+gkrellm_draw_panel_layers(Panel *p)
+	{
+	GdkDrawable	*drawable;
 	GList		*list;
 	Krell		*k;
 	Decal		*d;			/* LED's or other graphic	*/
 	Draw_rec	*dr;
+	gboolean	do_top_layer_decals;
 
 	if (!p)
 		return;
@@ -306,8 +290,10 @@ gkrellm_draw_layers(Panel *p)
 	*/
 	if (p->modified)
 		{
-		push_decal_pixmaps(p);
+		do_top_layer_decals = push_decal_pixmaps(p, FALSE);
 		push_krell_pixmaps(p);
+		if (do_top_layer_decals)
+			push_decal_pixmaps(p, TRUE);
 
 		/* Draw regions from the expose pixmap onto the screen.  Draw the
 		|  Old krell region and the new krell region.
@@ -319,31 +305,42 @@ gkrellm_draw_layers(Panel *p)
 					0, 0,   0, 0,   p->w, p->h);
 		else
 #endif
+		drawable = p->drawing_area->window;
 		for (list = p->decal; list; list = list->next)
 			{
 			d = (Decal *) list->data;
-			if (d->state != DS_INVISIBLE)
-				gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC,
+			if (d->state != DS_INVISIBLE && drawable)
+				gdk_draw_pixmap(drawable, GK.draw1_GC,
 						p->pixmap, d->x, d->y,  d->x, d->y,   d->w, d->h);
 			d->modified = FALSE;
 			}
 		for (list = p->krell; list; list = list->next)
 			{
 			k = (Krell *) list->data;
-			dr = &k->old_draw;
-			gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC, p->pixmap,
+			if (drawable)
+				{
+				dr = &k->old_draw;
+				gdk_draw_pixmap(drawable, GK.draw1_GC, p->pixmap,
 					dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
-			dr = &k->draw;
-			gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC, p->pixmap,
+				dr = &k->draw;
+				gdk_draw_pixmap(drawable, GK.draw1_GC, p->pixmap,
 					dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
+				}
 			k->modified = FALSE;
 			}
 		}
 	p->modified = FALSE;
 	}
 
+/* ---- Deprecated function - name change ---- */
 void
 gkrellm_draw_layers_force(Panel *p)
+	{
+	gkrellm_draw_panel_layers_force(p);
+	}
+
+void
+gkrellm_draw_panel_layers_force(Panel *p)
 	{
 	GList	*list;
 
@@ -353,7 +350,79 @@ gkrellm_draw_layers_force(Panel *p)
 		((Decal *) list->data)->modified = TRUE;
 	for (list = p->krell; list; list = list->next)
 		((Krell *) list->data)->modified = TRUE;
-	gkrellm_draw_layers(p);
+	gkrellm_draw_panel_layers(p);
+	}
+
+
+void
+gkrellm_draw_decal_pixmap(Panel *p, Decal *d, gint index)
+	{
+	gint	y_src;
+
+	if (!d)
+		return;
+	if (d->value != index)
+		{
+		/* Write the new decal mask onto the panel stencil
+		*/
+		y_src = index * d->h;
+		d->y_src = y_src;
+		if (d->mask)		/* Can be NULL if no transparency	*/
+			gdk_draw_pixmap(d->stencil, GK.bit1_GC, d->mask,
+					0, y_src, 0, 0, d->w, d->h);
+		else	/* Fill decal extent with white	*/
+			gdk_draw_rectangle(d->stencil, GK.bit1_GC,
+					TRUE, 0, 0, d->w, d->h);
+		d->modified = TRUE;
+		}
+	d->value = index;
+	}
+
+void
+gkrellm_draw_decal_text(Panel *p, Decal *d, gchar *s, gint value)
+	{
+	TextStyle	*ts;
+	gint		x, y;
+
+	if (!s || !d || d->state == DS_INVISIBLE)
+		return;
+	if (d->value != value || value == -1)
+		{
+		ts = &d->text_style;
+		x = d->x_off;
+		y = d->y_baseline;
+//		gdk_draw_pixmap(d->pixmap, GK.draw1_GC, p->bg_pixmap,
+//				d->x, d->y,  0, 0,  d->w, d->h);
+		gkrellm_draw_string(d->pixmap, ts, x, y, s);
+		gdk_draw_rectangle(d->stencil, GK.bit0_GC, TRUE, 0, 0, d->w, d->h);
+		gdk_draw_string(d->stencil, ts->font, GK.bit1_GC, x, y, s);
+		if (ts->effect)
+			gdk_draw_string(d->stencil, ts->font, GK.bit1_GC, x + 1, y + 1, s);
+		d->modified = TRUE;
+		}
+	d->value = value;
+	}
+
+void
+gkrellm_move_krell_yoff(Panel *p, Krell *k, gint y)
+	{
+	Draw_rec	*dr, dr_old;
+
+	if (!k)
+		return;
+	dr_old = k->draw;
+	k->y0 = k->draw.y_dst = y;
+	dr = &dr_old;
+	if (p && p->pixmap && p->bg_pixmap)
+		{
+		gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
+				dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
+		if (p->drawing_area->window)
+			gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC,
+					p->bg_pixmap,
+					dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
+		gkrellm_draw_panel_layers_force(p);
+		}
 	}
 
 void
@@ -369,6 +438,49 @@ gkrellm_destroy_krell(Krell *k)
 		}
 	}
 
+void
+gkrellm_remove_krell(Panel *p, Krell *k)
+	{
+	Draw_rec	*dr;
+
+	if (!p || !k)
+		return;
+	if (g_list_find(p->krell, k))
+		{
+		p->krell = g_list_remove(p->krell, k);
+		dr = &k->draw;
+		if (p->pixmap && p->bg_pixmap)
+			{
+			gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
+				dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
+			if (p->drawing_area->window)
+				gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC,
+					p->bg_pixmap,
+					dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
+			gkrellm_draw_panel_layers_force(p);
+			}
+		}
+	}
+
+void
+gkrellm_remove_and_destroy_krell(Panel *p, Krell *k)
+	{
+	gkrellm_remove_krell(p, k);
+	gkrellm_destroy_krell(k);
+	}
+
+void
+gkrellm_insert_krell(Panel *p, Krell *k, gboolean append)
+	{
+	if (!p || !k)
+		return;
+	if (append)
+		p->krell = g_list_append(p->krell, k);
+	else
+		p->krell = g_list_prepend(p->krell, k);
+	gkrellm_draw_panel_layers_force(p);
+	}
+
 void
 gkrellm_destroy_krell_list(Panel *p)
 	{
@@ -378,22 +490,65 @@ gkrellm_destroy_krell_list(Panel *p)
 		return;
 	for (list = p->krell; list; list = list->next)
 		gkrellm_destroy_krell((Krell *) list->data);
-	g_list_free(p->krell);
+	if (p->krell)
+		g_list_free(p->krell);
 	p->krell = NULL;
 	}
 
+void
+gkrellm_set_krell_margins(Panel *p, Krell *k, gint left, gint right)
+	{
+	Draw_rec	*dr;
+
+	if (!k)
+		return;
+	if (left > GK.chart_width - 3)
+		left = GK.chart_width - 3;
+	k->x0 = left;
+	k->w_scale  = GK.chart_width - k->x0 - right - 1;
+	if (k->w_scale < 2)
+		k->w_scale = 2;
+	k->modified = TRUE;
+
+	dr = &k->draw;
+	if (p && p->pixmap && p->bg_pixmap)
+		{
+		gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
+				dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
+		if (p->drawing_area->window)
+			gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC,
+					p->bg_pixmap,
+					dr->x_dst, dr->y_dst, dr->x_dst, dr->y_dst, dr->w, dr->h);
+
+		/* Move the krell proportionally between the new limits
+		*/
+		k->x_position = -1;
+		if (k->monotonic)
+			k->previous -= k->value;
+		gkrellm_update_krell(p, k, k->value);
+		gkrellm_draw_panel_layers_force(p);
+		}
+	}
+
+void
+gkrellm_set_krell_full_scale(Krell *k, gint full_scale, gint expand)
+	{
+	if (!k)
+		return;
+	k->full_scale_expand = (expand <= 0 ) ? 1 : expand;
+	k->full_scale = full_scale * k->full_scale_expand;
+	}
+	
 Krell *
 gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 	{
-	GtkWidget	*top_win;
+	GtkWidget	*top_win = gkrellm_get_top_window();
 	Krell		*k;
 	gint		w, h, map_x, w_render;
 
-	if (!p)
-		return NULL;
-	top_win = gkrellm_get_top_window();
 	k = (Krell *) g_new0(Krell, 1);
-	p->krell = g_list_append(p->krell, k);
+	if (p)
+		p->krell = g_list_append(p->krell, k);
 
 	if (im == NULL || style == NULL)
 		{
@@ -402,10 +557,10 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 		}
 	k->x0 = 0;
 	k->y0 = style->krell_yoff;
-	if (k->y0 < 0)
+	if (k->y0 == -1)
 		gkrellm_get_top_bottom_margins(style, &k->y0, NULL);
 
-	k->w_scale  = UC.chart_width - k->x0 - 1;
+	k->w_scale  = GK.chart_width - k->x0 - 1;
 
 	w = im->rgb_width;
 	h = im->rgb_height;
@@ -415,34 +570,27 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 	w_render = w;
 	k->x_hot = style->krell_x_hot;
 
-	if (GK.debug_level & DEBUG_KRELL)
-		printf(_("Krell: x0=%d y0=%d w_scale=%d hot=%d w=%d h=%d exp=%d"),
-			k->x0, k->y0, k->w_scale,
-			k->x_hot, w, h, style->krell_expand);
-
 	if (style->krell_expand)
 		{
 		switch (style->krell_expand)
 			{
-			case LEFT:
+			case KRELL_EXPAND_LEFT:
 				if (style->krell_x_hot > 0)
 					{
-					map_x = UC.chart_width;
+					map_x = GK.chart_width;
 					w_render = map_x * w / style->krell_x_hot;
 					k->x_hot = map_x;
 					}
 				break;
-			case RIGHT:
+			case KRELL_EXPAND_RIGHT:
 				if (w > style->krell_x_hot)
 					{
-					map_x = UC.chart_width;
+					map_x = GK.chart_width;
 					w_render = map_x * w / (w - style->krell_x_hot);
 					k->x_hot = style->krell_x_hot * w_render / w;
 					}
 				break;
 			}
-		if (GK.debug_level & DEBUG_KRELL)
-			printf(_(" => w=%d hot=%d"), w_render, k->x_hot);
 		}
 	else
 		k->x_hot = style->krell_x_hot;
@@ -459,21 +607,31 @@ gkrellm_create_krell(Panel *p, GdkImlibImage *im, Style *style)
 		k->depth = 1;
 	k->h_frame = h / k->depth;
 
-	k->h_stencil = h / k->depth;
-	k->stencil = gdk_pixmap_new(top_win->window,
-					UC.chart_width, k->h_stencil, 1);
+	if (k->h_frame == 0)	/* gkrellmrc error or old style theme */
+		{
+		k->h_frame = 1;
+		k->depth = h / k->h_frame;
+		}
+
+	k->stencil = gdk_pixmap_new(top_win->window, GK.chart_width, k->h_frame,1);
+
 	k->period = style->krell_ema_period;
-	if (k->period >= 4 * UC.update_HZ)
-		k->period = 4 * UC.update_HZ;
-	if (GK.debug_level & DEBUG_KRELL)
-		printf(_("\n  h_stencil=%d depth=%d, period=%d\n"),
-			k->h_stencil, k->depth, k->period);
+	if (k->period >= 4 * GK.update_HZ)
+		k->period = 4 * GK.update_HZ;
+
 	k->x_position = -1;      /* Force initial draw   */
 	k->full_scale_expand = 1;
-
+	k->monotonic = TRUE;
 	return k;
 	}
 
+void
+gkrellm_monotonic_krell_values(Krell *k, gboolean mono)
+	{
+	if (k)
+		k->monotonic = mono;
+	}
+
 /* -------------------------------------------------------------- */
 
 static GList	*button_list;
@@ -487,12 +645,46 @@ disconnect_panel_signals(Panel *p, DecalButton *b)
 	gtk_signal_disconnect(GTK_OBJECT(p->drawing_area), b->id_leave);
 	}
 
+void
+gkrellm_move_decal(Panel *p, Decal *d, gint x, gint y)
+	{
+	gint	x_old, y_old;
+
+	if (!d)
+		return;
+	x_old = d->x;
+	y_old = d->y;
+	d->x = x;
+	d->y = y;
+	if (p && p->pixmap && p->bg_pixmap)
+		{
+		gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
+					x_old, y_old,  x_old, y_old,   d->w, d->h);
+		if (p->drawing_area->window)
+			gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC,
+					p->bg_pixmap, x_old, y_old,  x_old, y_old,   d->w, d->h);
+		gkrellm_draw_panel_layers_force(p);
+		}
+	}
+
+void
+gkrellm_decal_on_top_layer(Decal *d, gboolean top)
+	{
+	if (!d)
+		return;
+	if (top)
+		d->flags |= DF_TOP_LAYER;
+	else
+		d->flags &= ~DF_TOP_LAYER;
+	d->modified = TRUE;
+	}
+
 void
 gkrellm_destroy_decal(Decal *d)
 	{
 	if (d)
 		{
-		if ((d->flags & DF_LOCALPIXMAPS) && d->pixmap)
+		if ((d->flags & DF_LOCAL_PIXMAPS) && d->pixmap)
 			gdk_pixmap_unref(d->pixmap);
 		if (d->stencil)
 			gdk_pixmap_unref(d->stencil);
@@ -500,6 +692,45 @@ gkrellm_destroy_decal(Decal *d)
 		}
 	}
 
+void
+gkrellm_remove_decal(Panel *p, Decal *d)
+	{
+	if (!p || !d)
+		return;
+	if (g_list_find(p->decal, d))
+		{
+		p->decal = g_list_remove(p->decal, d);
+		if (p->pixmap && p->bg_pixmap)
+			{
+			gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
+					d->x, d->y,  d->x, d->y,   d->w, d->h);
+			if (p->drawing_area->window)
+				gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC,
+						p->bg_pixmap, d->x, d->y,  d->x, d->y,   d->w, d->h);
+			gkrellm_draw_panel_layers_force(p);
+			}
+		}
+	}
+
+void
+gkrellm_remove_and_destroy_decal(Panel *p, Decal *d)
+	{
+	gkrellm_remove_decal(p, d);
+	gkrellm_destroy_decal(d);
+	}
+
+void
+gkrellm_insert_decal(Panel *p, Decal *d, gboolean append)
+	{
+	if (!p || !d)
+		return;
+	if (append)
+		p->decal = g_list_append(p->decal, d);
+	else
+		p->decal = g_list_prepend(p->decal, d);
+	gkrellm_draw_panel_layers_force(p);
+	}
+
 void
 gkrellm_destroy_decal_list(Panel *p)
 	{
@@ -520,7 +751,8 @@ gkrellm_destroy_decal_list(Panel *p)
 			}
 		gkrellm_destroy_decal(d);
 		}
-	g_list_free(p->decal);
+	if (p->decal)
+		g_list_free(p->decal);
 	p->decal = NULL;
 	}
 
@@ -531,20 +763,23 @@ gkrellm_create_decal_pixmap(Panel *p, GdkPixmap *pixmap, GdkBitmap *mask,
 	GtkWidget	*top_win;
 	Decal		*d;
 
-	if (!p || !pixmap)
+	if (!pixmap)
 		return NULL;
 	top_win = gkrellm_get_top_window();
 	d = (Decal *) g_new0(Decal, 1);
-	p->decal = g_list_append(p->decal, d);
+	if (p)
+		p->decal = g_list_append(p->decal, d);
 
 	d->w = ((GdkWindowPrivate *) pixmap)->width;
 	d->h = ((GdkWindowPrivate *) pixmap)->height / depth;
+	if (d->h == 0)
+		d->h = 1;
 
 	d->x = x;
 	if (d->x < 0 && style)
 		{
 		if (style->label_position < 50)
-			d->x = UC.chart_width - d->w - style->margin;
+			d->x = GK.chart_width - d->w - style->margin;
 		else
 			d->x = style->margin;
 		}
@@ -570,11 +805,12 @@ gkrellm_create_decal_text(Panel *p, gchar *string, TextStyle *ts,
 	Decal		*d;
 	gint		ll, rr, ww, aa, dd;
 
-	if (!p)
+	if (!ts || !string)
 		return NULL;
 	top_win = gkrellm_get_top_window();
 	d = (Decal *) g_new0(Decal, 1);
-	p->decal = g_list_append(p->decal, d);
+	if (p)
+		p->decal = g_list_append(p->decal, d);
 
 	d->text_style = *ts;
 	gdk_string_extents(d->text_style.font, string, &ll, &rr, &ww, &aa, &dd);
@@ -589,7 +825,7 @@ gkrellm_create_decal_text(Panel *p, gchar *string, TextStyle *ts,
 	if (style)
 		{
 		if (w < 0)
-			d->w = UC.chart_width - 2 * style->margin;
+			d->w = GK.chart_width - 2 * style->margin;
 		else if (w == 0)
 			d->w = ww + d->text_style.effect;
 		else
@@ -603,17 +839,23 @@ gkrellm_create_decal_text(Panel *p, gchar *string, TextStyle *ts,
 		}
 	else
 		{
-		d->w = w;
+		if (w < 0)
+			d->w = GK.chart_width;
+		else if (w == 0)
+			d->w = ww;
+		else
+			d->w = w;
 		d->x = x;
 		d->y = y;
 		}
-
+	if (d->w == 0)
+		d->w = 1;
 	d->pixmap = gdk_pixmap_new(top_win->window, d->w, d->h, -1);;
 	d->mask   = NULL;
 	d->stencil = gdk_pixmap_new(top_win->window, d->w, d->h, 1);
 
 	d->value = -1;		/* Force initial draw */
-	d->flags = DF_LOCALPIXMAPS;
+	d->flags = DF_LOCAL_PIXMAPS;
 	d->state = DS_VISIBLE;
 
 	return d;
@@ -626,7 +868,7 @@ gkrellm_make_decal_invisible(Panel *p, Decal *d)
 		return;
 	d->state = DS_ERASING;
 	d->modified = TRUE;
-	gkrellm_draw_layers(p);
+	gkrellm_draw_panel_layers(p);
 	d->state = DS_INVISIBLE;
 	}
 
@@ -637,7 +879,7 @@ gkrellm_make_decal_visible(Panel *p, Decal *d)
 		return;
 	d->state = DS_VISIBLE;
 	d->modified = TRUE;
-	gkrellm_draw_layers(p);
+	gkrellm_draw_panel_layers(p);
 	}
 
 gint
@@ -672,7 +914,7 @@ set_button_index(DecalButton *b, gint index, gint do_draw)
 	gkrellm_draw_decal_pixmap(b->p, b->d, index);
 	b->cur_index = index;
 	if (do_draw)
-		gkrellm_draw_layers(b->p);
+		gkrellm_draw_panel_layers(b->p);
 	}
 
   /* FIXME, button_press and button_release are called multiple times
@@ -744,7 +986,7 @@ cb_decal_button_leave(GtkWidget *widget, GdkEventButton *ev)
 				b->d->state = DS_ERASING;
 				set_button_index(b, 0, 0);
 				b->d->modified = TRUE;
-				gkrellm_draw_layers(b->p);
+				gkrellm_draw_panel_layers(b->p);
 				b->d->state = DS_INVISIBLE;
 				}
 			}
@@ -771,7 +1013,7 @@ cb_decal_button_enter(GtkWidget *widget, GdkEventButton *ev)
 		b->d->state = DS_VISIBLE;
 		set_button_index(b, 0, 0);
 		b->d->modified = TRUE;
-		gkrellm_draw_layers(b->p);
+		gkrellm_draw_panel_layers(b->p);
 		}
 	return TRUE;
 	}
@@ -823,7 +1065,7 @@ gkrellm_destroy_button(DecalButton *b)
   |  do any decal freeing here, just clean up the button list.
   */
 void
-destroy_decal_button_list()
+destroy_decal_button_list(void)
 	{
 	GList		*list;
 	Panel		*p;
@@ -975,7 +1217,7 @@ gkrellm_make_overlay_button(Panel *p, void (*func)(), void *data,
 		}
 
 	d = gkrellm_create_decal_pixmap(p, pixmap, mask, 2, NULL, x, y);
-	d->flags |= DF_LOCALPIXMAPS;
+	d->flags |= DF_LOCAL_PIXMAPS;
 	d->state = DS_INVISIBLE;
 
 	b = gkrellm_make_decal_button(p, d, func, data, 0, 1);
