@@ -19,6 +19,8 @@
 */
 
 /*
+| 6/7/2001 Patch from Jason Lunz <j@falooley.org>
+|	Added multimua option.
 | 2/6/2001 Patch from Laurent Caillat-Vallet <caillat@in2p3.fr>
 |	POP3/IMAP connections can be on non-standard port numbers.
 | 1/14/2001 Patch from Masaharu FUJITA <fujita@hpc.cs.ehime-u.ac.jp>
@@ -59,6 +61,8 @@
 
 #include "gkrellm_threads.h"
 
+#include	"pixmaps/mail/decal_mail.xpm"
+
 #if defined(__linux__)
 #if defined(__GLIBC__) && ((__GLIBC__>2)||(__GLIBC__==2 && __GLIBC_MINOR__>=1))
 #define HAVE_GETADDRINFO	1
@@ -73,6 +77,14 @@
 #define HAVE_GETADDRINFO	1
 #endif
 
+#if defined(__solaris__)
+/* getaddrinfo is related to IPv6 stuff */
+# include <netconfig.h>
+# if defined(NC_INET6)
+#  define HAVE_GETADDRINFO	1
+# endif
+#endif
+
 #define MUTE_FLAG	-1
 
 /* msg_count_mode has 3 states
@@ -133,6 +145,7 @@
 #define	MBOX_EXTERNAL_INLINE	(MBOX_CHECK_INLINE   | MBOX_EXTERNAL_FLAG)
 #define	MBOX_EXTERNAL_THREADED	(MBOX_CHECK_THREADED | MBOX_EXTERNAL_FLAG)
 
+static Monitor	*mon_mail;
 
 typedef struct
 	{
@@ -177,6 +190,7 @@ Mailbox			*mail_fetch;		/* Internal mailbox: fetch command */
 static Mailproc	mail_user_agent;
 static gchar	*mail_notify;		/* Sound		*/
 
+static GdkImlibImage *decal_mail_image;
 static GdkPixmap	*decal_mail_pixmap;
 static GdkBitmap	*decal_mail_mask;
 
@@ -192,12 +206,12 @@ static GtkTooltips	*tooltip;
 
 static DecalButton	*mua_button;
 
-static gboolean	mail_visible = FALSE;
 static gboolean	enable_mail,
 				mute_mode,
 				super_mute_mode,
 				cont_animation_mode,
 				mua_inhibit_mode,		/* Inhibit checking if MUA launched */
+				enable_multimua,		/* allow multiple MUA instances */
 				msg_count_mode,
 				fetch_check_only_mode,
 				reset_remote_mode;
@@ -280,7 +294,7 @@ format_remote_mbox_name(Mailbox *mbox, gchar *buf, size_t len)
   |  containing new mail.
   */
 static void
-update_tooltip()
+update_tooltip(void)
 	{
 	GList	*list;
 	Mailbox	*mbox;
@@ -547,7 +561,7 @@ static gboolean
 check_mail_pop3(Mailbox *mbox)
 	{
 	gint	fd	= -1;
-	gchar	line[128];
+	gchar	line[128], buf[128];
 #ifdef HAVE_GETADDRINFO
 	gint 	rv	= 0;
 	struct addrinfo	hints, *res, *res0;
@@ -571,6 +585,12 @@ check_mail_pop3(Mailbox *mbox)
 		if ((fd = socket(res->ai_family, res->ai_socktype,
 				res->ai_protocol)) < 0)
 			continue;
+		if (GK.debug_level & DEBUG_MAIL)
+			{
+			format_remote_mbox_name(mbox, buf, sizeof(buf));
+			printf("check_mail_pop3 (%d,%d,%d): connecting to %s\n",
+				res->ai_family, res->ai_socktype, res->ai_protocol, buf);
+			}
 		if (connect(fd, res->ai_addr, res->ai_addrlen) >= 0)
 			break;
 		close(fd);
@@ -593,6 +613,11 @@ check_mail_pop3(Mailbox *mbox)
 	s.sin_family = AF_INET;
 	s.sin_port = htons (mbox->port);
 
+	if (GK.debug_level & DEBUG_MAIL)
+		{
+		format_remote_mbox_name(mbox, buf, sizeof(buf));
+		printf("check_mail_pop3: connecting to %s\n", buf);
+		}
 	if (connect (fd, (struct sockaddr *)&s, sizeof (s)) < 0)
 		return tcp_shutdown(fd, mbox, tcp_error_message[0]);
 #endif
@@ -611,8 +636,6 @@ check_mail_pop3(Mailbox *mbox)
 	write(fd, line, strlen (line));
 	if (GK.debug_level & DEBUG_MAIL)
 		{
-		gchar	buf[128];
-
 		hide_password(mbox, line, 5);
 		format_remote_mbox_name(mbox, buf, sizeof(buf));
 		printf("server_command( %s ):%s", buf, line);
@@ -627,19 +650,21 @@ check_mail_pop3(Mailbox *mbox)
 	sscanf(mbox->tcp_in->str, "+OK %d", &mbox->mail_count);
    	snprintf (line, sizeof (line), "UIDL %d\r\n", mbox->mail_count);
 	server_command(mbox, fd, line);
+
 	if (! server_response(mbox, fd, "+OK"))
-		return tcp_shutdown(fd, mbox, tcp_error_message[5]);
+		mbox->new_mail_count = mbox->mail_count;
+	else
+		/* Set the new_mail_count only if the UIDL is changed to avoid
+		|  re-reporting mail is new after MUA button has been clicked.
+		*/
+		if (   sscanf(mbox->tcp_in->str, "+OK %*d %128s", line) == 1
+			&& gkrellm_dup_string((gchar **) (&mbox->private), line)
+		   )
+			mbox->new_mail_count = mbox->mail_count;
 
 	server_command(mbox, fd, "QUIT\r\n");
 	close (fd);
 
-	/* Set the new_mail_count only if the UIDL is changed to avoid
-	|  re-reporting mail is new after MUA button has been clicked.
-	*/
-	if (   sscanf(mbox->tcp_in->str, "+OK %*d %128s", line) == 1
-		&& gkrellm_dup_string((gchar **) (&mbox->private), line)
-	   )
-		mbox->new_mail_count = mbox->mail_count;
 	return TRUE;
 	}
  
@@ -650,6 +675,8 @@ check_mail_imap(Mailbox *mbox)
 	gint	messages	= 0;
 	gint	unseen		= 0;
 	gchar	line[128], *ss;
+	gchar	buf[128];
+
 
 #ifdef HAVE_GETADDRINFO
 	gchar	portnumber[6];
@@ -674,6 +701,12 @@ check_mail_imap(Mailbox *mbox)
 		if ((fd = socket(res->ai_family, res->ai_socktype,
 				res->ai_protocol)) < 0)
 			continue;
+		if (GK.debug_level & DEBUG_MAIL)
+			{
+			format_remote_mbox_name(mbox, buf, sizeof(buf));
+			printf("check_mail_imap (%d,%d,%d): connecting to %s\n",
+				res->ai_family, res->ai_socktype, res->ai_protocol, buf);
+			}
 		if (connect(fd, res->ai_addr, res->ai_addrlen) >= 0)
 			break;
 		close(fd);
@@ -696,6 +729,11 @@ check_mail_imap(Mailbox *mbox)
 	s.sin_family = AF_INET;
 	s.sin_port = htons (mbox->port);
 
+	if (GK.debug_level & DEBUG_MAIL)
+		{
+		format_remote_mbox_name(mbox, buf, sizeof(buf));
+		printf("check_mail_imap: connecting to %s\n", buf);
+		}
 	if (connect (fd, (struct sockaddr *)&s, sizeof (s)) < 0)
 		return tcp_shutdown(fd, mbox, tcp_error_message[0]);
 #endif
@@ -710,8 +748,6 @@ check_mail_imap(Mailbox *mbox)
 	write(fd, line, strlen (line));
 	if (GK.debug_level & DEBUG_MAIL)
 		{
-		gchar	buf[128];
-
 		hide_password(mbox, line, strlen(line) - strlen(mbox->password) - 2);
 		format_remote_mbox_name(mbox, buf, sizeof(buf));
 		printf("server_command( %s ):%s", buf, line);
@@ -1072,7 +1108,7 @@ draw_mail_text_decal(gint new_mail_count, gint mail_count)
 		w = gdk_string_width(ts.font, buf);
 		}
 	style = gkrellm_meter_style(style_id);
-	x = UC.chart_width * p->label->position / LABEL_MAX;
+	x = gkrellm_chart_width() * p->label->position / LABEL_MAX;
 	x -= style->margin + w / 2;
 	if (p->label->position >= 50)
 		x -= mail_icon_decal->w;
@@ -1095,7 +1131,7 @@ mbox_set_animation_state(Mailbox *mbox)
 	}
 
 static void
-update_krell_animation_frame()
+update_krell_animation_frame(void)
 	{
 	/* Run the animation.  Just go back and forth with a pause on
 	|  frames 1 and full_scale - 1 (frames 0 and full_scale are cut off).
@@ -1131,7 +1167,7 @@ pipe_command(Mailproc *mp)
 					mp->command);
 		return;
 		}
-	if (mp->command == NULL || *(mp->command) == '\0')
+	if (!mp->command || *mp->command == '\0')
 		return;
 	snprintf(buf, sizeof(buf), "%s 2>&1", mp->command);
 	if (GK.debug_level & DEBUG_MAIL)
@@ -1195,7 +1231,7 @@ fgets_pipe(gchar *line, gint len, Mailproc *mp)
 	}
 
 static gboolean
-mua_is_launched()
+mua_is_launched(void)
 	{
 	gchar	buf[128];
 	gint	n;
@@ -1378,7 +1414,8 @@ make_fetch_tooltip(gchar *line, gint msg, gint seen)
 						mbox->folder, mbox->username, mbox->server);
 		mbox->mail_count = msg;
 		mbox->new_mail_count = msg - seen;
-		g_list_insert_sorted(mailbox_list, mbox, (GCompareFunc)(compare_mailboxes));
+		mailbox_list = g_list_insert_sorted(mailbox_list, mbox,
+								(GCompareFunc)(compare_mailboxes));
 		}
 	}
 
@@ -1389,7 +1426,7 @@ make_fetch_tooltip(gchar *line, gint msg, gint seen)
   |  is presumably being downloaded into local mailboxes.
   */
 static void
-read_mail_fetch()
+read_mail_fetch(void)
 	{
 	Mailproc	*mp		= (Mailproc *) mail_fetch->private;
 	gchar		buf[128];
@@ -1435,7 +1472,7 @@ read_mail_fetch()
 	}
 
 static void
-reset_mail_fetch()
+reset_mail_fetch(void)
 	{
 	Mailproc	*mp	= (Mailproc *) mail_fetch->private;
 
@@ -1454,7 +1491,7 @@ reset_mail_fetch()
 	}
 
 static gboolean
-run_fetch_program()
+run_fetch_program(void)
 	{
 	Mailbox		*mbox;
 	Mailproc	*mp	= (Mailproc *) mail_fetch->private;
@@ -1630,7 +1667,7 @@ update_mail(void)
 		   )
 			{
 			snprintf(cmd, sizeof(cmd), "%s &", mail_notify);
-			system(cmd);
+			gkrellm_system(cmd);
 			if (any_busy)
 				sound_inhibit = TRUE;
 			}
@@ -1670,7 +1707,7 @@ update_mail(void)
 	*/
 	KRELL(mail)->previous = 0;
 	gkrellm_update_krell(mail, KRELL(mail), anim_frame);
-	gkrellm_draw_layers(mail);
+	gkrellm_draw_panel_layers(mail);
 	}
 
 
@@ -1688,7 +1725,7 @@ mail_expose_event(GtkWidget *widget, GdkEventExpose *ev)
   |  remote mail counts.  So this routine decides if it should be enabled.
   */
 static void
-set_mua_button_sensitivity()
+set_mua_button_sensitivity(void)
 	{
 	if (   *(mail_user_agent.command) || reset_remote_mode
 		|| (animation_select_mode && !cont_animation_mode)
@@ -1735,7 +1772,18 @@ cb_mail_button(DecalButton *button)
 			}
 		run_animation = FALSE;
 		}
-	if (mua_is_launched() == FALSE)
+	if (enable_multimua)
+		{
+		if (mail_user_agent.command && *mail_user_agent.command != '\0')
+			{
+			gchar   *cmd;
+
+			cmd = g_strdup_printf("%s &", mail_user_agent.command);
+			gkrellm_system(cmd);
+			g_free(cmd);
+			}
+		}
+	else if (!mua_is_launched()) 
 		pipe_command(&mail_user_agent);
 	else
 		{
@@ -1753,11 +1801,14 @@ cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
 	{
 	Decal	*d 	= mail_icon_decal;
 
-	/* button 2 or 3 anywhere in the panel toggles the mute mode.
+	if (ev->button == 3)
+		gkrellm_open_config_window(mon_mail);
+
+	/* button 2 anywhere in the panel toggles the mute mode.
 	*/
-	if (ev->button == 2 || ev->button == 3)
+	if (ev->button == 2)
 		{
-		mute_mode = 1 - mute_mode;
+		mute_mode = !mute_mode;
 
 		/* If coming out of mute mode and mail checking was inhibited,
 		|  force a check.
@@ -1788,7 +1839,8 @@ add_mailbox(gchar *arg)
 		{
 		case 1:
 			if (parms[0][0] == '~')
-				mbox->path = g_strdup_printf("%s%s", homedir(), parms[0] + 1);
+				mbox->path = g_strdup_printf("%s%s", gkrellm_homedir(),
+							parms[0] + 1);
 			else
 				mbox->path = g_strdup(parms[0]);
 			if (isdir(mbox->path, "new"))
@@ -1890,14 +1942,9 @@ create_mail(GtkWidget *vbox, gint first_create)
 			style);
 	KRELL(p)->full_scale = style->krell_depth - 1;
 
-	/* But for 0.9.x theme compatibility, I should #include decal_mail.xpm
-	|  and use it here.  But has to be done in config.c for now.
-	|  This will grab new theme style net/decal_mail.png images if
-	|  present and reload decal_mail_image.
-	*/
-	gkrellm_load_image("decal_mail", NULL, &GK.decal_mail_image,
+	gkrellm_load_image("decal_mail", decal_mail_xpm, &decal_mail_image,
 			MAIL_STYLE_NAME);
-	gkrellm_render_to_pixmap(GK.decal_mail_image, &decal_mail_pixmap,
+	gkrellm_render_to_pixmap(decal_mail_image, &decal_mail_pixmap,
 			&decal_mail_mask, 0, 0);
 	mail_icon_decal = gkrellm_create_decal_pixmap(p, decal_mail_pixmap,
 				decal_mail_mask, GK.decal_mail_frames, style, -1, -1);
@@ -1909,15 +1956,14 @@ create_mail(GtkWidget *vbox, gint first_create)
 	x = style->margin;
 	if (style->label_position >= 50)
 		x += mail_icon_decal->w;
-	w = UC.chart_width - mail_icon_decal->w - 2 * style->margin;
+	w = gkrellm_chart_width() - mail_icon_decal->w - 2 * style->margin;
 
-	p->textstyle = gkrellm_meter_textstyle(style_id);
-
-    mail_text_decal = gkrellm_create_decal_text(p, "0", p->textstyle,
+    mail_text_decal = gkrellm_create_decal_text(p, "0",
+				gkrellm_meter_textstyle(style_id),
                 style, x, -1, w);    /* -1 means use y default */
 
-	gkrellm_configure_panel(p, NULL, style);
-	gkrellm_create_panel(vbox, p, gkrellm_bg_meter_image(style_id));
+	gkrellm_panel_configure(p, NULL, style);
+	gkrellm_panel_create(vbox, mon_mail, p);
 
 	/* Center the decals with respect to each other.
 	*/
@@ -1930,13 +1976,8 @@ create_mail(GtkWidget *vbox, gint first_create)
 			cb_mail_button, NULL);
 	set_mua_button_sensitivity();
 
-	if(enable_mail)
-		{
-		gkrellm_monitor_height_adjust(p->h);
-		mail_visible = TRUE;
-		}
-	else
-		gtk_widget_hide(p->hbox);
+	if(!enable_mail)
+		gkrellm_panel_hide(p);
 
 	if (first_create)
 		{
@@ -1990,8 +2031,8 @@ save_mail_config(FILE *f)
 	fprintf(f, "mail animation_select_mode %d\n", animation_select_mode);
 	fprintf(f, "mail fetch_check_only_mode %d\n", fetch_check_only_mode);
 	fprintf(f, "mail reset_remote_mode %d\n", reset_remote_mode);
-	fprintf(f, "mail enable %d %d %d\n", enable_mail, super_mute_mode,
-						mua_inhibit_mode);
+	fprintf(f, "mail enable %d %d %d %d\n", enable_mail, super_mute_mode,
+						mua_inhibit_mode, enable_multimua);
 	fprintf(f, "mail animation_continuous %d\n", cont_animation_mode);
 	fprintf(f, "mail show_tooltip %d\n", show_tooltip);
 	fprintf(f, "mail mh_seq_ignore %d\n", mh_seq_ignore);
@@ -2029,8 +2070,8 @@ load_mail_config(gchar *arg)
 		else if (strcmp(mail_config, "reset_remote_mode") == 0)
 			sscanf(item, "%d", &reset_remote_mode);
 		else if (strcmp(mail_config, "enable") == 0)
-			sscanf(item, "%d %d %d", &enable_mail, &super_mute_mode,
-						&mua_inhibit_mode);
+			sscanf(item, "%d %d %d %d", &enable_mail, &super_mute_mode,
+						&mua_inhibit_mode, &enable_multimua);
 		else if (strcmp(mail_config, "animation_continuous") == 0)
 			sscanf(item, "%d", &cont_animation_mode);
 		else if (strcmp(mail_config, "show_tooltip") == 0)
@@ -2052,10 +2093,11 @@ load_mail_config(gchar *arg)
 static GtkWidget		*mbox_path_entry;
 
 static GtkWidget		*mail_user_agent_entry;
+static GtkWidget		*enable_multimua_button;
 static GtkWidget		*mail_fetch_entry;
 static GtkWidget		*mail_notify_entry;
 static GtkWidget		*enable_mail_button;
-static GtkWidget		*enabel_cont_anim_button;
+static GtkWidget		*enable_cont_anim_button;
 static GtkWidget		*super_mute_button;
 static GtkWidget		*mbox_clist;
 static GtkWidget		*mail_fetch_spin_button;
@@ -2093,7 +2135,7 @@ cb_mailbox_unselected(GtkWidget *clist, gint row, gint column,
 	}
 
 static void
-cb_enter_mailbox()
+cb_enter_mailbox(void)
 	{
 	gchar			*buf[2], *s;
 
@@ -2143,7 +2185,7 @@ cb_delete_mailbox(GtkWidget *widget, gpointer *data)
 
 #if 0
 static void
-dump_mailboxes()
+dump_mailboxes(void)
 	{
 	Mailbox	*mbox;
 	GList	*list;
@@ -2160,7 +2202,7 @@ dump_mailboxes()
 #endif
 
 static void 
-apply_mail_config()
+apply_mail_config(void)
 	{
 	Mailbox			*mbox;
 	Mailproc		*mp		= (Mailproc *) mail_fetch->private;
@@ -2170,10 +2212,14 @@ apply_mail_config()
 	GtkSpinButton	*spin;
 
 	super_mute_mode = GTK_TOGGLE_BUTTON(super_mute_button)->active;
+	enable_multimua = GTK_TOGGLE_BUTTON(enable_multimua_button)->active;
+	cont_animation_mode = GTK_TOGGLE_BUTTON(enable_cont_anim_button)->active;
 	enable_mail = GTK_TOGGLE_BUTTON(enable_mail_button)->active;
-	cont_animation_mode = GTK_TOGGLE_BUTTON(enabel_cont_anim_button)->active;
-	gkrellm_enable_visibility(enable_mail, &mail_visible,
-					mail->hbox, mail->h);
+	if (enable_mail)
+		gkrellm_panel_show(mail);
+	else
+		gkrellm_panel_hide(mail);
+
 	for (i = 0; i < 3; ++i)
 		if (GTK_TOGGLE_BUTTON(msg_count_button[i])->active)
 			msg_count_mode = i;
@@ -2265,13 +2311,13 @@ N_(" tab is (in the same order as listed above):\n"),
 N_("<i>\tfilename\n"),
 N_("<i>\tdirectory\n"),
 N_("<i>\tdirectory\n"),
-N_("<i>\tmailserver   username   password\n"),
-N_("<i>\tmailserver   mailbox   username   password\n"),
+N_("<i>\tmailserver  username  password\n"),
+N_("<i>\tmailserver  mailbox  username  password\n"),
 
 N_("\nSo, a simple example for monitoring your local mbox and a POP3 mailbox:\n"
 ),
 N_("<i>\t/var/mail/bill\n"),
-N_("<i>\tmail.gnu.org   bill2   PASSWORD\n"),
+N_("<i>\tmail.gnu.org  bill2  PASSWORD\n"),
 
 #else
 N_("Mailbox files to monitor can be mbox or maildir style mailboxes.\n"),
@@ -2281,7 +2327,9 @@ N_("<b>\n\nSetup: Mail reading program\n"),
 N_("If you enter a mail reading program (your mail user agent or MUA)\n"
 "it can be launched by clicking on the mail monitor message count button.\n"
 "For example you could enter:\n"),
-N_("<i>\tcscmail\n"),
+N_("<i>\tsylpheed\n"),
+N_("or\n"),
+N_("<i>\tpronto\n"),
 N_("or\n"),
 N_("<i>\txterm -e mutt\n"),
 
@@ -2326,7 +2374,7 @@ N_("\t\tIf options permit, also stop animations and reset remote counts.\n"),
 N_("<b>\tLeft "),
 N_("click the envelope decal to force a mail check regardless of\n"
 "\t\tany mute mode or inhibit mail checking option settings.\n"),
-N_("<b>\tMiddle or Right "),
+N_("<b>\tMiddle "),
 N_("click the mail panel to toggle a mute mode which inhibits\n"
 	"\t\tthe sound notify program and optionally inhibits all mail\n"
 	"\t\tchecking.\n"),
@@ -2340,9 +2388,7 @@ create_mail_tab(GtkWidget *tab_vbox)
 	{
 	GtkWidget		*tabs;
 	GtkWidget		*table;
-	GtkWidget		*frame;
-	GtkWidget		*vbox, *hbox;
-	GtkWidget		*separator;
+	GtkWidget		*vbox, *vbox1, *hbox;
 	GtkWidget		*label;
 	GtkWidget		*button;
 	GtkWidget		*scrolled;
@@ -2363,20 +2409,15 @@ create_mail_tab(GtkWidget *tab_vbox)
 	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);
 
 /* --Setup tab */
-	vbox = gkrellm_create_tab(tabs, _("Setup"));
+	vbox = gkrellm_create_framed_tab(tabs, _("Setup"));
 
-	enable_mail_button = gtk_check_button_new_with_label(
-				_("Enable Mailcheck"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable_mail_button),
-				enable_mail);
-	gtk_box_pack_start(GTK_BOX(vbox), enable_mail_button, TRUE, TRUE, 0);
-
-	separator = gtk_hseparator_new();
-	gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 0);
+	gkrellm_check_button(vbox, &enable_mail_button, enable_mail,
+		FALSE, 10, _("Enable Mailcheck"));
 
+	vbox1 = gkrellm_framed_vbox_end(vbox, NULL, 4, FALSE, 0, 2);
 	table = gtk_table_new(5, 3, TRUE /*homogeneous*/);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 2);
-	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 2);
+	gtk_box_pack_start(GTK_BOX(vbox1), table, FALSE, FALSE, 2);
 
 	hbox = gtk_hbox_new(TRUE, 0);
 	/* Attach left right top bottom */
@@ -2414,52 +2455,43 @@ create_mail_tab(GtkWidget *tab_vbox)
 	label = gtk_label_new(_("Check local mailboxes every"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 4);
-	hbox = gtk_hbox_new(TRUE, 0);
+	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 3, 3, 4);
 	gkrellm_spin_button(hbox, &mail_check_spin_button,
 			(gfloat) mail_check_timeout, 2.0, 100.0, 1.0, 5.0, 0, 60,
 			NULL, NULL, FALSE, _("seconds"));
 
-	hbox = gtk_hbox_new(TRUE, 0);
+	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 0, 1, 4, 5);
 	label = gtk_label_new(_("Do remote checks every"));
 	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
 	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 4);
-	hbox = gtk_hbox_new(TRUE, 0);
+	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_table_attach_defaults(GTK_TABLE(table), hbox, 1, 3, 4, 5);
 	gkrellm_spin_button(hbox, &mail_fetch_spin_button,
 			(gfloat) mail_fetch_timeout, 1.0, 60.0, 1.0, 5.0, 0, 60,
 			NULL, NULL, FALSE, _("minutes"));
 
-	label = gtk_label_new(
-_("Note: remote check interval applies to the \"Mail fetch/check program\"\n"
-	"as well as to builtin remote POP3 and IMAP checking - see Info page"));
-	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
-	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 4);
-
-
 /* --Mailboxes tab */
-	vbox = gkrellm_create_tab(tabs, _("Mailboxes"));
+	vbox = gkrellm_create_framed_tab(tabs, _("Mailboxes"));
+
+	vbox1 = gkrellm_framed_vbox(vbox, NULL, 4, FALSE, 0, 2);
 	mbox_path_entry = gtk_entry_new_with_max_length(255);
-	gtk_box_pack_start(GTK_BOX(vbox), mbox_path_entry, FALSE, FALSE, 2);
+	gtk_box_pack_start(GTK_BOX(vbox1), mbox_path_entry, FALSE, FALSE, 2);
 	gtk_entry_set_text(GTK_ENTRY(mbox_path_entry), "");
 	gtk_signal_connect (GTK_OBJECT (mbox_path_entry), "activate",
 			(GtkSignalFunc) cb_enter_mailbox, NULL);			
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-	label = gtk_label_new("");
-	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 5);
-	button = gtk_button_new_with_label(_("Enter"));
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			(GtkSignalFunc) cb_enter_mailbox, NULL);
-	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 5);
-	button = gtk_button_new_with_label(_("Delete"));
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			(GtkSignalFunc) cb_delete_mailbox, NULL);
-	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 5);
 
-	separator = gtk_hseparator_new();
-	gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 10);
+	hbox = gtk_hbutton_box_new();
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
+	gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 5);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+
+	gkrellm_button_connected(hbox, &button, FALSE, FALSE, 5,
+			cb_enter_mailbox, NULL, _("Enter"));
+
+	gkrellm_button_connected(hbox, &button, FALSE, FALSE, 5,
+			cb_delete_mailbox, NULL, _("Delete"));
 
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
@@ -2503,16 +2535,16 @@ _("Note: remote check interval applies to the \"Mail fetch/check program\"\n"
 			g_free(hidden_passwd);
 		}
 
-/* --Options tab */
-	vbox = gkrellm_create_tab(tabs, _("Options"));
+/* --Animation tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Animation"));
+	vbox1 = gkrellm_framed_vbox(vbox, _("Animation Select"),
+			4, FALSE, 0, 2);
 
-	frame = gtk_frame_new (_("Animation Select"));
-	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 2);
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(frame), hbox);
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
 
 	button = gtk_radio_button_new_with_label(NULL, _("None"));
-	gtk_box_pack_start(GTK_BOX (hbox), button, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
 	group = gtk_radio_button_group(GTK_RADIO_BUTTON (button));
 	animation_select_button[0] = button;
 	button = gtk_radio_button_new_with_label(group, _("Envelope"));
@@ -2533,10 +2565,19 @@ _("Note: remote check interval applies to the \"Mail fetch/check program\"\n"
 	button = animation_select_button[animation_select_mode];
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
-	frame = gtk_frame_new (_("Message Counting"));
-	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 2);
+    gkrellm_check_button(vbox, &enable_cont_anim_button, cont_animation_mode,
+		FALSE, 10,
+		_("Run animation continuously as long as there is a new mail count"));
+
+
+/* --Options tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Options"));
+
+	vbox1 = gkrellm_framed_vbox(vbox, _("Message Counting"),
+			4, FALSE, 0, 2);
 	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(frame), hbox);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+
 	button = gtk_radio_button_new_with_label(NULL, _("new/total"));
 	gtk_box_pack_start(GTK_BOX (hbox), button, TRUE, TRUE, 0);
 	group = gtk_radio_button_group(GTK_RADIO_BUTTON (button));
@@ -2551,10 +2592,6 @@ _("Note: remote check interval applies to the \"Mail fetch/check program\"\n"
 	button = msg_count_button[msg_count_mode];
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
 
-    gkrellm_check_button(vbox, &enabel_cont_anim_button, cont_animation_mode,
-		TRUE, 0,
-		_("Run animation continuously as long as there is a new mail count"));
-
     gkrellm_check_button(vbox, &check_only_button, fetch_check_only_mode,
 		TRUE, 0,
 		_("Fetch/check program checks messages only (see Mail Info tab)"));
@@ -2568,6 +2605,9 @@ _("Note: remote check interval applies to the \"Mail fetch/check program\"\n"
     gkrellm_check_button(vbox, &mua_inhibit_button, mua_inhibit_mode, TRUE, 0,
 		_("Inhibit all mail checking while the mail reader is running"));
 
+	gkrellm_check_button(vbox, &enable_multimua_button, enable_multimua,
+		TRUE, 0, _("Allow multiple launches of the mail reader program"));
+
     gkrellm_check_button(vbox, &show_tooltip_button, show_tooltip, TRUE, 0,
 		_("List mailboxes containing new mail in a tooltip"));
 
@@ -2576,16 +2616,11 @@ _("Note: remote check interval applies to the \"Mail fetch/check program\"\n"
 			TRUE, 0, _("Ignore .mh_sequences when checking MH mail."));
 	
 /* --Info tab */
-	vbox = gkrellm_create_tab(tabs, _("Info"));
-	scrolled = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
-			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
-	text = gtk_text_new(NULL, NULL);
+	vbox = gkrellm_create_framed_tab(tabs, _("Info"));
+	text = gkrellm_scrolled_text(vbox, NULL,
+				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	for (i = 0; i < sizeof(mail_info_text)/sizeof(gchar *); ++i)
 		gkrellm_add_info_text_string(text, _(mail_info_text[i]));
-	gtk_text_set_editable(GTK_TEXT(text), FALSE);
-	gtk_container_add(GTK_CONTAINER(scrolled), text);
 	}
 
 
@@ -2619,9 +2654,12 @@ init_mail_monitor(void)
 	{
 	monitor_mail.name = _(monitor_mail.name);
 	enable_mail = TRUE;
+	enable_multimua = FALSE;
 	cont_animation_mode = FALSE;
 	super_mute_mode = FALSE;
 	mua_inhibit_mode = FALSE;
+	GK.decal_mail_frames = 18;
+	GK.decal_mail_delay = 1;
 
 	mail_fetch = g_new0(Mailbox, 1);
 	mail_fetch->private = g_new0(Mailproc, 1);
@@ -2635,6 +2673,8 @@ init_mail_monitor(void)
 
 	style_id = gkrellm_add_meter_style(&monitor_mail, MAIL_STYLE_NAME);
 	force_mail_check = TRUE;	/* Do a check at startup */
+	mon_mail = &monitor_mail;
+
 	return &monitor_mail;
 	}
 
@@ -2642,7 +2682,7 @@ init_mail_monitor(void)
   /* ====================== Exported Mail Functions =====================*/
 
 gboolean
-gkrellm_get_mail_mute_mode()
+gkrellm_get_mail_mute_mode(void)
 	{
 	return mute_mode;
 	}
@@ -2739,4 +2779,4 @@ gkrellm_destroy_external_mbox(gpointer mbox_ptr)
 	free_mailbox(mbox_ptr);
 	}
 
-  /* ====================================================================*/
+  /* =======================================================================*/
