@@ -40,8 +40,32 @@ char	errbuf[_POSIX2_LINE_MAX];
 extern void scan_for_sensors();
 #endif
 
+#if defined(__solaris__)
+#include <kstat.h>
+#include <kvm.h>
+#include <fcntl.h>
+
+kstat_ctl_t *kc;
+kvm_t *kd = NULL;
+
+struct nlist nl[] = { 
+    { "mpid" },
+    { 0 }
+};
+
+extern void solaris_list_harddisks(void);
+#endif /* __solaris__ */ 
+
+
+/* Duplicate some constant values from gnome-winhints.h and use them
+|  in X lib or gdk property calls.
+*/
+/* flags for the window layer */
+#define	WIN_LAYER_NORMAL	4
+#define	WIN_LAYER_ONTOP		6
+
+
 struct ThemeConfig	GK;
-struct UserConfig	UC;
 
 struct
 	{
@@ -83,31 +107,28 @@ struct
 
 GList			*gkrellm_monitor_list;
 GtkWidget		*top_window;				/* Make static at 1.1.0 */
+gint			w_display, h_display;
+gint			window_decorations;
+time_t			time_now;
 
-static GList	*chart_list;
-static GList	*panel_list;
 
 static GtkItemFactory *item_factory;
 
 static Pixmap	root_xpixmap	= None;
 static GdkGC	*trans_gc;
 
-gint			w_display, h_display;
-gint			window_decorations;
 static gchar	*geometry;
 
 static gint		x_position, y_position;
 
 static gint		withdrawn;
 static gint		total_frame_height;
-static gint		initialized;
 static gint		monitor_previous_height;
 static gint		monitors_visible	= TRUE;
 static gint		mask_monitors_visible = TRUE;
 static gint		check_rootpixmap_transparency,
 				check_frame_transparency;
 static gint		no_transparency;
-static gint		any_transparency;
 
 static void		apply_rootpixmap_transparency(void);
 static void		apply_frame_transparency(void);
@@ -131,45 +152,52 @@ static gchar	*fail_small_font[2] =
 };
 
 
+
 static void
 setup_fonts()
 	{
+	GdkFont	*(*font_load_func)();
 	GdkFont	*font;
 	gint	i;
 
-	if ((font = gdk_font_load(GK.large_font_string)) == NULL)
+	if (GK.use_fontset)
+		font_load_func = gdk_fontset_load;
+	else
+		font_load_func = gdk_font_load;
+
+	if ((font = (*font_load_func)(GK.large_font_string)) == NULL)
 		{
 		if (GK.debug_level & DEBUG_GKRELLMRC)
 			printf(_("Could not load large font\n  \"%s\"\nFalling back...\n"),
 						GK.large_font_string);
 		for (i = 0; i < 2; ++i)
-			if ((font = gdk_font_load(fail_large_font[i])) != NULL)
+			if ((font = (*font_load_func)(fail_large_font[i])) != NULL)
 				break;
 		}
 	if (GK.large_font && ! gdk_font_equal(GK.large_font, font))
 		gdk_font_unref(GK.large_font);
 	GK.large_font = font;
 
-	if ((font = gdk_font_load(GK.normal_font_string)) == NULL)
+	if ((font = (*font_load_func)(GK.normal_font_string)) == NULL)
 		{
 		if (GK.debug_level & DEBUG_GKRELLMRC)
 			printf(_("Could not load label font\n  \"%s\"\nFalling back...\n"),
 						GK.normal_font_string);
 		for (i = 0; i < 2; ++i)
-			if ((font = gdk_font_load(fail_normal_font[i])) != NULL)
+			if ((font = (*font_load_func)(fail_normal_font[i])) != NULL)
 				break;
 		}
 	if (GK.normal_font && ! gdk_font_equal(GK.normal_font, font))
 		gdk_font_unref(GK.normal_font);
 	GK.normal_font = font;
 
-	if ((font = gdk_font_load(GK.small_font_string)) == NULL)
+	if ((font = (*font_load_func)(GK.small_font_string)) == NULL)
 		{
 		if (GK.debug_level & DEBUG_GKRELLMRC)
 			printf(_("Could not load alt font\n  \"%s\"\nFalling back...\n"),
 							GK.small_font_string);
 		for (i = 0; i < 2; ++i)
-			if ((font = gdk_font_load(fail_small_font[i])) != NULL)
+			if ((font = (*font_load_func)(fail_small_font[i])) != NULL)
 				break;
 		}
 	if (GK.small_font && ! gdk_font_equal(GK.small_font, font))
@@ -192,6 +220,8 @@ map_color_string(gchar *color_string, GdkColor *color)
 		colormap = gtk_widget_get_colormap(top_window);
 	if (color->red || color->green || color->blue)
 		gdk_colormap_free_colors(colormap, color, 1);
+	if (!color_string)
+		color_string = "black";
 	gdk_color_parse(color_string, color);
 	gdk_color_alloc(colormap, color);
 	}
@@ -258,9 +288,6 @@ setup_colors()
 
 
 
-#define	WIN_LAYER_NORMAL	4
-#define	WIN_LAYER_ONTOP		6
-
 static Atom	xa_check;
 static Atom	xa_layer;
 
@@ -377,16 +404,45 @@ update_monitors()
 	{
 	GList			*list;
 	Monitor			*mon;
+	struct tm		*pCur, *pPrev;
 	static time_t	time_prev;
 
 	time(&time_now);
 	GK.second_tick = (time_now == time_prev) ? FALSE : TRUE;
 	time_prev = time_now;
 
+	if (GK.second_tick)
+		{
+		pPrev = &gkrellm_current_tm;
+		pCur = localtime(&time_now);
+		GK.two_second_tick  = ((pCur->tm_sec % 2) == 0) ? TRUE : FALSE;
+		GK.five_second_tick = ((pCur->tm_sec % 5) == 0) ? TRUE : FALSE;
+		GK.ten_second_tick  = ((pCur->tm_sec % 10) == 0) ? TRUE : FALSE;
+		GK.minute_tick = (pCur->tm_min  != pPrev->tm_min)  ? TRUE : FALSE;
+		GK.hour_tick   = (pCur->tm_hour != pPrev->tm_hour) ? TRUE : FALSE;
+		GK.day_tick    = (pCur->tm_mday != pPrev->tm_mday) ? TRUE : FALSE;
+
+		/* Copy localtime() data to my global struct tm data so clock (or
+		|  anybody) has access to current struct tm data.  Must copy so
+		|  data is not munged by plugins which might call localtime().
+		*/
+		gkrellm_current_tm = *pCur;
+		}
+	else
+		{
+		GK.two_second_tick = FALSE;
+		GK.five_second_tick = FALSE;
+		GK.ten_second_tick = FALSE;
+		GK.minute_tick = FALSE;
+		GK.hour_tick = FALSE;
+		GK.day_tick = FALSE;
+		}
+
+	update_alerts();
 	for (list = gkrellm_monitor_list; list; list = list->next)
 		{
 		mon = (Monitor *) list->data;
-		if (mon->update_monitor && mon->private->enabled)
+		if (mon->update_monitor && mon->privat->enabled)
 			(*(mon->update_monitor))();
 		}
 	++GK.timer_ticks;
@@ -394,18 +450,18 @@ update_monitors()
 	if (save_position_countdown > 0 && --save_position_countdown == 0)
 		set_or_save_position(1);
 
-	if (check_rootpixmap_transparency)
-		apply_rootpixmap_transparency();
+	if (check_rootpixmap_transparency > 0)
+		if (--check_rootpixmap_transparency == 0)
+			apply_rootpixmap_transparency();
 	if (check_frame_transparency)
 		{
 		/* Force a redraw of the mask */
-		mask_monitors_visible = 1 - monitors_visible;
+		mask_monitors_visible = !monitors_visible;
 		apply_frame_transparency();
 		}
-	check_rootpixmap_transparency = FALSE;
 	check_frame_transparency = FALSE;
 
-	return TRUE;			/* This restarts timeout */
+	return TRUE;			/* restarts timeout */
 	}
 
 void
@@ -483,75 +539,6 @@ gkrellm_get_top_bottom_margins(Style *style, gint *top, gint *bottom)
 		*bottom = b;
 	}
 
-  /* Calculate panel height required for given border, label height,
-  |  krell heights, and decal heights.  Also calculate label extents
-  |  and load all this info into a Label structure.
-  |  After panel height is calculated, calling routine may want to adjust
-  |  krell or decal y offsets for centering.  
-  */
-void
-gkrellm_configure_panel(Panel *p, gchar *string, Style *style)
-	{
-	GList			*list;
-	Krell			*k;
-	Decal			*d;
-	Label			*lbl;
-	gint			h_panel, h_decal;
-	gint			top_margin = 0, bottom_margin = 0;
-
-	if (p == NULL)
-		return;
-	p->style = style;
-	lbl = p->label;
-	lbl->string = string;
-	lbl->position = style ? style->label_position : LABEL_CENTER;
-	if (style)
-		{
-		p->transparency = style->transparency;
-		any_transparency |= p->transparency;
-		gkrellm_get_top_bottom_margins(style, &top_margin, &bottom_margin);
-		}
-	if (   lbl->string && p->textstyle && p->textstyle->font
-		&& lbl->position >= 0
-	   )
-		gdk_string_extents(p->textstyle->font, string, &lbl->lbearing,
-				&lbl->rbearing, &lbl->width, &lbl->ascent, &lbl->descent);
-	else
-		{
-		lbl->lbearing = 0;
-		lbl->rbearing = 0;
-		lbl->width = 0;
-		lbl->ascent = 0;
-		lbl->descent = 0;
-		}
-	h_panel = top_margin + bottom_margin;
-	h_panel += lbl->ascent + lbl->descent
-				+ (p->textstyle ? p->textstyle->effect : 0);
-
-	/* Krells have their own y_off and are not placed inside of margins.
-	|  Decals are assumed to fit inside of margins.  Caller must subtract
-	|  off bottom_margin if this is not so.
-	*/
-	for (list = p->krell; list; list = list->next)
-		{
-		k = (Krell *) list->data;
-		if (k->y0 + k->h_frame > h_panel)
-			h_panel = k->y0 + k->h_frame;
-		}
-	for (list = p->decal; list; list = list->next)
-		{
-		d = (Decal *) list->data;
-		h_decal = d->y + d->h + bottom_margin;
-		if (h_decal > h_panel)
-			h_panel = h_decal;
-		}
-
-	if (h_panel <= 0)
-		h_panel = 1;
-	lbl->y_baseline = top_margin + lbl->ascent;
-	lbl->h_panel = h_panel;
-	}
-
 void
 gkrellm_draw_string(GdkDrawable *drawable, TextStyle *ts, gint x, gint y,
 			gchar *s)
@@ -567,22 +554,19 @@ gkrellm_draw_string(GdkDrawable *drawable, TextStyle *ts, gint x, gint y,
 	gdk_draw_string(drawable, ts->font, GK.text_GC, x, y, s);
 	}
 
-gint
-gkrellm_draw_chart_label(Chart *cp, TextStyle *ts, gint x, gint y, gchar *s)
+void
+gkrellm_draw_text(GdkDrawable *drawable, TextStyle *ts, gint x, gint y,
+			gchar *s, gint len)
 	{
-	gint		w, l, r, a, d;
-
-	if (!cp || !ts || !s)
-		return x;
-	gdk_string_extents(ts->font, s, &l, &r, &w, &a, &d);
-
-	gdk_draw_pixmap(cp->pixmap, GK.draw1_GC,cp->bg_grided_pixmap,
-		x - l, y - a, x - l, y - a, l + r + ts->effect, a + d + ts->effect);
-    gkrellm_draw_string(cp->pixmap, ts, x, y, s);
-	gdk_draw_pixmap(cp->drawing_area->window, GK.draw1_GC, cp->pixmap,
-		x - l, y - a, x - l, y - a, l + r + ts->effect, a + d + ts->effect);
-
-	return x + r;
+	if (!drawable || !ts || !s)
+		return;
+	if (ts->effect)
+		{
+		gdk_gc_set_foreground(GK.text_GC, &ts->shadow_color);
+		gdk_draw_text(drawable, ts->font, GK.text_GC, x + 1, y + 1, s, len);
+		}
+	gdk_gc_set_foreground(GK.text_GC, &ts->color);
+	gdk_draw_text(drawable, ts->font, GK.text_GC, x, y, s, len);
 	}
 
 gint
@@ -615,27 +599,28 @@ draw_rootpixmap_onto_transparent_chart(Chart *cp)
 
 	/* First make the chart totally transparent
 	*/
-	gdk_draw_rectangle(cp->bg_pixmap, trans_gc, TRUE, 0, 0, cp->w, cp->h);
+	gdk_draw_rectangle(cp->bg_src_pixmap, trans_gc,
+					TRUE, 0, 0, cp->w, cp->h);
 
 	/* If mode permits, stencil on non transparent parts of bg_clean_pixmap.
 	*/
 	if (cp->transparency == 2 && cp->bg_mask)
 		{
         gdk_gc_set_clip_mask(GK.text_GC, cp->bg_mask);
-        gdk_draw_pixmap(cp->bg_pixmap, GK.text_GC, cp->bg_clean_pixmap,
-                    0, 0, 0, 0, cp->w, cp->h);
-        gdk_gc_set_clip_mask(GK.text_GC, NULL);
+        gdk_draw_pixmap(cp->bg_src_pixmap, GK.text_GC,
+				cp->bg_clean_pixmap, 0, 0, 0, 0, cp->w, cp->h);
+		gdk_gc_set_clip_mask(GK.text_GC, NULL);
 		}
 	}	
 
-static void
+gboolean
 draw_rootpixmap_onto_transparent_panel(Panel *p)
 	{
 	Window	child;
 	gint	x, y;
 
-	if (root_xpixmap == None)
-		return;
+	if (root_xpixmap == None || !p->transparency)
+		return FALSE;
 	XTranslateCoordinates(GDK_DISPLAY(),
 			GDK_WINDOW_XWINDOW(p->drawing_area->window),
 			GDK_ROOT_WINDOW(),
@@ -655,51 +640,9 @@ draw_rootpixmap_onto_transparent_panel(Panel *p)
                     0, 0, 0, 0, p->w, p->h);
         gdk_gc_set_clip_mask(GK.text_GC, NULL);
 		}
+	return TRUE;
 	}	
 
-
-static void
-draw_panel(Panel *p, gint to_screen)
-	{
-	Label		*lbl	= p->label;
-	TextStyle	*ts		= p->textstyle;
-	gchar		*s;
-	gint		xdst, margin;
-
-	if (root_xpixmap != None && p->transparency > 0)
-		draw_rootpixmap_onto_transparent_panel(p);
-	else
-		gdk_draw_pixmap(p->bg_pixmap, GK.draw1_GC, p->bg_clean_pixmap,
-					0, 0, 0, 0, p->w, p->h);
-
-	gdk_draw_pixmap(p->pixmap, GK.draw1_GC, p->bg_pixmap,
-					0, 0, 0, 0, p->w, p->h);
-	if (   lbl && ts
-		&& ((s = lbl->string) != NULL)
-		&& lbl->position >= 0
-	   )
-		{
-		lbl->width = gdk_string_width(ts->font, s);
-		margin = p->style ? p->style->margin : 0;
-		xdst = x_label_position(lbl->position, p->w, lbl->width, margin);
-		lbl->x_panel = xdst;
-		gkrellm_draw_string(p->pixmap, ts, xdst, lbl->y_baseline, s);
-		gkrellm_draw_string(p->bg_pixmap, ts, xdst, lbl->y_baseline, s);
-		}
-	if (p->drawing_area->window && to_screen)
-		{
-		gdk_draw_pixmap(p->drawing_area->window, GK.draw1_GC, p->pixmap,
-					0, 0, 0, 0, p->w, p->h);
-		gkrellm_draw_layers_force(p);
-		}
-	}
-
-void
-gkrellm_draw_panel_label(Panel *p, GdkImlibImage *image /* obsolete */)
-	{
-	draw_panel(p, TRUE);
-	}
-
 static void
 apply_rootpixmap_transparency()
 	{
@@ -718,7 +661,7 @@ apply_rootpixmap_transparency()
 	/* Some systems are getting BadDrawable errors when trying to use
 	|  root_xpixmap even though I'm using what XGetWindowProperty() returns???
 	*/
-	if (no_transparency || !any_transparency)
+	if (no_transparency || !GK.any_transparency)
 		return;
 	prop = XInternAtom(GDK_DISPLAY(), "_XROOTPMAP_ID", True);
 	if (prop == None)
@@ -764,226 +707,25 @@ apply_rootpixmap_transparency()
 	*/
 	XSetTile(GDK_DISPLAY(), GDK_GC_XGC(trans_gc), root_xpixmap);
 	XSetFillStyle(GDK_DISPLAY(), GDK_GC_XGC(trans_gc), FillTiled);
-	for (list = chart_list; list; list = list->next)
+	for (list = gkrellm_get_chart_list(); list; list = list->next)
 		{
 		cp = (Chart *) list->data;
-		if (!cp->transparency)
+		if (!cp->transparency || !cp->shown)
 			continue;
 		draw_rootpixmap_onto_transparent_chart(cp);
-		cp->scale_max = 0;
-		cp->need_redraw = TRUE;	/* Charts must be redrawn by monitors */
+		gkrellm_refresh_chart(cp);
 		}
-	for (list = panel_list; list; list = list->next)
+	for (list = gkrellm_get_panel_list(); list; list = list->next)
 		{
 		p = (Panel *) list->data;
 		if (!p->transparency)
 			continue;
-		draw_panel(p, TRUE);	/* Panels know how to redraw themselves */
+		gkrellm_draw_panel_label(p, NULL);
 		}
 	}
 
 /* ---------------------------------------------------------------------- */
 
-  /* Destroy everything inside a panel, but caller must free Panel storage
-  |  if not a static panel.
-  */
-void
-gkrellm_destroy_panel(Panel *p)
-	{
-	gkrellm_destroy_krell_list(p);
-	gkrellm_destroy_decal_list(p);
-
-	if (p->label)
-		g_free(p->label);
-
-	if (p->pixmap)
-		gdk_pixmap_unref(p->pixmap);
-	if (p->bg_pixmap)
-		gdk_pixmap_unref(p->bg_pixmap);
-	if (p->bg_clean_pixmap)
-		gdk_imlib_free_pixmap(p->bg_clean_pixmap);
-
-	if (p->drawing_area)
-		gtk_signal_handlers_destroy(GTK_OBJECT(p->drawing_area));
-	if (p->hbox)
-		gtk_widget_destroy(p->hbox);
-	panel_list = g_list_remove(panel_list, p);
-	memset(p, 0, sizeof(Panel));
-	}
-
-  /* Krells, LED's, and labels are drawn on the panel area drawing_area.
-  */
-void
-gkrellm_create_panel(GtkWidget *vbox, Panel *p, GdkImlibImage *bg_image)
-	{
-	GtkWidget	*hbox;
-	GtkWidget	*fixed;
-
-	p->h = p->label->h_panel;
-	p->w = UC.chart_width;
-
-	if (p->hbox == NULL)
-		{
-		hbox = gtk_hbox_new(FALSE, 0);
-		gtk_container_add (GTK_CONTAINER(vbox), hbox);
-		p->hbox = hbox;
-
-		/* And create the drawing area that the middle panel lives in and put
-		|  it into a fixed widget.
-		*/
-		fixed = gtk_fixed_new();
-		gtk_box_pack_start(GTK_BOX(hbox), fixed, TRUE, TRUE, 0);
-		gtk_widget_show(fixed);
-		p->fixed = fixed;
-		p->drawing_area = gtk_drawing_area_new();
-		gtk_widget_set_events (p->drawing_area, GDK_EXPOSURE_MASK
-				| GDK_LEAVE_NOTIFY_MASK | GDK_ENTER_NOTIFY_MASK
-				| GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK
-				| GDK_POINTER_MOTION_MASK);
-		gtk_fixed_put(GTK_FIXED(fixed), p->drawing_area, 0, 0);
-		gtk_widget_show(p->drawing_area);
-		gtk_widget_show(hbox);
-		gtk_widget_realize(hbox);
-		gtk_widget_realize(p->fixed);
-		gtk_widget_realize(p->drawing_area);
-		panel_list = g_list_append(panel_list, p);
-		}
-	gtk_drawing_area_size(GTK_DRAWING_AREA (p->drawing_area), p->w, p->h);
-	gkrellm_render_to_pixmap(bg_image, &p->bg_clean_pixmap,
-			&p->bg_mask, p->w, p->h);
-	if (p->bg_pixmap)
-		gdk_pixmap_unref(p->bg_pixmap);
-	p->bg_pixmap = gdk_pixmap_new(top_window->window, p->w, p->h, -1);
-	if (p->pixmap)
-		gdk_pixmap_unref(p->pixmap);
-	p->pixmap = gdk_pixmap_new(top_window->window, p->w, p->h, -1);
-
-	draw_panel(p, FALSE);
-	}
-
-static void
-free_chart_pixmaps(Chart *cp)
-	{
-	gdk_pixmap_unref(cp->in_data_bitmap);
-	gdk_pixmap_unref(cp->out_data_bitmap);
-
-	gdk_imlib_free_pixmap(cp->bg_clean_pixmap);
-	gdk_imlib_free_pixmap(cp->bg_pixmap);
-	gdk_imlib_free_pixmap(cp->grid_pixmap);
-	gdk_imlib_free_pixmap(cp->bg_grided_pixmap);
-	gdk_imlib_free_pixmap(cp->in_grided_pixmap);
-	gdk_imlib_free_pixmap(cp->out_grided_pixmap);
-	gdk_imlib_free_pixmap(cp->pixmap);
-	}
-
-  /* Destroy everything inside a chart, but caller must free Chart storage
-  |  if not a static chart.
-  */
-void
-gkrellm_destroy_chart(Chart *cp)
-	{
-	free_chart_pixmaps(cp);
-
-	if (cp->pDataIn)
-		g_free(cp->pDataIn);
-	if (cp->pDataOut)
-		g_free(cp->pDataOut);
-
-	gtk_signal_handlers_destroy(GTK_OBJECT(cp->drawing_area));
-	gtk_widget_destroy(cp->hbox);
-	chart_list = g_list_remove(chart_list, cp);
-	memset(cp, 0, sizeof(Chart));
-	}
-
-void
-gkrellm_chart_images_override(Chart *cp, GdkImlibImage *bg_image,
-			GdkImlibImage *grid_image)
-	{
-	if (!cp || !bg_image || !grid_image)
-		return;
-	cp->bg_image = bg_image;
-	cp->grid_image = grid_image;
-	cp->override_images = TRUE;
-	}
-
-void
-gkrellm_create_chart(GtkWidget *vbox, Chart *cp, gint style_id)
-	{
-	GtkWidget		*hbox;
-	GdkWindow		*window;
-	Style			*style;
-	gint			h;
-
-	if (!vbox || !cp)
-		return;
-	if (!cp->override_images)
-		{
-		cp->bg_image = gkrellm_bg_chart_image(style_id);
-		cp->grid_image = gkrellm_bg_grid_image(style_id);
-		}
-	cp->override_images = FALSE;
-	cp->x = 0;
-/*	cp->y = 0; */
-	cp->w = UC.chart_width;
-	if (cp->h == 0)
-		cp->h = UC.chart_height[0];
-
-	window = top_window->window;
-
-	if (cp->hbox == NULL)
-		{
-		hbox = gtk_hbox_new(FALSE, 0);
-		gtk_container_add (GTK_CONTAINER(vbox), hbox);
-		cp->hbox = hbox;
-
-		/* Create the drawing area and the various pixmap layers to draw on.
-		|  Must gtk_widget_set_events() immediately.
-		*/
-		cp->drawing_area = gtk_drawing_area_new();
-		gtk_widget_set_events (cp->drawing_area, GDK_EXPOSURE_MASK
-				| GDK_LEAVE_NOTIFY_MASK | GDK_ENTER_NOTIFY_MASK
-				| GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK
-				| GDK_POINTER_MOTION_MASK);
-		gtk_box_pack_start (GTK_BOX(hbox), cp->drawing_area, FALSE, FALSE, 0);
-		gtk_widget_show(cp->drawing_area);
-		gtk_widget_show(hbox);
-		gtk_widget_realize(hbox);
-		gtk_widget_realize(cp->drawing_area);
-		chart_list = g_list_append(chart_list, cp);
-		}
-	else
-		free_chart_pixmaps(cp);
-
-	gtk_drawing_area_size(GTK_DRAWING_AREA (cp->drawing_area), cp->w, cp->h);
-
-	/* In/out chart data is drawn on data_bitmaps which are then used
-	|  as stencils for pushing in/out_grided_pixmap pixels onto expose pixmap.
-	*/
-	cp->in_data_bitmap = gdk_pixmap_new(window, cp->w, cp->h, 1);
-	cp->out_data_bitmap = gdk_pixmap_new(window, cp->w, cp->h, 1);
-
-	if (gdk_imlib_render(cp->bg_image, cp->w, cp->h) == 0)
-		printf(_("create_chart, imlib could not render image.\n"));
-	cp->bg_clean_pixmap = gdk_imlib_copy_image(cp->bg_image);
-	cp->bg_mask = gdk_imlib_copy_mask(cp->bg_image);
-	cp->bg_pixmap = gdk_imlib_copy_image(cp->bg_image);
-	cp->bg_mask = gdk_imlib_copy_mask(cp->bg_image);
-	cp->bg_grided_pixmap = gdk_imlib_copy_image(cp->bg_image);
-	cp->in_grided_pixmap = gdk_imlib_copy_image(cp->bg_image);
-	cp->out_grided_pixmap = gdk_imlib_copy_image(cp->bg_image);
-	cp->pixmap = gdk_imlib_copy_image(cp->bg_image);
-
-	if ((h = cp->grid_image->rgb_height) > 2)
-		h = 2;
-	gdk_imlib_render(cp->grid_image, cp->w, h);
-	cp->grid_pixmap = gdk_imlib_copy_image(cp->grid_image);
-
-	style = gkrellm_chart_style(style_id);
-	cp->transparency = style->transparency;
-	any_transparency |= cp->transparency;
-	}
-
-
 static gint		moving_gkrellm	= FALSE;
 static gint		x_press_event, y_press_event;
 
@@ -1044,7 +786,7 @@ top_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	if (time_check > time_now + 2 || time_check < time_now)
 		{
 		printf(_("GKrellM restarting timer...\n"));
-		start_timer(UC.update_HZ);
+		start_timer(GK.update_HZ);
 		}
 
 	if (ev->button == 3)
@@ -1087,7 +829,7 @@ apply_frame_transparency()
 		return;
 
 	win = gtree.window;
-	w = UC.chart_width + GK.frame_left_width + GK.frame_right_width;
+	w = GK.chart_width + GK.frame_left_width + GK.frame_right_width;
 	h = GK.monitor_height + total_frame_height;
 	if (!gtree.window_transparency_mask || w != w_prev || h != h_prev)
 		{
@@ -1176,7 +918,7 @@ side_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 
 	if (ev->button == 2 || (GK.m2 && ev->button == 1))
 		{
-		monitors_visible = 1 - monitors_visible;
+		monitors_visible = !monitors_visible;
 		if (monitors_visible)
 			{
 			gtk_widget_show(gtree.middle_vbox);
@@ -1184,7 +926,7 @@ side_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 			gtk_widget_show(gtree.bottom_vbox);
 			if (direction == CLOSE_RIGHT)
 				gdk_window_move(gtree.window->window,
-								x_gkrell - UC.chart_width, y_gkrell);
+								x_gkrell - GK.chart_width, y_gkrell);
 			}
 		else
 			{
@@ -1193,7 +935,7 @@ side_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 			gtk_widget_hide(gtree.middle_vbox);
 			if (direction == CLOSE_RIGHT)
 				gdk_window_move(gtree.window->window,
-								x_gkrell + UC.chart_width, y_gkrell);
+								x_gkrell + GK.chart_width, y_gkrell);
 			}
 		XSync(GDK_DISPLAY(), False);  /* Avoid double click race */
 		}
@@ -1207,16 +949,22 @@ gkrellm_render_to_pixmap(GdkImlibImage *im, GdkPixmap **pixmap,
 	/* I want the pixmap freed even if there is no image to render back
 	|  in.  Eg. theme switch to one with no data_in/out_image.
 	*/
-	if (*pixmap)
+	if (pixmap && *pixmap)
 		gdk_imlib_free_pixmap(*pixmap);
-	if (im == NULL)
+	if (!im || !pixmap)
 		return FALSE;
 	if (w <= 0)
 		w = im->rgb_width;
 	if (h <= 0)
 		h = im->rgb_height;
 	if (gdk_imlib_render(im, w, h) == 0)
+		{
 		printf(_("render_to_pixmap, imlib could not render image.\n"));
+		*pixmap = NULL;
+		if (mask)
+			*mask = NULL;
+		return FALSE;
+		}
 	*pixmap = gdk_imlib_copy_image(im);
 	if (mask)
 		*mask = gdk_imlib_copy_mask(im);
@@ -1242,7 +990,7 @@ create_frame_top(GtkWidget *vbox)
 		return;
 	im = GK.frame_top_image;
 
-	w = UC.chart_width + GK.frame_left_width + GK.frame_right_width;
+	w = GK.chart_width + GK.frame_left_width + GK.frame_right_width;
 	h = (GK.frame_top_height > 0) ? GK.frame_top_height : im->rgb_height;
 	GK.frame_top_height = h;
 
@@ -1280,7 +1028,7 @@ create_frame_bottom(GtkWidget *vbox)
 		return;
 	im = GK.frame_bottom_image;
 
-	w = UC.chart_width + GK.frame_left_width + GK.frame_right_width;
+	w = GK.chart_width + GK.frame_left_width + GK.frame_right_width;
 	h = (GK.frame_bottom_height > 0) ? GK.frame_bottom_height : im->rgb_height;
 	GK.frame_bottom_height = h;
 
@@ -1332,15 +1080,37 @@ gkrellm_enable_visibility(int new_visibility, gint *current_visibility,
 	return FALSE;
 	}
 
+static gint	freeze_packing;
+
 void
-gkrellm_pack_side_frames()
+gkrellm_freeze_side_frame_packing(void)
+	{
+	freeze_packing += 1;
+	}
+
+void
+gkrellm_thaw_side_frame_packing(void)
+	{
+	if (freeze_packing > 0)
+		{
+		freeze_packing -= 1;
+		if (freeze_packing == 0)
+			gkrellm_pack_side_frames();
+		}
+	}
+
+void
+gkrellm_pack_side_frames(void)
 	{
 	static GtkWidget	*lf_pixmapwid, *rf_pixmapwid;
 	gint				x_gkrell, y_gkrell, y_bottom;
 	gint				y, w, h;
 	gint				was_on_bottom;
 
-	if (! initialized || monitor_previous_height == GK.monitor_height)
+	if (   ! GK.initialized
+		|| monitor_previous_height == GK.monitor_height
+		|| freeze_packing
+	   )
 		return;
 
 	if (window_decorations)		/* was > 1 */
@@ -1351,7 +1121,7 @@ gkrellm_pack_side_frames()
 		}
 	if (GK.debug_level & DEBUG_SIZES)
 		printf(_("pack_side_frames mon_width=%d mon_height=%d "),
-				UC.chart_width + GK.frame_left_width + GK.frame_right_width,
+				GK.chart_width + GK.frame_left_width + GK.frame_right_width,
 				GK.monitor_height + total_frame_height);
 
 	gdk_window_get_origin(gtree.window->window, &x_gkrell, &y_gkrell);
@@ -1401,6 +1171,7 @@ gkrellm_pack_side_frames()
 		gdk_window_move(gtree.window->window, x_gkrell, y);
 		}
 	monitor_previous_height = GK.monitor_height;
+	check_rootpixmap_transparency = 3;
 	}
 
 static gint	on_edge[4];		/* left, right, top, bottom */
@@ -1424,7 +1195,7 @@ fix_edges()
 	gint	x, y, w, h, x0, y0;
 
 	gdk_window_get_origin(gtree.window->window, &x, &y);
-	w = UC.chart_width + GK.frame_left_width + GK.frame_right_width;
+	w = GK.chart_width + GK.frame_left_width + GK.frame_right_width;
 	h = GK.monitor_height + total_frame_height;
 
 	x0 = x;
@@ -1441,60 +1212,30 @@ fix_edges()
 		gdk_window_move(gtree.window->window, x, y);
 	}
 
-  /* My own parsing of +x+y, -x-y geometry placements.
+  /* Use XParseGeometry, but width and height are ignored.
   */
 static void
 place_gkrellm(gchar *geom)
 	{
-	gchar	*sx, *sy, x_sign, y_sign;
-	gint	x, y, w_gkrell, h_gkrell;
+	gint	place, x, y, w_gkrell, h_gkrell;
 
-	w_gkrell = UC.chart_width + GK.frame_left_width + GK.frame_right_width;
+	x = y = 0;
+	place = XParseGeometry(geom, &x, &y, &w_gkrell, &h_gkrell);
+
+	w_gkrell = GK.chart_width + GK.frame_left_width + GK.frame_right_width;
 	h_gkrell = GK.monitor_height + total_frame_height;
-	sx = geom;
-	if (sx == NULL || *sx == '\0')
-		{
-		printf(_("Cannot parse geometry.\n"));
-		return;
-		}
-	if (*sx != '+' && *sx != '-')
-		x_sign = '+';
-	else
-		x_sign = *sx++;
-	for (sy = sx; *sy != '\0'; ++sy)
-		if (*sy == '+' || *sy == '-')
-			break;
-	if (*sy == '\0')
-		{
-		printf(_("Cannot parse geometry.\n"));
-		return;
-		}
-	y_sign = *sy;
-	*sy++ = '\0';
 
-	x = atoi(sx);
-	if (x_sign == '-')
-		x = w_display - w_gkrell - x;
-
-	y = atoi(sy);
-	if (y_sign == '-')
-		y = h_display - h_gkrell - y;
-
-	if (x > w_display - w_gkrell)
-		x = w_display - w_gkrell;
-	if (x < 0)
-		x = 0;
-	if (y > h_display - h_gkrell)
-		y = h_display - h_gkrell;
-	if (y < 0)
-		y = 0;
+	if (place & YNegative)
+		y = h_display - h_gkrell + y;
+	if (place & XNegative)
+		x = w_display - w_gkrell + x;
 	gdk_window_move(gtree.window->window, x, y);
 	}
 
 static void
 create_spacers(Monitor *mon)
 	{
-	MonPrivate	*mp		= mon->private;
+	MonPrivate	*mp		= mon->privat;
 	GdkPixmap	*pixmap;
 	gint		h, w;
 
@@ -1505,10 +1246,12 @@ create_spacers(Monitor *mon)
 		gtk_widget_destroy(mp->bottom_pixmapwid);
 	mp->bottom_pixmapwid = NULL;
 
-	w = UC.chart_width;
-	if (mon->private->spacer_top_image)
+	w = GK.chart_width;
+	if (mon->privat->spacer_top_image)
 		{
-		h = mp->spacer_top_height ? mp->spacer_top_height : 3;
+		if (mp->spacer_top_height == 0)
+			mp->spacer_top_height = 3;
+		h = mp->spacer_top_height;
 		pixmap = gdk_pixmap_new(top_window->window, w, h, -1);
 		gdk_imlib_paste_image(mp->spacer_top_image, pixmap, 0, 0, w, h);
 		mp->top_pixmapwid = gtk_pixmap_new(pixmap, NULL);
@@ -1520,10 +1263,12 @@ create_spacers(Monitor *mon)
 	else
 		gtk_widget_hide(mp->spacer_top_vbox);
 
-	if (mon->private->spacer_bottom_image)
+	if (mon->privat->spacer_bottom_image)
 		{
-		h = mp->spacer_bottom_height ? mp->spacer_bottom_height : 3;
-		pixmap = gdk_pixmap_new(top_window->window, UC.chart_width, h, -1);
+		if (mp->spacer_bottom_height == 0)
+			mp->spacer_bottom_height = 3;
+		h = mp->spacer_bottom_height;
+		pixmap = gdk_pixmap_new(top_window->window, GK.chart_width, h, -1);
 		gdk_imlib_paste_image(mp->spacer_bottom_image, pixmap, 0, 0, w, h);
 		mp->bottom_pixmapwid = gtk_pixmap_new(pixmap, NULL);
 		gtk_box_pack_start(GTK_BOX(mp->spacer_bottom_vbox),
@@ -1533,6 +1278,49 @@ create_spacers(Monitor *mon)
 		}
 	else
 		gtk_widget_hide(mp->spacer_bottom_vbox);
+	mon->privat->spacers_shown = TRUE;
+	}
+
+void
+show_spacers(Monitor *mon)
+	{
+	MonPrivate	*mp = mon->privat;
+
+	if (mp->spacers_shown)
+		return;
+	if (mp->spacer_top_image)
+		{
+		gtk_widget_show(mp->spacer_top_vbox);
+		GK.monitor_height += mp->spacer_top_height;
+		}
+	if (mp->spacer_bottom_image)
+		{
+		gtk_widget_show(mp->spacer_bottom_vbox);
+		GK.monitor_height += mp->spacer_bottom_height;
+		}
+	mp->spacers_shown = TRUE;
+	gkrellm_pack_side_frames();
+	}
+
+void
+hide_spacers(Monitor *mon)
+	{
+	MonPrivate	*mp = mon->privat;
+
+	if (!mp->spacers_shown)
+		return;
+	if (mp->spacer_top_image)
+		{
+		gtk_widget_hide(mp->spacer_top_vbox);
+		GK.monitor_height -= mp->spacer_top_height;
+		}
+	if (mp->spacer_bottom_image)
+		{
+		gtk_widget_hide(mp->spacer_bottom_vbox);
+		GK.monitor_height -= mp->spacer_bottom_height;
+		}
+	mp->spacers_shown = FALSE;
+	gkrellm_pack_side_frames();
 	}
 
 static void
@@ -1645,7 +1433,7 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 	gdk_window_get_position(widget->window, &x, &y);
 	gdk_window_get_size(gtree.window->window, &w, &h);
 
-	w_gkrellm = UC.chart_width + GK.frame_left_width + GK.frame_right_width;
+	w_gkrellm = GK.chart_width + GK.frame_left_width + GK.frame_right_width;
 	h_gkrellm = GK.monitor_height + total_frame_height;
 
 	/* If window manager decorated, I can't allow a user to resize GKrellM
@@ -1676,7 +1464,7 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 	|  right place yet even though configure notify is sent.
 	*/
 	if ((size_change || position_change) && !moving_gkrellm)
-		check_rootpixmap_transparency = TRUE;
+		check_rootpixmap_transparency = 3;
 	x_prev = x;
 	y_prev = y;
 	w_prev = w;
@@ -1691,15 +1479,15 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 	|  a chance to save its position.  So write the position file
 	|  10 seconds after window moves stop.
 	*/
-	if (UC.save_position)
-		save_position_countdown = 10 * UC.update_HZ;
+	if (GK.save_position)
+		save_position_countdown = 10 * GK.update_HZ;
 	}
 
 void
 gkrellm_save_all()
 	{
 	save_inet_data();
-	if (UC.save_position)
+	if (GK.save_position)
 		set_or_save_position(1);
 	if (GK.config_modified)
 		save_user_config();
@@ -1866,6 +1654,11 @@ gkrellm_get_restart_options(gchar **argv, gint max_argc)
 	return argc;
 	}
 
+GtkWidget *
+gkrellm_monitor_vbox(void)
+	{
+	return gtree.monitor_vbox;
+	}
 
 void
 build_gkrellm()
@@ -1874,22 +1667,26 @@ build_gkrellm()
 	GList		*list;
 	static gint	first_create	= TRUE;
 
-	initialized = FALSE;
+	GK.initialized = FALSE;
 	monitor_previous_height = 0;
 	total_frame_height = 0;
 	root_xpixmap = None;
-	any_transparency = FALSE;
+	GK.any_transparency = FALSE;
 	start_timer(0);				/* Stop the timer */
 
 	if (!first_create)
 		edge_record();
-	destroy_decal_button_list();
+	reset_all_alerts();
+	clean_out_panels();
+
 	gkrellmrc_theme_config();
 	if (first_create)
 		read_user_config();
 	setup_colors();
 	setup_fonts();
 	load_theme_images();
+	setup_charts();
+	gkrellm_freeze_side_frame_packing();
 
 	create_frame_top(gtree.top0_vbox);
 
@@ -1901,32 +1698,35 @@ build_gkrellm()
 	for (list = gkrellm_monitor_list; list; list = list->next)
 		{
 		mon = (Monitor *) list->data;
-		if (mon->private->vbox)
+		if (mon->privat->main_vbox)
 			{
 			if (first_create)
 				{
 				gtk_box_pack_start(GTK_BOX(gtree.monitor_vbox),
-							mon->private->spacer_top_vbox, FALSE, FALSE, 0);
-				gtk_box_pack_start(GTK_BOX(gtree.monitor_vbox),
-							mon->private->vbox, FALSE, FALSE, 0);
-				gtk_box_pack_start(GTK_BOX(gtree.monitor_vbox),
-							mon->private->spacer_bottom_vbox, FALSE, FALSE, 0);
+							mon->privat->main_vbox, FALSE, FALSE, 0);
+				gtk_box_pack_start(GTK_BOX(mon->privat->main_vbox),
+							mon->privat->spacer_top_vbox, FALSE, FALSE, 0);
+				gtk_box_pack_start(GTK_BOX(mon->privat->main_vbox),
+							mon->privat->vbox, FALSE, FALSE, 0);
+				gtk_box_pack_start(GTK_BOX(mon->privat->main_vbox),
+							mon->privat->spacer_bottom_vbox, FALSE, FALSE, 0);
 				}
-			if (mon->create_monitor && mon->private->enabled)
+			if (mon->create_monitor && mon->privat->enabled)
 				{
-				gtk_widget_show(mon->private->vbox);
-				(*(mon->create_monitor))(mon->private->vbox,
-							mon->private->created ? FALSE : TRUE);
-				mon->private->created = TRUE;
+				gtk_widget_show(mon->privat->vbox);
+				gtk_widget_show(mon->privat->main_vbox);
 				create_spacers(mon);
+				(*(mon->create_monitor))(mon->privat->vbox,
+							mon->privat->created ? FALSE : TRUE);
+				mon->privat->created = TRUE;
 				}
 			}
 		}
 	create_frame_bottom(gtree.bottom_vbox);
 
-	initialized = TRUE;
-	gkrellm_pack_side_frames();
-	start_timer(UC.update_HZ);
+	GK.initialized = TRUE;
+	gkrellm_thaw_side_frame_packing();
+	start_timer(GK.update_HZ);
 	if (!first_create)
 		{
 		fix_edges();
@@ -1941,15 +1741,16 @@ add_builtin(Monitor *mon)
 	if (mon == NULL)
 		return;
 	gkrellm_monitor_list = g_list_append(gkrellm_monitor_list, mon);
-	if (mon->private == NULL)		/* Won't be null if style was added */
-		mon->private = g_new0(MonPrivate, 1);
+	if (mon->privat == NULL)		/* Won't be null if style was added */
+		mon->privat = g_new0(MonPrivate, 1);
 	if (mon->create_monitor)
 		{
-		mon->private->spacer_top_vbox = gtk_vbox_new(FALSE, 0);
-		mon->private->vbox = gtk_vbox_new(FALSE, 0);
-		mon->private->spacer_bottom_vbox = gtk_vbox_new(FALSE, 0);
+		mon->privat->main_vbox = gtk_vbox_new(FALSE, 0);
+		mon->privat->spacer_top_vbox = gtk_vbox_new(FALSE, 0);
+		mon->privat->vbox = gtk_vbox_new(FALSE, 0);
+		mon->privat->spacer_bottom_vbox = gtk_vbox_new(FALSE, 0);
 		}
-	mon->private->enabled = TRUE;
+	mon->privat->enabled = TRUE;
 	}
 
   /* Eventually, add a config for selecting specific monitors here so
@@ -1980,8 +1781,6 @@ load_builtin_monitors()
 	add_builtin(init_mem_monitor());
 	add_builtin(init_swap_monitor());
 	add_builtin(init_fs_monitor());
-	if (GK.demo)
-		add_builtin(init_demo_monitor());
 	add_builtin(init_mail_monitor());
 	add_builtin(init_apm_monitor());
 	add_builtin(init_uptime_monitor());
@@ -1997,6 +1796,21 @@ set_window_title()
 	g_free(title);
 	}
 
+static void
+quit_gkrellm(gint sig)
+	{
+	gkrellm_save_all();
+	exit(1);			/* Plugins get a chance for g_atexit() calls */	
+	}
+
+static void
+setup_signal_handler(void)
+	{
+	signal(SIGINT, quit_gkrellm);
+	signal(SIGQUIT, quit_gkrellm);
+	signal(SIGTERM, quit_gkrellm);
+	}
+
 int
 main (int argc, char *argv[])
 	{
@@ -2023,6 +1837,37 @@ main (int argc, char *argv[])
 		}
 #endif
 
+#if defined(__solaris__)
+	/* 
+	 * Most of stats (cpu, proc, disk, memory, net and uptime) are 
+	 * unavailable if kstat_open() failed. So we just exit in that case.
+	 */
+	if ((kc = kstat_open()) == NULL) {
+		perror("kstat_open");
+		return 1;
+	}
+
+	/*
+	 * kvm is utilized solely for getting a value for proc.n_forks 
+	 * from kernel variable called mpid. Even if kvm_open() fails,
+	 * we proceed without it.
+	 */
+	if ((kd = kvm_open(NULL, NULL, NULL, O_RDONLY, NULL)) != NULL) { 
+		kvm_nlist(kd, nl);
+	}
+
+        /*
+         * a function called by the following requires sys gid privilege.
+         * the folowing function should be performed here just for that reason.
+         */ 
+        solaris_list_harddisks();
+
+        if (setgid(getgid()) != 0) {
+		perror("Failed to drop setgid privilege");
+		return 1;
+        }
+#endif /* __solaris__ */
+
 #ifdef ENABLE_NLS
 	gtk_set_locale();
 #endif
@@ -2047,7 +1892,7 @@ main (int argc, char *argv[])
 
 
 #if !defined(__FreeBSD__) && !defined(__linux__) && !defined(__NetBSD__) \
-	&& !defined(__OpenBSD__)
+	&& !defined(__OpenBSD__) && !defined(__solaris__)
 	glibtop_init();
 #endif
 
@@ -2057,8 +1902,8 @@ main (int argc, char *argv[])
 	gdk_imlib_set_cache_info(0, 1);		/* cache_pixmaps, cache_images	*/
 
 	GK.start_time = time(0);
-	current_tm = *(localtime(&GK.start_time));
-	initialized = FALSE;
+	gkrellm_current_tm = *(localtime(&GK.start_time));
+	GK.initialized = FALSE;
 
 	GK.sm_argc = 1;
 	GK.sm_argv[0] = argv[0];
@@ -2144,6 +1989,7 @@ main (int argc, char *argv[])
 
 	create_widget_tree();
 	top_window = gtree.window;
+	
 
 	check_gkrellm_directories();
 	load_builtin_monitors();
@@ -2186,7 +2032,7 @@ main (int argc, char *argv[])
 
 	if (geometry)		/* Command line placement overrides */
 		place_gkrellm(geometry);
-	else if (UC.save_position)
+	else if (GK.save_position)
 		set_or_save_position(0);
 
     if (withdrawn)
@@ -2202,29 +2048,10 @@ main (int argc, char *argv[])
 	if (on_top)
 		set_on_top();
 
-	load_inet_data();
-
-	start_timer(UC.update_HZ);
+	start_timer(GK.update_HZ);
+	setup_signal_handler();
 	gtk_main ();
 	gkrellm_save_all();
 
 	return 0;
 	}
-
-
-#if 0
-/* Debugging aid */
-void
-exit(int code)
-    {
-	printf("exit: gkrellm forcing core dump.\n");
-	abort();
-	}
- 
-void
-_exit(int code)
-    {
-	printf("_exit: gkrellm forcing core dump.\n");
-	abort();
-	}
-#endif
