@@ -19,46 +19,53 @@
 */
 
 /*
-|  1/23/2001 Hajimu UMEMOTO added get_bufspace(), improved
-|				read_freebsd_meminfo().
-| 10/12/2000 NetBSD code contributed by Anthony Mallet
-|			<metall@ficus.yi.org>
-|  5/27/2000 Patch from William Carrel <william.a@carrel.org> and 
-|		Hajimu UMEMOTO ume@mahoroba.org to cleanup FreeBSD swap code.
+|  4/22/2001  Solaris code contributed by Daisuke Yabuki <dxy@acm.org>
+|  1/23/2001  Hajimu UMEMOTO added get_bufspace(), improved
+|               read_freebsd_meminfo().
+| 10/12/2000  NetBSD code contributed by Anthony Mallet <metall@ficus.yi.org>
+|  5/27/2000  Patch from William Carrel <william.a@carrel.org> and 
+|               Hajimu UMEMOTO ume@mahoroba.org to cleanup FreeBSD swap code.
 |  2/25/2000  FreeBSD code contributed by Hajimu UMEMOTO ume@mahoroba.org
 */
 
 #include "gkrellm.h"
 #include "gkrellm_private_proto.h"
 
-  /* The meminfo monitor is a hybrid of chart and meter types.
-  |  There is a swap_chart without a panel, and a mem and swap meter.
-  |  The swap meter has both a standard meter styled krell (krell_used)
-  |  and a chart panel styled krell (krell_delta).
-  |  There are two monitors created here, mem and swap meters.  Because
-  |  this was the original setup and many themes have customized based
-  |  on this, the swap_chart is sort of retrofitted on.  I put it on
-  |  top of the mem/swap meter pair for looks and it uses the default
-  |  chart style (ie cannot be custom themed).  So from the point of
-  |  view of themers, there are still only mem and swap meter monitors.
-  */
-  /* Also, enabling is handled differently for the chart vs meters.  The
-  |  chart is created/destroyed when enabled/disabled while the meters
-  |  just toggle hbox visibility when enabled/disabled.  The reason is ...
-  |  ooops, I'm out of time ...
+#include "pixmaps/mem/krell_buffers.xpm"
+#include "pixmaps/mem/krell_cache.xpm"
+
+  /* The mem monitor has two extra krells which can be themed.
+  |  So, the theme "mem" subdir can have:
+  |      bg_panel.png
+  |      krell.png
+  |      krell_buffers.png		# an extension image, defaults to included xpm
+  |      krell_cache.png		# an extension image, defaults to included xpm
+  |
+  |  The gkrellmrc can have theme extension variables for these extra krells:
+  |      set_integer mem_krell_buffers_yoff n
+  |      set_integer mem_krell_buffers_depth n
+  |      set_integer mem_krell_buffers_x_hot n
+  |      set_integer mem_krell_cache_yoff n
+  |      set_integer mem_krell_cache_depth n
+  |      set_integer mem_krell_cache_x_hot n
   */
+
 typedef struct
 	{
 	Panel		*panel;
 	Krell		*krell_used,	/* Meter styled, shows fraction used	*/
-				*krell_delta;	/* Chart styled, for page in/out deltas */
+				*krell_delta,	/* Chart styled, for page in/out deltas */
+				*krell_buffers,
+				*krell_cache;
 	gchar		*label,
 				*data_format;	/* Format string for scrolling text		*/
+	gint		style_id;
 	gint		x_label;
 	Decal		*decal_label;
 	gboolean	label_is_data,
-				restore_label;	/* Helper to know when to toggle data fmt off*/
-	gint		style_id;
+				restore_label,	/* Helper to know when to toggle data fmt off*/
+				mouse_entered,
+				all_krells;
 	gint		enabled;
 	Launcher	launch;
 
@@ -79,12 +86,13 @@ MeminfoMeter	mem,
 
 typedef struct
 	{
-	GtkWidget	*vbox;			/* Holds the chart */
+	GtkWidget	*vbox;
 	Chart		*chart;
-	gint		enabled;
-	gint		resolution;
-	gint		extra_info;
-	gulong		cur;
+	ChartData	*in_cd,
+				*out_cd;
+	ChartConfig	*chart_config;
+	gboolean	enabled;
+	gboolean	extra_info;
 
 	/* (*(read_system_meminfo))() fills in this data
 	*/
@@ -524,7 +532,7 @@ read_freebsd_meminfo()
 
 
 static void
-read_proc_meminfo()
+read_proc_meminfo(void)
 	{
 	FILE		*f;
 	gchar		buf[160];
@@ -637,6 +645,68 @@ read_netbsd_meminfo()
 
 #endif /* __NetBSD__ || __OpenBSD__ */
 
+/* --------- Solaris -------------------------------------------------- */
+
+#if defined(__solaris__)
+
+#include <unistd.h>
+#include <kstat.h>
+#include <sys/stat.h>
+#include <sys/swap.h>
+
+static void
+read_solaris_meminfo() {
+
+    gulong pagesize;
+    static gulong pageshift = 0, physpages = 0;
+    extern kstat_ctl_t *kc;
+    kstat_t *ksp;
+    kstat_named_t *knp;  
+
+    struct anoninfo ai;
+
+    if (!GK.second_tick)
+        return;
+
+    if (pageshift == 0) {
+        for (pagesize = sysconf(_SC_PAGESIZE); pagesize > 1; pagesize >>= 1)
+            pageshift++;
+    }
+    if (physpages == 0) {
+        physpages = sysconf(_SC_PHYS_PAGES);
+    }
+
+    mem.total = physpages << pageshift;
+
+    ksp = kstat_lookup(kc, "unix", -1, "system_pages");
+    if (ksp && kstat_read(kc, ksp, NULL) >= 0) {
+        knp = (kstat_named_t *)kstat_data_lookup(ksp, "pagesfree");
+        if (knp) {
+            mem.free = knp->value.ui32 << pageshift;
+            mem.used = mem.total - mem.free;
+        }
+    }
+
+    if (swapctl(SC_AINFO, &ai) == -1) {
+        perror("swapctl");
+    }
+    swap.total = ai.ani_max << pageshift;
+    swap.used  = ai.ani_resv << pageshift;
+
+    /* UNIMPLEMENTED */
+    swap_chart.page_in = swap_chart.page_out = 0; 
+
+    /* NEED TO BE COMPLETED
+     * mem.total, mem.used, (they are used in memory meter)
+     * mem.x_used, mem.free, mem.shared, mem.buffers, mem.cached 
+     * swap.total, swap.used (they are used in swap meter) 
+     * swap_chart.page_in, swap_chart.page_out (for swap pages in/out chart) 
+     */ 
+
+}
+
+#endif /* __solaris__ */
+
 
 /* --------- Others --------------------------------------------------- */
 #if defined(USE_LIBGTOP)
@@ -645,7 +715,7 @@ read_netbsd_meminfo()
 #include <glibtop/swap.h>
 
 static void
-read_glibtop_meminfo()
+read_glibtop_meminfo(void)
 	{
 	glibtop_mem		glt_mem;
 	glibtop_swap	glt_swap;
@@ -691,7 +761,7 @@ read_glibtop_meminfo()
 
 /* ----- Pick a system interface ----------------------------------------- */
 static gint
-setup_meminfo_interface()
+setup_meminfo_interface(void)
 	{
 #if defined(__FreeBSD__)
 	read_system_meminfo = read_freebsd_meminfo;
@@ -699,6 +769,8 @@ setup_meminfo_interface()
 	read_system_meminfo = read_proc_meminfo;
 #elif defined(__NetBSD__) || defined(__OpenBSD__)
 	read_system_meminfo = read_netbsd_meminfo;
+#elif defined(__solaris__)
+	read_system_meminfo = read_solaris_meminfo;
 #else
 	read_system_meminfo = read_glibtop_meminfo;
 #endif
@@ -707,9 +779,17 @@ setup_meminfo_interface()
 
 /* ======================================================================== */
 
+#define	MIN_GRID_RES		20
+#define	MAX_GRID_RES		100000
+#define DEFAULT_GRID_RES	1000
+
+#define DEFAULT_SWAP_CHART_HEIGHT 20
 
 #define	DEFAULT_FORMAT	_("$t - $f free")
 
+static Monitor		*mon_mem,
+					*mon_swap;
+
 static gint			x_scroll,
 					x_mon_motion,
 					x_moved,
@@ -717,8 +797,6 @@ static gint			x_scroll,
 
 static MeminfoMeter		*mon_in_motion;
 
-static gint			chart_height	= 20;
-
 #define MEG(x)	(((x) + (1 << 19)) >> 20)
 
 
@@ -735,7 +813,7 @@ static gint		mem_pipe[PIPE_SIZE],
 static gint		force_update	= TRUE;
 
 gboolean
-force_meminfo_update()
+force_meminfo_update(void)
 	{
 	gint	i, force;
 
@@ -865,7 +943,7 @@ draw_decal_label(MeminfoMeter *mm, gint draw_to_screen)
 		d->text_style = ts_save;
 		}
 	if (draw_to_screen)
-		gkrellm_draw_layers(mm->panel);
+		gkrellm_draw_panel_layers(mm->panel);
 	return w;
 	}
 
@@ -880,56 +958,105 @@ record_activity(gint *pipe, gint modified)
 	pipe[0] = modified;
 	}
 
+
+static size_abbrev_table	swap_blocks_abbrev[] =
+	{
+	{ KB_SIZE(1),		1,				"%.0f" },
+	{ KB_SIZE(20),		KB_SIZE(1),		"%.1fK" },
+	{ MB_SIZE(1),		KB_SIZE(1),		"%.0fK" },
+	{ MB_SIZE(20),		MB_SIZE(1),		"%.1fM" }
+	};
+
+
+static gchar    *text_format;
+
+static void
+format_chart_text(MeminfoChart *mc, gchar *buf, gint size)
+	{
+	Chart	*cp;
+	gchar	c, *s;
+	size_t	tbl_size;
+	gint	len, in_blocks, out_blocks, blocks;
+
+	--size;
+	*buf = '\0';
+	cp = mc->chart;
+	in_blocks = gkrellm_get_current_chartdata(mc->in_cd);
+	out_blocks = gkrellm_get_current_chartdata(mc->out_cd);
+	tbl_size = sizeof(swap_blocks_abbrev) / sizeof(size_abbrev_table);
+	for (s = text_format; *s != '\0' && size > 0; ++s)
+		{
+		len = 1;
+		if (*s == '$' && *(s + 1) != '\0')
+			{
+			blocks = -1;
+			if ((c = *(s + 1)) == 'T')
+				blocks = in_blocks + out_blocks;
+			else if (c == 'M')
+				blocks = gkrellm_get_chart_scalemax(cp);
+			else if (c == 'i')
+				blocks = in_blocks;
+			else if (c == 'o')
+				blocks = out_blocks;
+			else
+				{
+				*buf = *s;
+				if (size > 1)
+					{
+					*(buf + 1) = *(s + 1);
+					++len;
+					}
+				}
+			if (blocks >= 0)
+				len = format_size_abbrev(buf, size, (gfloat) blocks,
+						&swap_blocks_abbrev[0], tbl_size);
+			++s;
+			}
+		else
+			*buf = *s;
+		size -= len;
+		buf += len;
+		}
+	*buf = '\0';	
+	}
+
 static void
 draw_extra(MeminfoChart *mc)
     {
-    TextStyle   *ts;
-    gchar       buf[32];
-	gint		n;
-
-    ts = gkrellm_chart_textstyle(DEFAULT_STYLE_ID);
-    if (ascent == 0)
-		ascent = gdk_char_height(ts->font, '8');
-	n = (gint) mc->cur;
-	if (n < 1000)
-		snprintf(buf, sizeof(buf), "%d", n);
-	else if (n < 19950)
-		{
-		n += 50;
-		snprintf(buf, sizeof(buf), "%d.%dK", n / 1000, (n % 1000) / 100);
-		}
-	else
-		{
-		n += 500;
-		snprintf(buf, sizeof(buf), "%dK", n / 1000);
-		}
-    gkrellm_draw_chart_label(mc->chart, ts, 4, 4 + ascent, buf);
-    }
+    gchar       buf[128];
 
+	if (!mc->extra_info)
+		return;
+	format_chart_text(mc, buf, sizeof(buf));
+	gkrellm_draw_chart_text(mc->chart, DEFAULT_STYLE_ID, buf);	/* XXX */
+	}
 
 static void
 refresh_chart(MeminfoChart *mc)
 	{
 	if (mc->chart)
 		{
-		gkrellm_draw_chart(mc->chart);
-		if (mc->extra_info)
-			draw_extra(mc);
-		mc->chart->need_redraw = FALSE;
+		gkrellm_draw_chartdata(mc->chart);
+		draw_extra(mc);
+		gkrellm_draw_chart_to_screen(mc->chart);
 		}
 	}
 
 static gint
-cb_extra(GtkWidget *widget, GdkEventButton *event, gpointer data)
+cb_extra(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 	{
 	MeminfoChart	*mc	= (MeminfoChart *) data;
 
-	if (event->button == 1)
+	if (ev->button == 1 && ev->type == GDK_BUTTON_PRESS)
 		{
-		mc->extra_info = 1 - mc->extra_info;
+		mc->extra_info = !mc->extra_info;
 		gkrellm_config_modified();
+		refresh_chart(mc);
 		}
-	refresh_chart(mc);
+	else if (   ev->button == 3
+			 || (ev->button == 1 && ev->type == GDK_2BUTTON_PRESS)
+			)
+		gkrellm_chartconfig_window_create(mc->chart);
 	return TRUE;
 	}
 
@@ -937,7 +1064,7 @@ static void
 update_meminfo(void)
 	{
 	Chart	*cp;
-	gulong	u;
+	gulong	u, b, c;
 	gint	w_scroll, w;
 
 	if (! (mem.enabled || swap.enabled || swap_chart.enabled))
@@ -950,29 +1077,34 @@ update_meminfo(void)
 
 		if ((cp = mc->chart) != NULL && GK.second_tick)
 			{
-			if (cp->primed)
-				mc->cur = (mc->page_in - cp->prevIn)
-							+ (mc->page_out - cp->prevOut);
-			gkrellm_store_chart_data(cp, mc->page_out, mc->page_in, 0);
+			gkrellm_store_chartdata(cp, 0, mc->page_out, mc->page_in);
 			refresh_chart(mc);
 			}
 		}
 	if (mem.enabled)
 		{
 		mem.krell_used->full_scale = (gint) (mem.total >> 12);
-		mem.krell_used->previous = 0;
+		mem.krell_buffers->full_scale = mem.krell_used->full_scale;
+		mem.krell_cache->full_scale = mem.krell_used->full_scale;
 		u = mem.used >> 12;
-		if (mem.label_is_data && mon_in_motion && x_moved)
-			u = 0;
+		b = u + (mem.buffers >> 12);
+		c = b + (mem.cached >> 12);
+		if (   (mem.label_is_data && mon_in_motion && x_moved)
+			|| mem.mouse_entered
+		   )
+			u = b = c = 0;
 		gkrellm_update_krell(mem.panel, mem.krell_used, u);
+		gkrellm_update_krell(mem.panel, mem.krell_buffers, b);
+		gkrellm_update_krell(mem.panel, mem.krell_cache, c);
 		record_activity(mem_pipe, mem.krell_used->modified);
 		}
 	if (swap.enabled)
 		{
 		swap.krell_used->full_scale = (gint) (swap.total >> 12);
-		swap.krell_used->previous = 0;
 		u = swap.used >> 12;
-		if (swap.label_is_data && mon_in_motion && x_moved)
+		if (   (swap.label_is_data && mon_in_motion && x_moved)
+			|| swap.mouse_entered
+		   )
 			u = 0;
 		gkrellm_update_krell(swap.panel, swap.krell_used, u);
 		record_activity(swap_pipe, swap.krell_used->modified);
@@ -980,8 +1112,6 @@ update_meminfo(void)
 	if (swap.krell_delta && swap.enabled)
 		gkrellm_update_krell(swap.panel, swap.krell_delta,
 				swap_chart.page_in + swap_chart.page_out);
-	if (swap_chart.chart && swap_chart.chart->need_redraw)
-		refresh_chart(&swap_chart);
 
 	w = w_scroll = 0;
 	if (mem.label_is_data && mon_in_motion != &mem && mem.enabled)
@@ -994,13 +1124,13 @@ update_meminfo(void)
 	if (!mon_in_motion)
 		{
 		if (w_scroll > mem.decal_label->w)
-			x_scroll = (x_scroll + ((UC.update_HZ < 7) ? 2 : 1))
+			x_scroll = (x_scroll + ((gkrellm_update_HZ() < 7) ? 2 : 1))
 						% (w_scroll - mem.decal_label->w / 3);
 		else
 			x_scroll = 0;
 		}
-	gkrellm_draw_layers(mem.panel);
-	gkrellm_draw_layers(swap.panel);
+	gkrellm_draw_panel_layers(mem.panel);
+	gkrellm_draw_panel_layers(swap.panel);
 	}
 
 static gint
@@ -1021,9 +1151,26 @@ meminfo_expose_event(GtkWidget *widget, GdkEventExpose *ev)
 	return FALSE;
 	}
 
+static gint
+cb_panel_enter(GtkWidget *w, GdkEventButton *ev, MeminfoMeter *mm)
+	{
+	if (mm->label_is_data)
+		mm->mouse_entered = TRUE;
+	return TRUE;
+	}
+
+static gint
+cb_panel_leave(GtkWidget *w, GdkEventButton *ev, MeminfoMeter *mm)
+	{
+	mm->mouse_entered = FALSE;
+	return TRUE;
+	}
+
 static gint
 cb_panel_release(GtkWidget *widget, GdkEventButton *ev)
 	{
+    if (ev->button != 2)
+		return TRUE;
 	if (mon_in_motion)
 		{
 		if (mon_in_motion->restore_label)
@@ -1043,7 +1190,7 @@ cb_panel_release(GtkWidget *widget, GdkEventButton *ev)
 static gint
 cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
     {
-    if (ev->button != 2 && ev->button != 3)
+    if (ev->button != 2)
 		return TRUE;
 	if (widget == mem.panel->drawing_area)
 		mon_in_motion = &mem;
@@ -1055,6 +1202,7 @@ cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
 		{
 		mon_in_motion->label_is_data = TRUE;
 		mon_in_motion->restore_label = FALSE;
+		mon_in_motion->mouse_entered = TRUE;
 		gkrellm_config_modified();
 		}
 	x_mon_motion = ev->x;
@@ -1107,20 +1255,22 @@ cb_panel_motion(GtkWidget *widget, GdkEventButton *ev)
 	}
 
 static void
-setup_scaling(MeminfoChart *mc, MeminfoMeter *mm)
+setup_scaling(ChartConfig *cf, MeminfoChart *mc)
 	{
-	Chart	*cp	= mc->chart;
-	gint	grids;
+	Chart	*cp	  = mc->chart;
+	gint	res   = DEFAULT_GRID_RES,
+			grids = FULL_SCALE_GRIDS;
 
-	grids = UC.fixed_scale ? UC.fixed_scale : FULL_SCALE_GRIDS;
 	if (cp)
 		{
-		cp->scale_min = mc->resolution;
-		cp->scale_max = 0;		/* Force a chart redraw */
+		grids = gkrellm_get_chartconfig_fixed_grids(cp->config);
+		res = gkrellm_get_chartconfig_grid_resolution(cp->config);
 		}
-	if (mm->krell_delta)
-		mm->krell_delta->full_scale =
-					mc->resolution * grids / UC.update_HZ;
+	if (grids == 0)
+		grids = FULL_SCALE_GRIDS;
+
+	if (swap.krell_delta)
+		swap.krell_delta->full_scale = res * grids / gkrellm_update_HZ();
 	}
 
 static void
@@ -1128,12 +1278,9 @@ destroy_chart(MeminfoChart *mc)
 	{
 	if (! mc->chart)
 		return;
-	gkrellm_monitor_height_adjust( - mc->chart->h);
-	gkrellm_destroy_chart(mc->chart);
-	g_free(mc->chart);
+	gkrellm_chart_destroy(mc->chart);
 	mc->chart = NULL;
 	mc->enabled = FALSE;
-	gtk_widget_hide(mc->vbox);
 	}
 
 static void
@@ -1143,13 +1290,25 @@ create_chart(MeminfoChart *mc, gint first_create)
 
 	if (first_create)
 		mc->chart = gkrellm_chart_new0();
+	cp = mc->chart;
 
-	if ((cp = mc->chart) == NULL)
-		return;
-	cp->h = chart_height;
-	gkrellm_create_chart(mc->vbox, cp, DEFAULT_STYLE_ID);
-	gkrellm_monitor_height_adjust(cp->h);
-	gkrellm_alloc_chart_data(cp);
+	gkrellm_set_chart_height_default(cp, DEFAULT_SWAP_CHART_HEIGHT);
+	gkrellm_chart_create(mc->vbox, mon_swap, cp, &mc->chart_config);
+	mc->out_cd = gkrellm_add_default_chartdata(cp, _("Swap Out"));
+	mc->in_cd = gkrellm_add_default_chartdata(cp, _("Swap In"));
+	gkrellm_set_draw_chart_function(cp, refresh_chart, mc);
+    gkrellm_chartconfig_fixed_grids_connect(cp->config,
+                setup_scaling, mc);
+    gkrellm_chartconfig_grid_resolution_connect(cp->config,
+                setup_scaling, mc);
+    gkrellm_chartconfig_grid_resolution_adjustment(cp->config, TRUE,
+                0, (gfloat) MIN_GRID_RES, (gfloat) MAX_GRID_RES, 0, 0, 0, 70);
+    gkrellm_chartconfig_grid_resolution_label(cp->config,
+                _("Swap in/out pages per sec"));
+    if (gkrellm_get_chartconfig_grid_resolution(cp->config) < MIN_GRID_RES)
+        gkrellm_set_chartconfig_grid_resolution(cp->config, DEFAULT_GRID_RES);
+
+	gkrellm_alloc_chartdata(cp);
 
 	if (first_create)
 		{
@@ -1165,44 +1324,16 @@ create_chart(MeminfoChart *mc, gint first_create)
 	ascent = 0;
 	}
 
-static void
-create_panel(GtkWidget *vbox, MeminfoMeter *mm, gint first_create)
+static gint
+cb_panel_config(GtkWidget *widget, GdkEventButton *ev)
 	{
-	Panel		*p;
-	TextStyle	*ts;
-	Style		*style;
+	if (ev->button == 3)
+		gkrellm_open_config_window(mon_mem);
+	return TRUE;
+	}
 
-	(*read_system_meminfo)();
-	if (first_create)
-		mm->panel = gkrellm_panel_new0();
-	else
-		{
-		gkrellm_destroy_decal_list(mm->panel);
-		gkrellm_destroy_krell_list(mm->panel);
-		}
-	p = mm->panel;
-
-	if (mm == &swap)
-		{
-		style = gkrellm_panel_style(DEFAULT_STYLE_ID);
-		mm->krell_delta = gkrellm_create_krell(p,
-					gkrellm_krell_panel_image(DEFAULT_STYLE_ID), style);
-		}
-
-	style = gkrellm_meter_style(mm->style_id);
-	mm->krell_used = gkrellm_create_krell(p,
-				gkrellm_krell_meter_image(mm->style_id), style);
-	p->textstyle = gkrellm_meter_textstyle(mm->style_id);
-	mm->decal_label = gkrellm_create_decal_text(p,
-			(mm == &swap) ? "Sp8" : "M8", p->textstyle, style, -1, -1, -1);
-	gkrellm_configure_panel(p, NULL, style);
-	gkrellm_create_panel(vbox, p, gkrellm_bg_meter_image(mm->style_id));
-	ts = &mm->decal_label->text_style;
-	mm->x_label = x_label_position(p->label->position, mm->decal_label->w,
-				gdk_string_width(ts->font, mm->label), 0);
-	draw_decal_label(mm, 0);
-
-	if (first_create)
+static void
+connect_panel_signals(Panel *p, MeminfoMeter *mm)
 		{
 		gtk_signal_connect(GTK_OBJECT (p->drawing_area),
 				"expose_event", (GtkSignalFunc) meminfo_expose_event, NULL);
@@ -1212,18 +1343,133 @@ create_panel(GtkWidget *vbox, MeminfoMeter *mm, gint first_create)
 			"button_release_event", (GtkSignalFunc) cb_panel_release, NULL);
 		gtk_signal_connect(GTK_OBJECT(p->drawing_area),
 				"motion_notify_event", (GtkSignalFunc) cb_panel_motion, NULL);
+		gtk_signal_connect(GTK_OBJECT(p->drawing_area),
+				"button_press_event", (GtkSignalFunc) cb_panel_config, NULL);
+		gtk_signal_connect(GTK_OBJECT(p->drawing_area),
+				"enter_notify_event", (GtkSignalFunc) cb_panel_enter, mm);
+		gtk_signal_connect(GTK_OBJECT(p->drawing_area),
+				"leave_notify_event", (GtkSignalFunc) cb_panel_leave, mm);
 		}
+
+static void
+create_mem_panel(GtkWidget *vbox, gint first_create)
+	{
+	GdkImlibImage	*im = NULL;
+	MeminfoMeter	*mm;
+	Panel			*p;
+	TextStyle		*ts;
+	Style			*s;
+
+	mm = &mem;
+	if (first_create)
+		mm->panel = gkrellm_panel_new0();
+	p = mm->panel;
+
+	/* I've got two extra krells for buffers and cache.  Use the #included
+	|  images unless themer has customized.
+	*/
+	s = gkrellm_copy_style(gkrellm_meter_style(mm->style_id));
+	s->krell_yoff = 0;
+	s->krell_depth = 1;
+	s->krell_x_hot = -1;
+	gkrellm_get_gkrellmrc_integer("mem_krell_buffers_depth", &s->krell_depth);
+	gkrellm_get_gkrellmrc_integer("mem_krell_buffers_x_hot", &s->krell_x_hot);
+	gkrellm_get_gkrellmrc_integer("mem_krell_buffers_yoff", &s->krell_yoff);
+	gkrellm_load_image("krell_buffers", krell_buffers_xpm, &im,MEM_STYLE_NAME);
+	mm->krell_buffers = gkrellm_create_krell(p, im, s);
+	gkrellm_monotonic_krell_values(mm->krell_buffers, FALSE);
+
+	s->krell_yoff = 0;
+	s->krell_depth = 1;
+	s->krell_x_hot = -1;
+	gkrellm_get_gkrellmrc_integer("mem_krell_cache_depth", &s->krell_depth);
+	gkrellm_get_gkrellmrc_integer("mem_krell_cache_x_hot", &s->krell_x_hot);
+	gkrellm_get_gkrellmrc_integer("mem_krell_cache_yoff", &s->krell_yoff);
+	gkrellm_load_image("krell_cache", krell_cache_xpm, &im, MEM_STYLE_NAME);
+	mm->krell_cache = gkrellm_create_krell(p, im, s);
+	gkrellm_monotonic_krell_values(mm->krell_cache, FALSE);
+
+	/* Unlike the style pointer passed to gkrellm_panel_configure(), the krells
+	|  don't need the style to persist.
+	*/
+	g_free(s);
+	if (im)
+		gdk_imlib_kill_image(im);
+
+	s = gkrellm_meter_style(mm->style_id);
+	mm->krell_used = gkrellm_create_krell(p,
+						gkrellm_krell_meter_image(mm->style_id), s);
+	gkrellm_monotonic_krell_values(mm->krell_used, FALSE);
+
+	mm->decal_label = gkrellm_create_decal_text(p, "M8",
+				gkrellm_meter_textstyle(mm->style_id), s, -1, -1, -1);
+	gkrellm_panel_configure(p, NULL, s);
+	gkrellm_panel_create(vbox, mon_mem, p);
+	ts = &mm->decal_label->text_style;
+	mm->x_label = x_label_position(p->label->position, mm->decal_label->w,
+				gdk_string_width(ts->font, mm->label), 0);
+	draw_decal_label(mm, 0);
+
+	if (first_create)
+		connect_panel_signals(p, mm);
 	gkrellm_setup_launcher(p, &mm->launch, METER_PANEL_TYPE, 4);
 
-	if (mm->enabled)
-		gkrellm_monitor_height_adjust(p->h);
-	else
-		gtk_widget_hide(p->hbox);
+	if (!mm->enabled)
+		gkrellm_panel_hide(p);
+	if (!mem.all_krells)
+		{
+		gkrellm_remove_krell(mem.panel, mem.krell_buffers);
+		gkrellm_remove_krell(mem.panel, mem.krell_cache);
+		}
 	}
 
-  /* The mem and swap meters are a unit (for looks and because some themes
-  |  assume that), so I stick the swap_chart just above the mem meter.
-  |  This code assumes mem is stacked on top of swap (ie mem created first)
+static void
+create_swap_panel(GtkWidget *vbox, gint first_create)
+	{
+	MeminfoMeter	*mm;
+	Panel			*p;
+	Style			*style;
+	TextStyle		*ts;
+
+	mm = &swap;
+	if (first_create)
+		mm->panel = gkrellm_panel_new0();
+	p = mm->panel;
+
+	/* Need a chart styled krell on the swap meter panel
+	*/
+	mm->krell_delta = gkrellm_create_krell(p,
+			gkrellm_krell_panel_image(DEFAULT_STYLE_ID),
+			gkrellm_panel_style(DEFAULT_STYLE_ID));
+
+	style = gkrellm_meter_style(mm->style_id);
+	mm->krell_used = gkrellm_create_krell(p,
+				gkrellm_krell_meter_image(mm->style_id), style);
+	gkrellm_monotonic_krell_values(mm->krell_used, FALSE);
+
+	mm->decal_label = gkrellm_create_decal_text(p, "Sp8",
+				gkrellm_meter_textstyle(mm->style_id), style, -1, -1, -1);
+
+	gkrellm_panel_configure(p, NULL, style);
+	gkrellm_panel_create(vbox, mon_swap, p);
+
+	ts = &mm->decal_label->text_style;
+	mm->x_label = x_label_position(p->label->position, mm->decal_label->w,
+				gdk_string_width(ts->font, mm->label), 0);
+	draw_decal_label(mm, 0);
+
+	if (first_create)
+		connect_panel_signals(p, mm);
+
+	gkrellm_setup_launcher(p, &mm->launch, METER_PANEL_TYPE, 4);
+
+	if (!mm->enabled)
+		gkrellm_panel_hide(p);
+	}
+
+  /* No separate swap monitor create function.  Use create_mem() to create
+  |  swap chart, mem meter, and swap meter so they will all be a unit in
+  |  the same vbox.
   */
 static void
 create_mem(GtkWidget *vbox, gint first_create)
@@ -1231,38 +1477,34 @@ create_mem(GtkWidget *vbox, gint first_create)
 	if (first_create)
 		{
 		mem.label = _("Mem");
-
-		swap_chart.vbox = gtk_vbox_new(FALSE, 0);
-		gtk_container_add(GTK_CONTAINER(vbox), swap_chart.vbox);
-		}
-	create_panel(vbox, &mem, first_create);
-	}
-
-static void
-create_swap(GtkWidget *vbox, gint first_create)
-	{
-	if (first_create)
 		swap.label = _("Swap");
 
+		swap_chart.vbox = gtk_vbox_new(FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(vbox), swap_chart.vbox, FALSE, FALSE, 0);
+
+		(*read_system_meminfo)();
+		}
 	if (swap_chart.enabled)
 		create_chart(&swap_chart, first_create);
-
-	create_panel(vbox, &swap, first_create);
-	setup_scaling(&swap_chart, &swap);
+	create_mem_panel(vbox, first_create);
+	create_swap_panel(vbox, first_create);
+	setup_scaling(NULL, &swap_chart);
 	}
 
+
 #define	MEM_CONFIG_KEYWORD	"meminfo"
 
 static void
 save_meminfo_config(FILE *f)
 	{
-	fprintf(f, "%s mem_meter %d %d\n", MEM_CONFIG_KEYWORD,
-				mem.enabled, mem.label_is_data);
+	fprintf(f, "%s mem_meter %d %d %d\n", MEM_CONFIG_KEYWORD,
+				mem.enabled, mem.label_is_data, mem.all_krells);
 	fprintf(f, "%s swap_meter %d %d\n", MEM_CONFIG_KEYWORD,
 				swap.enabled, swap.label_is_data);
-	fprintf(f, "%s swap_chart %d %d %d\n", MEM_CONFIG_KEYWORD,
-			swap_chart.enabled, swap_chart.resolution, swap_chart.extra_info);
-	fprintf(f, "%s chart_height %d\n", MEM_CONFIG_KEYWORD, chart_height);
+	fprintf(f, "%s swap_chart %d %d\n", MEM_CONFIG_KEYWORD,
+				swap_chart.enabled, swap_chart.extra_info);
+	gkrellm_save_chartconfig(f, swap_chart.chart_config,
+				MEM_CONFIG_KEYWORD, NULL);
 
 	fprintf(f, "%s mem_launch %s\n", MEM_CONFIG_KEYWORD,
 				mem.launch.command);
@@ -1276,6 +1518,8 @@ save_meminfo_config(FILE *f)
 				swap.launch.tooltip_comment);
 	fprintf(f, "%s swap_data_format %s\n", MEM_CONFIG_KEYWORD,
 				swap.data_format);
+
+	fprintf(f, "%s text_format %s\n", MEM_CONFIG_KEYWORD, text_format);
 	}
 
 static void
@@ -1289,7 +1533,8 @@ load_meminfo_config(gchar *arg)
 		{
 		if (strcmp(config, "mem_meter") == 0)
 			{
-			sscanf(item, "%d %d", &mem.enabled, &mem.label_is_data);
+			sscanf(item, "%d %d %d", &mem.enabled,
+					&mem.label_is_data, &mem.all_krells);
 			if (mem.label_is_data)
 				mem.restore_label = TRUE;
 			}
@@ -1300,32 +1545,31 @@ load_meminfo_config(gchar *arg)
 				swap.restore_label = TRUE;
 			}
 		else if (strcmp(config, "swap_chart") == 0)
-			sscanf(item, "%d %d %d", &swap_chart.enabled,
-			&swap_chart.resolution, &swap_chart.extra_info);
-		else if (strcmp(config, "chart_height") == 0)
-			sscanf(item, "%d", &chart_height);
-		else if (strcmp(config, "mem_launch") == 0)
+			sscanf(item, "%d %d", &swap_chart.enabled, &swap_chart.extra_info);
+		else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
+			gkrellm_load_chartconfig(&swap_chart.chart_config, item, 2);
+		else if (!strcmp(config, "mem_launch"))
 			mem.launch.command = g_strdup(item);
-		else if (strcmp(config, "mem_tooltip") == 0)
+		else if (!strcmp(config, "mem_tooltip"))
 			mem.launch.tooltip_comment = g_strdup(item);
-		else if (strcmp(config, "mem_data_format") == 0)
+		else if (!strcmp(config, "mem_data_format"))
 			gkrellm_dup_string(&mem.data_format, item);
-		else if (strcmp(config, "swap_launch") == 0)
+		else if (!strcmp(config, "swap_launch"))
 			swap.launch.command = g_strdup(item);
-		else if (strcmp(config, "swap_tooltip") == 0)
+		else if (!strcmp(config, "swap_tooltip"))
 			swap.launch.tooltip_comment = g_strdup(item);
-		else if (strcmp(config, "swap_data_format") == 0)
+		else if (!strcmp(config, "swap_data_format"))
 			gkrellm_dup_string(&swap.data_format, item);
+		else if (!strcmp(config, "text_format"))
+			gkrellm_dup_string(&text_format, item);
 		}
 	}
 
 /* --------------------------------------------------------------------- */
 static GtkWidget	*mem_meter_enable_button,
 					*swap_meter_enable_button,
-					*swap_chart_enable_button;
-
-static GtkWidget	*swap_chart_spin_button,
-					*height_spin_button;
+					*swap_chart_enable_button,
+					*all_krells_enable_button;
 
 static GtkWidget	*mem_launch_entry,
 					*mem_tooltip_entry,
@@ -1335,116 +1579,161 @@ static GtkWidget	*mem_launch_entry,
 static GtkWidget	*mem_format_combo,
 					*swap_format_combo;
 
+static GtkWidget	*text_format_combo;
+
 static void
-apply_meminfo_config()
+apply_meminfo_config(void)
 	{
 	gchar	*s;
-	gint	new_enabled, new_height;
+	gint	new_enabled;
 
 	s = gkrellm_entry_get_text(&(GTK_COMBO(mem_format_combo)->entry));
 	gkrellm_dup_string(&mem.data_format, s);
 	s = gkrellm_entry_get_text(&(GTK_COMBO(swap_format_combo)->entry));
 	gkrellm_dup_string(&swap.data_format, s);
 
+	s = gkrellm_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
+	gkrellm_dup_string(&text_format, s);
+
 	gkrellm_apply_launcher(&mem_launch_entry, &mem_tooltip_entry, mem.panel,
 				&mem.launch, gkrellm_launch_button_cb);
 	gkrellm_apply_launcher(&swap_launch_entry, &swap_tooltip_entry, swap.panel,
 				&swap.launch, gkrellm_launch_button_cb);
 
 	new_enabled = GTK_TOGGLE_BUTTON(mem_meter_enable_button)->active;
-	gkrellm_enable_visibility(new_enabled, &mem.enabled,
-					mem.panel->hbox, mem.panel->h);
+	gkrellm_panel_enable_visibility(mem.panel, new_enabled, &mem.enabled);
 	new_enabled = GTK_TOGGLE_BUTTON(swap_meter_enable_button)->active;
-	gkrellm_enable_visibility(new_enabled, &swap.enabled,
-					swap.panel->hbox, swap.panel->h);
+	gkrellm_panel_enable_visibility(swap.panel, new_enabled, &swap.enabled);
+	if (swap.enabled)
+		show_spacers(mon_swap);
+	else
+		hide_spacers(mon_swap);
+	if (mem.enabled)
+		show_spacers(mon_mem);
+	else
+		hide_spacers(mon_mem);
 
-	new_height = gtk_spin_button_get_value_as_int(
-				GTK_SPIN_BUTTON(height_spin_button));
-	if (new_height != chart_height)
-		{
-		if (swap_chart.enabled)
-			destroy_chart(&swap_chart);
-		}
-	chart_height = new_height;
-
-	swap_chart.resolution = gtk_spin_button_get_value_as_int(
-				GTK_SPIN_BUTTON(swap_chart_spin_button));
 	new_enabled = GTK_TOGGLE_BUTTON(swap_chart_enable_button)->active;
 	if (new_enabled && ! swap_chart.enabled)
 		create_chart(&swap_chart, TRUE);
 	else if (! new_enabled && swap_chart.enabled)
 		destroy_chart(&swap_chart);
 
-	setup_scaling(&swap_chart, &swap);
-	}
-
-static gint     res_map[] =
-	{ 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000, 100000 };
-
-static void
-cb_resolution(GtkWidget *widget, GtkSpinButton *spin)
-	{
-	gint    	value;
-	static gint	temp;
-
-	value = gtk_spin_button_get_value_as_int(spin);
-	if (value != temp) /* Avoid recursion */
+	new_enabled = GTK_TOGGLE_BUTTON(all_krells_enable_button)->active;
+	if (new_enabled && !mem.all_krells)
 		{
-		value = map_1_2_5(value, res_map, sizeof(res_map) / sizeof(gint));
-		temp = value;
-		gtk_spin_button_set_value(spin, (gfloat) value);
+		gkrellm_insert_krell(mem.panel, mem.krell_buffers, FALSE);
+		gkrellm_insert_krell(mem.panel, mem.krell_cache, FALSE);
 		}
+	else if (!new_enabled && mem.all_krells)
+		{
+		gkrellm_remove_krell(mem.panel, mem.krell_buffers);
+		gkrellm_remove_krell(mem.panel, mem.krell_cache);
+		}
+	mem.all_krells = new_enabled;
+	setup_scaling(NULL, &swap_chart);
 	}
 
+#define	DEFAULT_TEXT_FORMAT	"$T"
+
+static gchar	*mem_info_text[] =
+{
+N_("<b>Used and Free\n"),
+N_("The used and free memory here are calculated from the kernel reported\n"
+"used and free by subtracting or adding the buffers and cache memory.  See\n"
+"the README and compare to the \"-/+ buffers/cache:\" line from the free\n"
+"command.  If you show three memory krells, the kernel \"raw free\" is\n"
+"the space after the rightmost krell.\n"),
+"\n",
+N_("<b>Chart Labels\n"),
+N_("Substitution variables for the format string for chart labels:\n"),
+N_("\t$M    maximum chart value\n"),
+N_("\t$T    total swap in blocks + swap out blocks\n"),
+N_("\t$i    swap in blocks\n"),
+N_("\t$o    swap out blocks\n"),
+"\n",
+N_("<b>Panel Labels\n"),
+N_("Substition variables for the format string for the Mem and Swap\n"
+"panels (a MiB is a binary megabyte - 2^20):\n"),
+
+N_("For memory and swap:\n"),
+N_("\t$t    total MiB\n"),
+N_("\t$u    used MiB\n"),
+N_("\t$f    free MiB\n"),
+N_("\t$U    used %\n"),
+N_("\t$F    free %\n"),
+N_("\t$l    the panel label"),
+"\n",
+N_("For memory only:\n"),
+N_("\t$s    shared MiB\n"),
+N_("\t$b    buffered MiB\n"),
+N_("\t$c    cached MiB\n"),
+"\n",
+N_("<b>Mouse Button Actions:\n"),
+N_("<b>\tMiddle "),
+N_("click on a panel to scroll a programmable display of\n"
+"\t\tof memory or swap usage.\n")
+
+
+};
+
 static void
 create_meminfo_tab(GtkWidget *tab_vbox)
 	{
 	GtkWidget		*tabs;
-	GtkWidget		*vbox;
+	GtkWidget		*vbox, *vbox1;
 	GtkWidget		*table;
 	GtkWidget		*hbox;
-	GtkWidget		*label;
-	GtkWidget		*sep;
+	GtkWidget		*text, *label;
 	GList			*list;
+	gint			i;
 
 	tabs = gtk_notebook_new();
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
 	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);
 
-/* --Setup Tab */
-	vbox = gkrellm_create_tab(tabs, _("Setup"));
-
-    gkrellm_check_button(vbox, &swap_chart_enable_button, swap_chart.enabled,
-				 TRUE, 0, _("Enable swap pages in/out chart"));
-    gkrellm_check_button(vbox, &mem_meter_enable_button, mem.enabled,
-				TRUE, 0, _("Enable memory meter"));
-    gkrellm_check_button(vbox, &swap_meter_enable_button, swap.enabled,
-				TRUE, 0, _("Enable swap meter"));
-
-	gkrellm_spin_button(vbox, &swap_chart_spin_button,
-			(gfloat) swap_chart.resolution,
-			20.0, 100000.0, 1.0, 1.0, 0, 60, cb_resolution, NULL,
-			FALSE, _("Swap chart resolution in pages/sec per grid"));
-
-	gkrellm_spin_button(vbox, &height_spin_button, (gfloat) chart_height,
-			10.0, (gfloat) GK.max_chart_height, 1.0, 5.0, 0, 60, NULL, NULL,
-			FALSE, _("Swap chart height"));
-
 /* --Options Tab */
-	vbox = gkrellm_create_tab(tabs, _("Options"));
-	label = gtk_label_new(
-_("A middle or right click in the Mem or Swap panel toggles a display of\n"
-"memory or swap capacities in a format controlled by a user supplied format\n"
-"string.  For memory and swap, $t is total MiB, $u is used MiB, $f is free MiB,\n"
-"$U is used %, $F is free %, and $l is the panel label.\n"
-"For memory only, $s is shared MiB, $b is buffered MiB, and $c is cached MiB.\n"
-"MiB is a binary megabyte (2^20)."));
-	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 2);
-	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	vbox = gkrellm_create_framed_tab(tabs, _("Options"));
 
+	vbox1 = gkrellm_framed_vbox(vbox, _("Swap"), 4, FALSE, 0, 2);
+    gkrellm_check_button(vbox1, &swap_chart_enable_button, swap_chart.enabled,
+				 FALSE, 0, _("Enable swap pages in/out chart"));
+    gkrellm_check_button(vbox1, &swap_meter_enable_button, swap.enabled,
+				FALSE, 0, _("Enable swap meter"));
+
+	vbox1 = gkrellm_framed_vbox(vbox, _("Memory"), 4, FALSE, 0, 2);
+    gkrellm_check_button(vbox1, &mem_meter_enable_button, mem.enabled,
+				FALSE, 0, _("Enable memory meter"));
+    gkrellm_check_button(vbox1, &all_krells_enable_button, mem.all_krells,
+				FALSE, 0,
+		_("Show three memory krells:   [used | buffers | cache | raw free]"));
+
+/* -- Setup tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Setup"));
+
+	vbox1 = gkrellm_framed_vbox(vbox, _("Format String for Chart Labels"),
+			 4, FALSE, 0, 2);
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+	text_format_combo = gtk_combo_new();
+	gtk_widget_set_usize (GTK_WIDGET(text_format_combo), 300, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
+	list = NULL;
+	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
+	list = g_list_append(list, "$T\\C\\f$M");
+	list = g_list_append(list, "\\c\\f$M\\b$T");
+	list = g_list_append(list,
+				"\\ww\\C\\f$M\\D2\\f\\ai\\.$i\\D1\\f\\ao\\.$o");
+	list = g_list_append(list,
+				"\\ww\\C\\f$M\\D3\\f\\ai\\.$i\\D0\\f\\ao\\.$o");
+	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
+	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
+			text_format);
+
+	vbox1 = gkrellm_framed_vbox(vbox, _("Format String for Panel Labels"),
+			4, FALSE, 0, 2);
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
 	mem_format_combo = gtk_combo_new();
 	gtk_box_pack_start(GTK_BOX(hbox), mem_format_combo, FALSE, TRUE, 0);
 	list = NULL;
@@ -1455,11 +1744,11 @@ _("A middle or right click in the Mem or Swap panel toggles a display of\n"
 	gtk_combo_set_popdown_strings(GTK_COMBO(mem_format_combo), list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(mem_format_combo)->entry),
 			mem.data_format);
-	label = gtk_label_new(_("Format string for Mem data display"));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
+	label = gtk_label_new(_("Mem"));
+	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);
 
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
 	swap_format_combo = gtk_combo_new();
 	gtk_box_pack_start(GTK_BOX(hbox), swap_format_combo, FALSE, TRUE, 0);
 	list = NULL;
@@ -1469,17 +1758,24 @@ _("A middle or right click in the Mem or Swap panel toggles a display of\n"
 	gtk_combo_set_popdown_strings(GTK_COMBO(swap_format_combo), list);
 	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(swap_format_combo)->entry),
 			swap.data_format);
-	label = gtk_label_new(_("Format string for Swap data display"));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
+	label = gtk_label_new(_("Swap"));
+	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);
 
-	sep = gtk_hseparator_new();
-	gtk_box_pack_start(GTK_BOX(vbox), sep, TRUE, TRUE, 0);
-
-	table = gkrellm_launcher_table_new(vbox, 2);
+	vbox1 = gkrellm_framed_vbox_end(vbox, _("Launch Commands"), 4, FALSE, 0, 2);
+	table = gkrellm_launcher_table_new(vbox1, 2);
 	gkrellm_config_launcher(table, 0,  &mem_launch_entry, &mem_tooltip_entry,
 		_("Mem"), &(mem.launch));
 	gkrellm_config_launcher(table, 1,  &swap_launch_entry, &swap_tooltip_entry,
 		_("Swap"), &(swap.launch));
+
+/* --Info tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Info"));
+	text = gkrellm_scrolled_text(vbox, NULL,
+				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	for (i = 0; i < sizeof(mem_info_text)/sizeof(gchar *); ++i)
+		gkrellm_add_info_text_string(text, _(mem_info_text[i]));
 	}
 
 
@@ -1489,7 +1785,7 @@ _("A middle or right click in the Mem or Swap panel toggles a display of\n"
   |  a common routine is used, it is entered in only one of the Monitor
   |  structures, and NULL is entered in the others.
   */
-static Monitor	mon_mem =
+static Monitor	monitor_mem =
 	{
 	N_("Memory"),			/* Name, for config tab.	*/
 	MON_MEM,					/* Id,  0 if a plugin		*/
@@ -1515,21 +1811,24 @@ static Monitor	mon_mem =
 Monitor *
 init_mem_monitor(void)
 	{
-	mon_mem.name = _(mon_mem.name);
-	mem.style_id = gkrellm_add_meter_style(&mon_mem, MEM_STYLE_NAME);
+	monitor_mem.name = _(monitor_mem.name);
+	mon_mem = &monitor_mem;
+	mem.style_id = gkrellm_add_meter_style(mon_mem, MEM_STYLE_NAME);
+
 	mem.data_format = g_strdup(DEFAULT_FORMAT);
 	mem.enabled = TRUE;
 
 	if (setup_meminfo_interface())
-		return &mon_mem;
+		return &monitor_mem;
 	return NULL;
 	}
 
-static Monitor	mon_swap =
+
+static Monitor	monitor_swap =
 	{
 	NULL,			/* Name, for config tab. Done in mon_mem*/
 	MON_SWAP,		/* Id,  0 if a plugin		*/
-	create_swap,	/* The create function		*/
+	NULL,			/* The create function		*/
 	NULL,			/* The update function		*/
 	NULL,			/* The config tab create function	*/
 	NULL,			/* Apply the config function		*/
@@ -1547,16 +1846,17 @@ static Monitor	mon_swap =
 Monitor *
 init_swap_monitor(void)
 	{
-	swap.style_id = gkrellm_add_meter_style(&mon_swap, SWAP_STYLE_NAME);
+	mon_swap = &monitor_swap;
+	swap.style_id = gkrellm_add_meter_style(mon_swap, SWAP_STYLE_NAME);
+
 	swap.data_format = g_strdup(DEFAULT_FORMAT);
 	swap.enabled = TRUE;
 
-	/* See comments above about swap_chart wrt the meters.
-	*/
-	swap_chart.resolution = 1000;
 	swap_chart.enabled = FALSE;
+	swap_chart.extra_info = TRUE;
+	text_format = g_strdup(DEFAULT_TEXT_FORMAT);
 
-	if (setup_meminfo_interface())
-		return &mon_swap;
+	if (setup_meminfo_interface())	/* XXX */
+		return &monitor_swap;
 	return NULL;
 	}
