@@ -19,10 +19,11 @@
 */
 
 /*
-|  1/23/2001 Hajimu UMEMOTO added init_freebsd_proc(), improved
-|				read_freebsd_proc()
-| 10/12/2000 NetBSD code contributed by Anthony Mallet <metall@ficus.yi.org>
-| 2/25/2000  FreeBSD code contributed by Hajimu UMEMOTO ume@mahoroba.org
+|  4/22/2001  Solaris code contributed by Daisuke Yabuki <dxy@acm.org>
+|  1/23/2001  Hajimu UMEMOTO added init_freebsd_proc(), improved
+|               read_freebsd_proc()
+| 10/12/2000  NetBSD code contributed by Anthony Mallet <metall@ficus.yi.org>
+|  2/25/2000  FreeBSD code contributed by Hajimu UMEMOTO ume@mahoroba.org
 */
 
 #include "gkrellm.h"
@@ -35,9 +36,10 @@ typedef struct
 	{
 	GtkWidget	*vbox;
 	Chart		*chart;
+	ChartData	*forks_cd;
+	ChartConfig	*chart_config;
 	gint		enabled;
-	gint		height;
-	gint		extra_info;
+	gboolean	extra_info;
 	gint		save_label_position;
 	Decal		*sensor_decal,
 				*fan_decal;
@@ -225,7 +227,7 @@ read_freebsd_proc()
 #define	PROC_LOADAVG_FILE	"/proc/loadavg"
 
 static void
-read_linux_proc()
+read_linux_proc(void)
 	{
 	FILE			*f;
 	gchar			buf[160], loadbuf[16];
@@ -334,6 +336,74 @@ read_netbsd_proc()
 
 #endif /* __NetBSD__ || __OpenBSD__ */
 
+/* ----- Solaris ----------------------------------------------------- */
+#if defined(__solaris__) 
+
+#include <utmp.h>
+#include <sys/types.h>
+#include <dirent.h>
+#include <sys/loadavg.h>
+#include <kstat.h>
+#include <fcntl.h>
+#include <kvm.h>
+#include <sys/sysinfo.h>
+
+static void
+read_solaris_proc() 
+{
+
+    static struct utmp *utmpp;
+    double avenrun[LOADAVG_NSTATS];
+
+    int last_pid;
+    extern kstat_ctl_t *kc;
+    kstat_t *ksp;
+    kstat_named_t *knp;
+
+    extern kvm_t *kd;
+    extern struct nlist nl[];
+
+    if (!GK.second_tick) /* Only one read per second */
+        return;
+
+    if (getloadavg(avenrun, LOADAVG_NSTATS) > 0)
+        proc.fload = avenrun[LOADAVG_1MIN];
+
+    if (kstat_chain_update(kc) == -1) {
+        perror("kstat_chain_update");
+        return;
+    }
+    ksp = kstat_lookup(kc, "unix", -1, "system_misc");
+    if (ksp && kstat_read(kc, ksp, NULL) >= 0) {
+        knp = (kstat_named_t *)kstat_data_lookup(ksp, "nproc");
+        if (knp) { 
+            proc.n_processes = knp->value.ui32;
+        }
+    }
+
+    if (kd) {
+        if (kvm_kread(kd, nl[0].n_value, (char *)&last_pid, sizeof(int)) != -1)
+            proc.n_forks = last_pid;
+    } else {
+        proc.n_forks = 0;
+    }
+
+    if (!GK.five_second_tick || !proc.extra_info) 
+        return;
+
+    proc.n_users = 0;
+    setutent();
+    while ((utmpp = getutent()) != NULL) {
+        if (utmpp->ut_type == USER_PROCESS && utmpp->ut_name[0] != '\0')
+            proc.n_users++;
+    }    
+
+    /* NOTE: code to get 'n_running' is not implemented (stays untouched).
+     * but it wouldn't do any harm since nobody seems to refer to it.
+     */
+}
+
+#endif /* __solaris__ */
 
 /* ----- Others ------------------------------------------------------ */
 #if defined(USE_LIBGTOP)
@@ -341,7 +411,7 @@ read_netbsd_proc()
 #include <glibtop/loadavg.h>
 
 static void
-read_glibtop_proc()
+read_glibtop_proc(void)
 	{
 	glibtop_loadavg		glt_loadavg;
 
@@ -363,7 +433,7 @@ read_glibtop_proc()
 
 /* ----- Pick a system interface ----------------------------------------- */
 static gint
-setup_proc_interface()
+setup_proc_interface(void)
 	{
 #if defined(__FreeBSD__)
 	init_freebsd_proc();
@@ -372,6 +442,8 @@ setup_proc_interface()
 	read_system_proc_info = read_linux_proc;
 #elif defined(__NetBSD__) || defined(__OpenBSD__)
 	read_system_proc_info = read_netbsd_proc;
+#elif defined(__solaris__)
+        read_system_proc_info = read_solaris_proc;
 #else
 	read_system_proc_info = read_glibtop_proc;
 #endif
@@ -380,63 +452,89 @@ setup_proc_interface()
 
 /* ======================================================================== */
 
+  /* Charts are integer only and load is a small real, so scale all load
+  |  reading by 100.
+  */
+#define	LOAD_SCALING	100.0
+
+static Monitor	*mon_proc;
+
 static Launcher	proc_launch;
 
-static gint		load_resolution;
-static gint		ascent;
 static gint		style_id;
 static gint		mapped_sensors;
 static gint		sensor_separate_mode;
+static gint		fork_scaling = 1;
+static gchar	*text_format;
 
 static void
-draw_proc_extra()
+format_chart_text(ProcMon *p, gchar *buf, gint size)
 	{
-	TextStyle	*ts, *ts_alt;
-	gint		x, x1, y;
-	gchar		buf[32];
+	Chart	*cp;
+	gchar	c, *s;
+	gint	len, value;
 
-	if (proc.chart == NULL)
-		return;
-	y = 4;
-	x1 = 4;
-	ts = gkrellm_chart_textstyle(style_id);
-	ts_alt = gkrellm_chart_alt_textstyle(style_id);
-	if (ascent == 0)
-		ascent = gdk_char_height(ts->font, '8');
-	if (proc.n_processes > 0)
+	--size;
+	*buf = '\0';
+	cp = p->chart;
+	for (s = text_format; *s != '\0' && size > 0; ++s)
 		{
-		y += ascent;
-		snprintf(buf, sizeof(buf), "%d", proc.n_processes);
-		x1 = gkrellm_draw_chart_label(proc.chart, ts, x1, y, buf);
-		gkrellm_draw_chart_label(proc.chart, ts_alt, x1 + 2, y, _("procs"));
-		y += 2;
-		}
-	if (proc.n_users > 0)
-		{
-		y += ascent;
-		snprintf(buf, sizeof(buf), "%d", proc.n_users);
-		x = gkrellm_draw_chart_label(proc.chart, ts, 4, y, buf);
-		if (x < x1)
-			x = x1;
-		gkrellm_draw_chart_label(proc.chart, ts_alt, x + 2, y, _("users"));
+		len = 1;
+		if (*s == '$' && *(s + 1) != '\0')
+			{
+			value = -1;
+			if ((c = *(s + 1)) == 'L')
+				len = snprintf(buf, size, "%.1f",
+					(gfloat) gkrellm_get_chart_scalemax(cp) / LOAD_SCALING);
+			else if (c == 'F' && fork_scaling > 0)
+				len = snprintf(buf, size, "%d",
+						gkrellm_get_chart_scalemax(cp) / fork_scaling);
+			else if (c == 'l')
+				len = snprintf(buf, size, "%.1f", p->fload);
+			else if (c == 'p')
+				value = p->n_processes;
+			else if (c == 'u')
+				value = p->n_users;
+			else if (c == 'f')
+				value = gkrellm_get_current_chartdata(p->forks_cd)
+						/ fork_scaling;
+			if (value >= 0)
+				len = snprintf(buf, size, "%d", value);
+			++s;
+			}
+		else
+			*buf = *s;
+		size -= len;
+		buf += len;
 		}
+	*buf = '\0';	
 	}
 
+static void
+draw_proc_extra(void)
+	{
+	gchar		buf[128];
+
+	if (!proc.chart || !proc.extra_info)
+		return;
+	format_chart_text(&proc, buf, sizeof(buf));
+	gkrellm_draw_chart_text(proc.chart, style_id, buf);
+	}
 
 static void
 refresh_proc_chart(Chart *cp)
 	{
 	if (proc.enabled)
 		{
-		draw_proc_chart(cp);
+		gkrellm_draw_chartdata(cp);
+		if (proc.extra_info)
+			draw_proc_extra();
+		gkrellm_draw_chart_to_screen(cp);
 		}
-	if (proc.extra_info)
-		draw_proc_extra();
-	cp->need_redraw = FALSE;
 	}
 
 static void
-draw_sensor_decals()
+draw_sensor_decals(void)
 	{
 	Panel	*p		= proc.chart->panel;
 	gchar	*name	= "mb";
@@ -476,7 +574,7 @@ draw_sensor_decals()
 	}
 
 static void
-update_proc()
+update_proc(void)
 	{
 	Chart	*cp;
 	Panel	*p;
@@ -489,25 +587,21 @@ update_proc()
 	cp = proc.chart;
 	p = cp->panel;
 	gkrellm_update_krell(p, KRELL(p), proc.n_forks);
-	gkrellm_draw_layers(p);
+	gkrellm_draw_panel_layers(p);
 
 	if (GK.second_tick)
 		{
-		cp->prevOut = 0;
-
-		/* Scale the forks number.  See setup_proc_scaling().
+		/* Scale load since it is a small real and charts are integer only.
+		|  Scale the forks number by fork_scaling.  See setup_proc_scaling().
 		*/
-		load = (int) (100.0 * proc.fload);
-		store_proc_chart_data(cp, load, cp->private * proc.n_forks);
+		load = (int) (LOAD_SCALING * proc.fload);
+		gkrellm_store_chartdata(cp, 0, load, fork_scaling * proc.n_forks);
 		refresh_proc_chart(cp);
 		}
 	if (   (GK.two_second_tick && !sensor_separate_mode)
 		|| (GK.five_second_tick && sensor_separate_mode)
 	   )
 		draw_sensor_decals();
-
-	if (cp->need_redraw)
-		refresh_proc_chart(cp);
 	}
 
 
@@ -533,48 +627,59 @@ proc_expose_event(GtkWidget *widget, GdkEventExpose *ev)
 
 
 static gint
-cb_proc_extra(GtkWidget *widget, GdkEventButton *event)
+cb_proc_extra(GtkWidget *widget, GdkEventButton *ev)
 	{
-	if (event->button == 1)
+	if (ev->button == 1 && ev->type == GDK_BUTTON_PRESS)
 		{
-		proc.extra_info = 1 - proc.extra_info;
+		proc.extra_info = !proc.extra_info;
 		gkrellm_config_modified();
+		refresh_proc_chart(proc.chart);
 		}
-	refresh_proc_chart(proc.chart);
+	else if (   ev->button == 3
+			 || (ev->button == 1 && ev->type == GDK_2BUTTON_PRESS)
+			)
+		gkrellm_chartconfig_window_create(proc.chart);
 	return TRUE;
 	}
 
 static void
-setup_proc_scaling()
+setup_proc_scaling(void)
 	{
-	Chart	*cp		= proc.chart;
-	gint	grids, fork_factor, i;
+	Chart		*cp		= proc.chart;
+	gint		grids, res, new_fork_scaling;
 
-	if (cp == NULL)
+	if (!cp)
 		return;
-	grids = UC.fixed_scale ? UC.fixed_scale : FULL_SCALE_GRIDS;
-	cp->scale_min = load_resolution;
 
-	/* Since scale_min is set for load, set krell_full_scale explicitely
+	grids = gkrellm_get_chartconfig_fixed_grids(cp->config);
+	if (!grids)
+		grids = FULL_SCALE_GRIDS;
+
+	res = gkrellm_get_chartconfig_grid_resolution(cp->config);
+
+	/* Since grid_resolution is set for load, set krell_full_scale explicitely
 	|  to get what I want, which is 10 forks full scale.
-	|  Use private field to scale forks/sec to get hardwired 50 forks/sec
-	|  max on the charts.
+	|  When res or number of grids is changed, scale all fork data to keep a
+	|  fixed 50 forks/sec max on the chart.
 	*/
 	KRELL(cp->panel)->full_scale = 10;
-	fork_factor = grids * load_resolution / 50;
 
-	/* If load resolution changed, scale all fork data to keep the
+	new_fork_scaling = grids * res / 50;
+	if (new_fork_scaling < 1)		/* shouldn't happen... */
+		new_fork_scaling = 1;
+
+	/* If load grid_resolution changed, scale all fork data to keep the
 	|  constant 50 forks/sec.
 	*/
-	if (fork_factor != cp->private && cp->private > 0)
+	if (fork_scaling != new_fork_scaling)
 		{
-		for (i = 0; i < cp->w; ++i)
-			cp->pDataIn[i] = cp->pDataIn[i] * fork_factor / cp->private;
-		cp->prevIn = cp->prevIn * fork_factor / cp->private;
+		/* When called as a callback a chart refresh will follow, but I'll
+		|  need a rescale here.
+		*/
+		gkrellm_scale_chartdata(proc.forks_cd, new_fork_scaling, fork_scaling);
+		gkrellm_rescale_chart(proc.chart);
 		}
-	cp->private = fork_factor;
-	cp->scale_max = 0;
-	cp->maxval = 0;
+	fork_scaling = new_fork_scaling;
 	}
 
 static void
@@ -585,18 +690,59 @@ destroy_proc_monitor(void)
 	if (proc_launch.button)
 		gkrellm_destroy_button(proc_launch.button);
 	proc_launch.button = NULL;
-
-	g_free(cp->name);
-	gkrellm_monitor_height_adjust( - proc.height);
-	gkrellm_destroy_panel(cp->panel);
     proc_launch.tooltip = NULL;
-	g_free(cp->panel);
-	gkrellm_destroy_chart(cp);
-	g_free(cp);
+	gkrellm_chart_destroy(cp);
 	proc.chart = NULL;
 	proc.enabled = FALSE;
 	}
 
+static void
+cb_mb_temp_alert_trigger(Alert *alert, ProcMon *proc)
+	{
+	AlertDecal	*ad;
+	Decal		*d;
+
+	if (alert && proc && proc->chart)
+		{
+		ad = &alert->ad;
+		d = proc->sensor_decal;
+		if (d)
+			{
+			ad->x = d->x - 1;
+			ad->y = d->y - 1;
+			ad->w = d->w + 2;
+			ad->h = d->h + 2;
+			gkrellm_render_default_alert_decal(alert);
+			}
+		alert->panel = proc->chart->panel;
+		}
+	}
+
+static void
+cb_mb_fan_alert_trigger(Alert *alert, ProcMon *proc)
+	{
+	AlertDecal	*ad;
+	Decal		*d;
+
+	if (alert && proc && proc->chart)
+		{
+		ad = &alert->ad;
+		if (sensor_separate_mode)
+			d = proc->fan_decal;
+		else
+			d = proc->sensor_decal;
+		if (d)
+			{
+			ad->x = d->x - 1;
+			ad->y = d->y - 1;
+			ad->w = d->w + 2;
+			ad->h = d->h + 2;
+			gkrellm_render_default_alert_decal(alert);
+			}
+		alert->panel = proc->chart->panel;
+		}
+	}
+
   /* Next routine is same as in cpu.c - perhaps should make into one?*/
   /* How to decide when to make the sensor_decal and fan_decal visible.
   |  The sensor_decal can show temp values, fan values, or alternating
@@ -622,6 +768,7 @@ adjust_sensors_display(gint force)
 	{
 	Panel	*p;
 	Decal	*ds, *df;
+	Alert	*alert;
 	gint	position	= 0;
 
 	ds = proc.sensor_decal;
@@ -633,6 +780,20 @@ adjust_sensors_display(gint force)
 	*/
 	p = proc.chart->panel;
 	mapped_sensors = 0;
+	if (!GK.demo)
+		{
+		gkrellm_mapped_sensor_alert_connect("mb",
+				SENSOR_TEMPERATURE, cb_mb_temp_alert_trigger, &proc);
+		gkrellm_mapped_sensor_alert_connect("mb",
+				SENSOR_FAN, cb_mb_fan_alert_trigger, &proc);
+		}
+
+	/* If a fan alert is triggered, turn it off in case fan decal being used
+	|  is changed.  The alert will just retrigger at next fan update.
+	*/
+	alert = gkrellm_mapped_sensor_alert("mb", SENSOR_FAN);
+	gkrellm_reset_alert(alert);
+
 	if (get_mapped_sensor("mb", SENSOR_TEMPERATURE) || GK.demo)
 		mapped_sensors |= SENSOR_TEMPERATURE;
 	if (get_mapped_sensor("mb", SENSOR_FAN) || GK.demo)
@@ -670,76 +831,82 @@ adjust_sensors_display(gint force)
 			p->label->position = position;	/* original label was visible.   */
 		gkrellm_draw_panel_label(p, gkrellm_bg_panel_image(style_id));
 		draw_sensor_decals();
-		gkrellm_draw_layers(p);
+		gkrellm_draw_panel_layers(p);
 		return TRUE;
 		}
 	return FALSE;
 	}
 
-  /* Proc monitor shows processes (forks) per second and load average.
-  |  The scaling goes like this.  Load is read from /proc/loadavg as a
-  |  float ranging from 0 - n where n may be 1, 2, 3, ...  Basically,
-  |  something small.  So, I scale load by 100 and set scale_min to
-  |  100.  Now grid lines have units of 1.
-  */
+static gint
+cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
+	{
+	if (ev->button == 3)
+		gkrellm_open_config_window(mon_proc);
+	return TRUE;
+	}
+
 static void
 create_proc_monitor(GtkWidget *vbox, gint first_create)
 	{
-	Chart	*cp;
-	Panel	*p;
-	Style	*style;
+	Chart		*cp;
+	ChartData	*cd;
+	ChartConfig	*cf;
+	Panel		*p;
+	Style		*style;
 
 	if (first_create)
 		{
 		proc.chart = gkrellm_chart_new0();
 		proc.chart->panel = gkrellm_panel_new0();
-		proc.chart->name = g_strdup(_("Proc"));
-		}
-	else
-		{
-		gkrellm_destroy_decal_list(proc.chart->panel);
-		gkrellm_destroy_krell_list(proc.chart->panel);
 		}
 	cp = proc.chart;
 	p = cp->panel;
-	cp->scale_min = 100;		/* For load */
 
 	style = gkrellm_panel_style(style_id);
 	gkrellm_create_krell(p, gkrellm_krell_panel_image(style_id), style);
 
-	cp->h = UC.chart_height[MON_PROC];
+	gkrellm_chart_create(proc.vbox, mon_proc, cp, &proc.chart_config);
+	gkrellm_set_draw_chart_function(cp, refresh_proc_chart, proc.chart);
+	cd = gkrellm_add_default_chartdata(cp, _("Load"));
+	gkrellm_monotonic_chartdata(cd, FALSE);
+	gkrellm_set_chartdata_draw_style_default(cd, CHARTDATA_LINE);
+	proc.forks_cd = gkrellm_add_default_chartdata(cp, _("Forks"));
+	gkrellm_set_chartdata_flags(proc.forks_cd, CHARTDATA_ALLOW_HIDE);
 
-	gkrellm_create_chart(proc.vbox, cp, style_id);
+	cf = cp->config;
+	gkrellm_chartconfig_fixed_grids_connect(cf, setup_proc_scaling, NULL);
+	gkrellm_chartconfig_grid_resolution_connect(cf, setup_proc_scaling, NULL);
+	gkrellm_chartconfig_grid_resolution_adjustment(cf, FALSE,
+			LOAD_SCALING, 0.5, 5.0, 0.5, 0.5, 1, 50);
+	gkrellm_chartconfig_grid_resolution_label(cf,
+					_("Average process load per minute"));
 
-	p->textstyle = gkrellm_panel_textstyle(style_id);
+	gkrellm_alloc_chartdata(cp);
+	setup_proc_scaling();
 
 	/* I put motherboard temp on Proc panel (if temperature sensors found)
 	*/
 	sensor_create_decals(p, style_id,
 					&proc.sensor_decal, &proc.fan_decal);
 
-	gkrellm_configure_panel(p, cp->name, style);
+	gkrellm_panel_configure(p, _("Proc"), style);
 
-	gkrellm_create_panel(proc.vbox, p, gkrellm_bg_panel_image(style_id));
-	proc.height = cp->h + p->h;
-	gkrellm_monitor_height_adjust(proc.height);
+	gkrellm_panel_create(proc.vbox, mon_proc, p);
 
 	proc.save_label_position = p->label->position;
 	if (proc.sensor_decal)
 		adjust_sensors_display(TRUE);
 
-	gkrellm_alloc_chart_data(cp);
-	cp->type = 1;
-	setup_proc_scaling();
-
 	if (first_create)
 		{
 		gtk_signal_connect(GTK_OBJECT(cp->drawing_area), "expose_event",
-					(GtkSignalFunc) proc_expose_event, NULL);
+				(GtkSignalFunc) proc_expose_event, NULL);
 		gtk_signal_connect(GTK_OBJECT(p->drawing_area), "expose_event",
-					(GtkSignalFunc) proc_expose_event, NULL);
+				(GtkSignalFunc) proc_expose_event, NULL);
 		gtk_signal_connect(GTK_OBJECT(cp->drawing_area),"button_press_event",
-					(GtkSignalFunc) cb_proc_extra, NULL);
+				(GtkSignalFunc) cb_proc_extra, NULL);
+		gtk_signal_connect(GTK_OBJECT(p->drawing_area),
+				"button_press_event", (GtkSignalFunc) cb_panel_press, NULL);
 		}
 	else
 		refresh_proc_chart(cp);
@@ -749,14 +916,7 @@ create_proc_monitor(GtkWidget *vbox, gint first_create)
 static void
 create_proc(GtkWidget *vbox, gint first_create)
 	{
-	if (first_create)
-		{
-		proc.vbox = gtk_vbox_new(FALSE, 0);
-		gtk_container_add(GTK_CONTAINER(vbox), proc.vbox);
-		if (proc.enabled)
-			gtk_widget_show(proc.vbox);
-		}
-	ascent = 0;
+	proc.vbox = vbox;
 	if (proc.enabled)
 		create_proc_monitor(proc.vbox, first_create);
 	}
@@ -768,15 +928,16 @@ create_proc(GtkWidget *vbox, gint first_create)
 static void
 save_proc_config(FILE *f)
 	{
-	fprintf(f, "%s resolution %d\n", PROC_CONFIG_KEYWORD, load_resolution);
-	fprintf(f, "%s enable %d %d %d %d\n", PROC_CONFIG_KEYWORD,
-				proc.enabled, UC.proc_load_bar_graph,
-				UC.proc_clip_processes, proc.extra_info);
+	fprintf(f, "%s enable %d %d\n", PROC_CONFIG_KEYWORD,
+				proc.enabled, proc.extra_info);
 	fprintf(f, "%s launch %s\n", PROC_CONFIG_KEYWORD, proc_launch.command);
 	fprintf(f, "%s tooltip_comment %s\n", PROC_CONFIG_KEYWORD,
 				proc_launch.tooltip_comment);
 	fprintf(f, "%s sensor_mode %d\n", PROC_CONFIG_KEYWORD,
 				sensor_separate_mode);
+	fprintf(f, "%s text_format %s\n", PROC_CONFIG_KEYWORD, text_format);
+	gkrellm_save_chartconfig(f, proc.chart_config,
+				PROC_CONFIG_KEYWORD, NULL);
 	}
 
 static void
@@ -788,62 +949,47 @@ load_proc_config(gchar *arg)
 	n = sscanf(arg, "%31s %[^\n]", config, item);
 	if (n == 2)
 		{
-		if (strcmp(config, "resolution") == 0)
-			sscanf(item, "%d\n", &load_resolution);
-		else if (strcmp(config, "enable") == 0)
-			sscanf(item, "%d %d %d %d\n", &proc.enabled,
-					&UC.proc_load_bar_graph,
-					&UC.proc_clip_processes, &proc.extra_info);
-		else if (strcmp(config, "launch") == 0)
+		if (!strcmp(config, "enable"))
+			sscanf(item, "%d %d\n", &proc.enabled, &proc.extra_info);
+		else if (!strcmp(config, "launch"))
 			proc_launch.command = g_strdup(item);
-		else if (strcmp(config, "tooltip_comment") == 0)
+		else if (!strcmp(config, "tooltip_comment"))
 			proc_launch.tooltip_comment = g_strdup(item);
-		else if (strcmp(config, "sensor_mode") == 0)
+		else if (!strcmp(config, "sensor_mode"))
 			sscanf(item, "%d\n", &sensor_separate_mode);
+		else if (!strcmp(config, "text_format"))
+			gkrellm_dup_string(&text_format, item);
+		else if (!strcmp(config, GKRELLM_CHARTCONFIG_KEYWORD))
+			gkrellm_load_chartconfig(&proc.chart_config, item, 2);
 		}
 	}
 
 /* ---------------------------------------------------------------------- */
-static GtkWidget	*load_spin_button;
+#define DEFAULT_TEXT_FORMAT	"\\w88\\a$p\\f procs\\n\\e$u\\f users"
 
-static GtkWidget	*bar_graph_button,
-					*clip_proc_button,
-					*proc_enable_button,
+static GtkWidget	*proc_enable_button,
 					*sensor_separate_button;
 static GtkWidget	*proc_launch_entry,
 					*proc_tooltip_entry;
 
+static GtkWidget	*text_format_combo;
+
 static void
-apply_proc_config()
+apply_proc_config(void)
 	{
-	GtkSpinButton	*spin;
-	gfloat			f;
-	gint			pe;
+	gchar		*s;
+	gint		pe;
 
-	spin = GTK_SPIN_BUTTON(load_spin_button);
-	f = gtk_spin_button_get_value_as_float(spin);
-	load_resolution = (gint) (f * 100.0);
-
-	UC.proc_load_bar_graph = GTK_TOGGLE_BUTTON(bar_graph_button)->active;
-	UC.proc_clip_processes = GTK_TOGGLE_BUTTON(clip_proc_button)->active;
 	if (sensor_separate_button)
 		sensor_separate_mode =
 				GTK_TOGGLE_BUTTON(sensor_separate_button)->active;
 
 	pe = GTK_TOGGLE_BUTTON(proc_enable_button)->active;
 	if (pe && ! proc.enabled)
-		{
-		gtk_widget_show(proc.vbox);
 		create_proc_monitor(proc.vbox, TRUE);
-		}
 	else if (! pe && proc.enabled)
-		{
-		gtk_widget_hide(proc.vbox);
 		destroy_proc_monitor();
-		}
 	proc.enabled = pe;
-	if (proc.chart)
-		setup_proc_scaling();
 
 	if (proc.enabled)
 		{
@@ -857,34 +1003,81 @@ apply_proc_config()
 					gkrellm_launch_button_cb, &proc_launch, proc_launch.pad);
 			}
 		}
+	s = gkrellm_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
+	if (gkrellm_dup_string(&text_format, s))
+		gkrellm_refresh_chart(proc.chart);
 	}
 
+
+static gchar	*proc_info_text[] =
+{
+N_("<b>Proc Chart"),
+"\n",
+N_("The krell shows process forks with a full scale value of 10 forks.\n"),
+N_("While both load and fork data are drawn on the chart, the grid\n"
+"resolution can be set for load only.  The resolution per grid for forks is\n"
+"fixed at 10 when using the auto number of grids mode, and at 50 divided by\n"
+"the number of grids when using a fixed number of grids mode.\n"),
+"\n",
+N_("<b>Chart Labels\n"),
+N_("Substitution variables for the format string for chart labels:\n"),
+N_("\t$L    maximum chart value (load)\n"),
+N_("\t$F    maximum chart value (forks)\n"),
+N_("\t$l    load\n"),
+N_("\t$f    forks\n"),
+N_("\t$p    processes\n"),
+N_("\t$u    users\n")
+};
+
 static void
 create_proc_tab(GtkWidget *tab_vbox)
 	{
-	GtkWidget		*table;
+	GtkWidget	*tabs, *table, *vbox, *vbox1, *hbox, *text;
+	GList		*list;
+	gint		i;
 
-	gkrellm_check_button(tab_vbox, &proc_enable_button, proc.enabled, TRUE, 0,
+	tabs = gtk_notebook_new();
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
+	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);
+
+/* ---Options tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Setup"));
+
+	vbox1 = gkrellm_framed_vbox(vbox, _("Options"), 4, FALSE, 0, 2);
+	gkrellm_check_button(vbox1, &proc_enable_button, proc.enabled, FALSE, 4,
 			_("Enable Proc chart"));
 
-	gkrellm_check_button(tab_vbox, &bar_graph_button, UC.proc_load_bar_graph,
-			FALSE, 2, _("Draw load data as solid bars instead of a line"));
-	gkrellm_check_button(tab_vbox, &clip_proc_button, UC.proc_clip_processes,
-			FALSE, 2,
-			_("Clip process fork data (chart scaling uses load graph only)"));
 	if (gkrellm_sensors_available())
-		gkrellm_check_button(tab_vbox, &sensor_separate_button,
+		gkrellm_check_button(vbox1, &sensor_separate_button,
 					sensor_separate_mode, FALSE, 0,
 		_("Draw fan and temperature values separately (not alternating)."));
 
-	gkrellm_spin_button(tab_vbox, &load_spin_button,
-			(gfloat) load_resolution / 100, 0.5, 5.0, 0.5, 0.5, 1, 50,
-			NULL, NULL, FALSE,
-			_("Load resolution in average process load per minute per grid"));
+	vbox1 = gkrellm_framed_vbox(vbox, _("Format String for Chart Labels"),
+				4, FALSE, 0, 2);
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
+	text_format_combo = gtk_combo_new();
+	gtk_widget_set_usize (GTK_WIDGET(text_format_combo), 350, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
+	list = NULL;
+	list = g_list_append(list, _(DEFAULT_TEXT_FORMAT));
+	list = g_list_append(list,
+			_("\\f$L\\r\\f$F \\w88\\b\\p\\a$p\\f procs\\n\\e$u\\f users"));
+	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
+	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
+			text_format);
 
-	table = gkrellm_launcher_table_new(tab_vbox, 1);
+	vbox1 = gkrellm_framed_vbox_end(vbox, _("Launch Commands"), 4, FALSE, 0, 2);
+	table = gkrellm_launcher_table_new(vbox1, 1);
 	gkrellm_config_launcher(table, 0,  &proc_launch_entry, &proc_tooltip_entry,
 			_("Proc"), &proc_launch);
+
+/* --Info tab */
+	vbox = gkrellm_create_framed_tab(tabs, _("Info"));
+	text = gkrellm_scrolled_text(vbox, NULL,
+				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	for (i = 0; i < sizeof(proc_info_text)/sizeof(gchar *); ++i)
+		gkrellm_add_info_text_string(text, _(proc_info_text[i]));
 	}
 
 static Monitor	monitor_proc =
@@ -913,12 +1106,26 @@ static Monitor	monitor_proc =
 Monitor *
 init_proc_monitor(void)
 	{
+	ChartConfig	*cf;
+
 	monitor_proc.name = _(monitor_proc.name);
 	proc.enabled = TRUE;
-	load_resolution = 100;
+	proc.extra_info = TRUE;
 	style_id = gkrellm_add_chart_style(&monitor_proc, PROC_STYLE_NAME);
+	text_format = g_strdup(_(DEFAULT_TEXT_FORMAT));
+	mon_proc = &monitor_proc;
 	if (setup_proc_interface())
+		{
+		/* Set chart config defaults.  Turn off auto grid resolution and
+		|  don't let user config it back on.
+		*/
+		cf = proc.chart_config = gkrellm_chartconfig_new0();
+		gkrellm_set_chartconfig_grid_resolution(cf, 100);
+		gkrellm_set_chartconfig_auto_grid_resolution(cf, FALSE);
+		gkrellm_set_chartconfig_flags(cf, NO_CONFIG_AUTO_GRID_RESOLUTION);
+
 		return &monitor_proc;
+		}
 	return NULL;
 	}
 
