@@ -36,6 +36,7 @@
 
 #include <dirent.h>
 
+
 typedef struct
 	{
 	gchar		*name;			/* cpuX, mb, Vx name mapped to this sensor */
@@ -47,19 +48,24 @@ typedef struct
 #endif
 	gint		id;
 	gint		enabled;		/* voltage enable */
-
 	gfloat		factor;			/* Scale sensor reading		*/
 	gfloat		offset;			/* Add to sensor reading	*/
 
-	GtkWidget	*select_label;	/* Voltages tab */
+	GtkWidget	*select_button;	/* Voltages tab */
 	GtkWidget	*entry;
 	GtkWidget	*factor_spin_button;
 	GtkWidget	*offset_spin_button;
+	GtkWidget	*alert_button;
 
 	gfloat		value;
+
+	Alert		*alert;
+	void		(*cb_alert)();
+	gpointer	cb_alert_data;
 	}
 	Sensor;
 
+
 static GList	*sensor_list	= NULL;
 
 static gboolean	(*get_temperature)(Sensor *);
@@ -193,7 +199,7 @@ get_freebsd_sensor(Sensor *sensor)
 	}
 
 void
-scan_for_sensors()
+scan_for_sensors(void)
 	{
 	gchar		*chip_dir = SENSORS_DIR;
 #if __FreeBSD_version >= 300000
@@ -327,20 +333,26 @@ static gboolean
 get_linux_sensor(Sensor *sensor)
 	{
 	FILE	*f;
-	gchar	buf[64], sbuf[16];
+	gchar	buf[64], sbuf[3][16];
+	gint	n;
+	gfloat	t[3];
 
 	if ((f = fopen(sensor->path, "r")) == NULL)
 		return FALSE;
 	fgets(buf, sizeof(buf), f);
 	fclose(f);
+	t[0] = t[1] = t[2] = 0;
 	if (sensor->type == SENSOR_TEMPERATURE)
 		{
-		if (sscanf(buf, "%*f %*f %f", &sensor->value) != 1)
+		if ((n = sscanf(buf, "%*f %*f %f %f %f", &t[0], &t[1], &t[2])) < 1)
 			{	/* Could fail because of decimal "comma" locales */
-			if (sscanf(buf, "%*d.%*d %*d.%*d %16s", sbuf) != 1)
+			if ((n = sscanf(buf, "%*d.%*d %*d.%*d %16s %16s %16s",
+						sbuf[0], sbuf[1], sbuf[2])) < 1)
 				return FALSE;
-			sensor->value = locale_float_fix(sbuf);
+			sensor->value = locale_float_fix(sbuf[n - 1]);
 			}
+		else
+			sensor->value = t[n - 1];
 		}
 	else if (sensor->type == SENSOR_FAN)
 		{
@@ -351,9 +363,9 @@ get_linux_sensor(Sensor *sensor)
 		{
 		if (sscanf(buf, "%*f %*f %f", &sensor->value) != 1)
 			{
-			if (sscanf(buf, "%*d.%*d %*d.%*d %16s", sbuf) != 1)
+			if (sscanf(buf, "%*d.%*d %*d.%*d %16s", sbuf[0]) != 1)
 				return FALSE;
-			sensor->value = locale_float_fix(sbuf);
+			sensor->value = locale_float_fix(sbuf[0]);
 			}
 		}
 	else
@@ -364,7 +376,7 @@ get_linux_sensor(Sensor *sensor)
 
 
 static void
-scan_for_sensors()
+scan_for_sensors(void)
 	{
 	DIR				*dir, *chip_dir;
 	struct dirent	*dentry;
@@ -459,7 +471,7 @@ scan_for_sensors()
 /* ----- Pick a system interface ----------------------------------------- */
 
 static gboolean
-setup_sensor_interface()
+setup_sensor_interface(void)
 	{
 #if defined(__FreeBSD__)
 	get_temperature = get_freebsd_sensor;
@@ -497,7 +509,8 @@ strcmp_sensor_path(Sensor *s1, Sensor *s2)
 	if (s1->type != SENSOR_FAN && s2->type == SENSOR_FAN)
 		return 1;
 
-	return strcmp_basename(s1->path, s2->path);
+	return strcmp(s1->path, s2->path);
+//	return strcmp_basename(s1->path, s2->path);
 	}
 
   /* Return pointer to the sensor basename part of a sensor pathname.
@@ -548,7 +561,7 @@ map_sensor_name(gchar *name, gchar *sensor_name)
 	}
 
 gboolean
-gkrellm_sensors_available()
+gkrellm_sensors_available(void)
 	{
 	return (sensor_list || GK.demo) ? TRUE : FALSE;
 	}
@@ -650,6 +663,8 @@ get_mapped_sensor(gchar *name, gint sensor_type)
 	GList	*list;
 	Sensor	*sensor;
 
+	if (!name)
+		return NULL;
 	for (list = sensor_list; list; list = list->next)
 		{
 		sensor = (Sensor *) list->data;
@@ -661,6 +676,48 @@ get_mapped_sensor(gchar *name, gint sensor_type)
 	return NULL;
 	}
 
+Sensor *
+lookup_sensor_from_basename(gchar *basename)
+	{
+	GList	*list;
+	Sensor	*s;
+
+	if (!basename)
+		return NULL;
+	for (list = sensor_list; list; list = list->next)
+		{
+		s = (Sensor *) list->data;
+		if (!strcmp(sensor_basename(s->path), basename))
+			return s;
+		}
+	return NULL;
+	}
+
+Alert *
+gkrellm_mapped_sensor_alert(gchar *name, gint sensor_type)
+	{
+	Sensor	*sensor;
+
+	sensor = (Sensor *) get_mapped_sensor(name, sensor_type);
+	if (sensor)
+		return sensor->alert;
+	return NULL;
+	}
+
+void
+gkrellm_mapped_sensor_alert_connect(gchar *name, gint sensor_type,
+		void (*cb_func)(), gpointer data)
+	{
+	Sensor	*sensor;
+
+	sensor = (Sensor *) get_mapped_sensor(name, sensor_type);
+	if (!sensor)
+		return;
+	sensor->cb_alert = cb_func;
+	sensor->cb_alert_data = data;
+	gkrellm_alert_trigger_connect(sensor->alert, cb_func, data);
+	}
+
   /* Temperature names are mapped to "cpuX" or "mb".  The CPU and Proc monitors
   |  will ask for such names and the user must have made the map in the config
   */
@@ -692,6 +749,8 @@ gkrellm_sensor_read_temperature(gchar *name, gfloat *temp, gchar *units)
 		*units = units_fahrenheit ? 'F':'C';
 	if (GK.debug_level & DEBUG_SENSORS)
 		printf("read_temperature: %s(%d) t=%f\n", name, found_temp, t);
+	if (found_temp && sensor)
+		gkrellm_check_alert(sensor->alert, t);
 	return found_temp;
 	}
 
@@ -722,19 +781,18 @@ gkrellm_sensor_read_fan(gchar *name, gfloat *fan)
 		*fan = f;
 	if (GK.debug_level & DEBUG_SENSORS)
 		printf("read_fan: %s(%d) rpm=%f\n", name, found_fan, f);
+	if (found_fan && sensor)
+		gkrellm_check_alert(sensor->alert, f);
 	return found_fan;
 	}
 
-gboolean
-gkrellm_sensor_read_voltage(gchar *name, gfloat *voltage)
+static gboolean
+sensor_read_voltage(Sensor *sensor, gfloat *voltage)
 	{
-	Sensor	*sensor;
 	gfloat	v = 0;
 	gint	found_voltage = FALSE;
 
-	if (   (sensor = get_mapped_sensor(name, SENSOR_VOLTAGE)) != NULL
-		&& get_voltage && (*get_voltage)(sensor)
-	   )
+	if (sensor && get_voltage && (*get_voltage)(sensor))
 		{
 		sensor->value = sensor->value * sensor->factor + sensor->offset;
 		v = sensor->value;
@@ -748,10 +806,21 @@ gkrellm_sensor_read_voltage(gchar *name, gfloat *voltage)
 	if (voltage)
 		*voltage = v;
 	if (GK.debug_level & DEBUG_SENSORS)
-		printf("read_voltage: %s(%d) v=%f\n", name, found_voltage, v);
+		printf("read_voltage: %s(%d) v=%f\n", sensor->name, found_voltage, v);
+	if (found_voltage && sensor)
+		gkrellm_check_alert(sensor->alert, v);
 	return found_voltage;
 	}
 
+gboolean
+gkrellm_sensor_read_voltage(gchar *name, gfloat *voltage)
+	{
+	Sensor	*sensor;
+
+	sensor = get_mapped_sensor(name, SENSOR_VOLTAGE);
+	return sensor_read_voltage(sensor, voltage);
+	}
+
 /* =================================================================== */
 /* The voltage monitor */
 
@@ -785,6 +854,9 @@ static gint		volt_mon_width,
 #define	MONITOR_PAD		6
 #define	NAME_PAD		4
 
+Monitor			*mon_volt;
+Monitor			*mon_sensors;
+
 static GdkImlibImage	*bezel_image;
 
 static Style	*bezel_style;		/* Just for the bezel image border */
@@ -814,7 +886,7 @@ add_volt_mon(Sensor *s)
 	}
 
 static void
-get_volt_list()
+get_volt_list(void)
 	{
 	GList	*list;
 
@@ -845,7 +917,7 @@ get_volt_list()
 #include "pixmaps/sensors/bg_volt.xpm"
 
 static void
-draw_volt_bezels()
+draw_volt_bezels(void)
 	{
 	GList			*list;
 	GdkImlibBorder	*b		= &bezel_style->border;
@@ -868,16 +940,21 @@ draw_volt_bezels()
 	}
 
 static gboolean
-any_negative_volts()
+any_negative_volts(void)
 	{
 	GList		*list;
 	VoltMon		*volt;
 	gfloat		v;
 
+	/* This routine can be called before any volt decals exist, but reading
+	|  voltages can trigger alerts which expect to find decals.  Hence freeze.
+	*/
 	for (list = volt_list; list; list = list->next)
 		{
 		volt = (VoltMon *) list->data;
-		gkrellm_sensor_read_voltage(volt->sensor->name, &v);
+		gkrellm_freeze_alert(volt->sensor->alert);
+		sensor_read_voltage(volt->sensor, &v);
+		gkrellm_thaw_alert(volt->sensor->alert);
 		if (v < 0.0)
 			return TRUE;
 		}
@@ -1013,7 +1090,7 @@ draw_voltages(gint do_names)
 					volt->sensor->name, -1);
 		if ((d = volt->volt_decal) != NULL)
 			{
-			gkrellm_sensor_read_voltage(volt->sensor->name, &v);
+			sensor_read_voltage(volt->sensor, &v);
 			if ((v < 10.0 && v > 0.0) || (v > -10.0 && v < 0.0))
 				fmt = "%.2f";
 			else
@@ -1023,7 +1100,7 @@ draw_voltages(gint do_names)
 			gkrellm_draw_decal_text(pVolt, d, buf, -1);
 			}
 		}
-	gkrellm_draw_layers(pVolt);
+	gkrellm_draw_panel_layers(pVolt);
 	}
 
 static void
@@ -1045,14 +1122,51 @@ expose_event(GtkWidget *widget, GdkEventExpose *ev)
 	return FALSE;
 	}
 
+static gint
+cb_panel_press(GtkWidget *widget, GdkEventButton *ev)
+	{
+	if (ev->button == 3)
+		gkrellm_open_config_window(mon_sensors);
+	return TRUE;
+	}
+
 static void
-destroy_voltage_monitor()
+cb_volt_alert_trigger(Alert *alert, Sensor *sensor)
+	{
+	GList		*list;
+	VoltMon		*volt;
+	AlertDecal	*ad;
+	Decal		*d;
+
+	ad = &alert->ad;
+	alert->panel = pVolt;
+	for (list = volt_list; list; list = list->next)
+		{
+		volt = (VoltMon *) list->data;
+
+		/* Make the AlertDecal show up under the right volt decal.
+		*/
+		if (volt->sensor != sensor)
+			continue;
+		d = volt->volt_decal;
+		if (d)
+			{
+			ad->x = d->x - 2;
+			ad->y = d->y - 2;
+			ad->w = d->w + 3;
+			ad->h = d->h + 4;
+			gkrellm_render_default_alert_decal(alert);
+			}
+		break;
+		}
+	}
+
+static void
+destroy_voltage_monitor(void)
 	{
 	if (!pVolt)
 		return;
-	gkrellm_monitor_height_adjust( - pVolt->h);
-	gkrellm_destroy_panel(pVolt);
-	g_free(pVolt);
+	gkrellm_panel_destroy(pVolt);
 	pVolt = NULL;
 	}
 
@@ -1072,22 +1186,19 @@ create_voltage_monitor(GtkWidget *vbox, gint first_create)
 		pVolt = gkrellm_panel_new0();
 		bezel_style = gkrellm_style_new0();
 		}
-	else
-		{
-		gkrellm_destroy_decal_list(pVolt);
-		gkrellm_destroy_krell_list(pVolt);
-		}
 	style = gkrellm_meter_style(style_id);
 
-	/* No default bezel, theme must provide their own, so use the included
-	|  one only if this is the default theme.  The bezel image is bg_volt
+	/* Here is where I define the volt panel theme image extensions.  I ask
+	|  for a theme extension image:
+	|      THEME_DIR/sensors/bg_volt.png
+	|  and for a border for it from the gkrellmrc in the format:
+	|      set_image_border sensors_bg_volt l,r,t,b
+	| There is no default for bg_volt image, ie it may end up being NULL. 
 	*/
 	xpm = gkrellm_using_default_theme() ? bg_volt_xpm : NULL;
 	if (bezel_image)
-		{
 		gdk_imlib_kill_image(bezel_image);
-		bezel_image = NULL;
-		}
+	bezel_image = NULL;
 	gkrellm_load_image("bg_volt", xpm, &bezel_image, SENSOR_STYLE_NAME);
 	if (!gkrellm_set_image_border("sensors_bg_volt", bezel_image, bezel_style))
 		bezel_style->border = default_bezel_border;
@@ -1095,14 +1206,14 @@ create_voltage_monitor(GtkWidget *vbox, gint first_create)
 	make_volt_decals(pVolt, style);
 	layout_volt_decals(pVolt, style);
 	
-	gkrellm_configure_panel(pVolt, NULL, style);
+	gkrellm_panel_configure(pVolt, NULL, style);
 
 	/* Make the bottom margin reference against the bottom volt decals
 	|  bezel image.  The volt decal height does not include the bezel so
-	|  gkrellm_configure_panel() did not account for the bezel.
+	|  gkrellm_panel_configure() did not account for the bezel.
 	*/
 	pVolt->label->h_panel += bezel_style->border.bottom;
-	gkrellm_create_panel(vbox, pVolt, gkrellm_bg_meter_image(style_id));
+	gkrellm_panel_create(vbox, mon_volt, pVolt);
 
 	draw_volt_bezels();
 
@@ -1110,8 +1221,9 @@ create_voltage_monitor(GtkWidget *vbox, gint first_create)
 		{
 		gtk_signal_connect(GTK_OBJECT(pVolt->drawing_area), "expose_event",
 				(GtkSignalFunc) expose_event, NULL);
+		gtk_signal_connect(GTK_OBJECT(pVolt->drawing_area),
+				"button_press_event", (GtkSignalFunc) cb_panel_press, NULL);
 		}
-	gkrellm_monitor_height_adjust(pVolt->h);
 	draw_voltages(TRUE);
 	}
 
@@ -1124,7 +1236,7 @@ create_voltage(GtkWidget *vbox, gint first_create)
 
 static Monitor	monitor_voltages =
 	{
-	NULL,				/* Voltage config handled in Sensors tab */
+	N_("Voltage"),		/* Voltage config handled in Sensors tab */
 	MON_VOLTAGE,		/* Id,  0 if a plugin		*/
 	create_voltage,		/* The create function		*/
 	update_voltage,		/* The update function		*/
@@ -1149,6 +1261,7 @@ Monitor *
 init_voltage_monitor(void)
 	{
 	style_id = gkrellm_add_meter_style(&monitor_voltages, SENSOR_STYLE_NAME);
+	mon_volt = &monitor_voltages;
 	return &monitor_voltages;
 	}
 
@@ -1158,6 +1271,35 @@ init_voltage_monitor(void)
 
 #define	SENSOR_CONFIG_KEYWORD	"sensor"
 
+static void
+create_sensor_alert(Sensor *s)
+	{
+	if (s->type == SENSOR_VOLTAGE)
+		{
+		s->alert = gkrellm_alert_create(NULL, s->name,
+				_("Sensor Volt Limits"),
+				TRUE, TRUE, TRUE, 20, -20, 0.01, 0.5, 2);
+		gkrellm_alert_trigger_connect(s->alert, cb_volt_alert_trigger, s);
+		}
+	else if (s->type == SENSOR_TEMPERATURE)
+		{
+		s->alert = gkrellm_alert_create(NULL, s->name,
+				_("Sensor Temperature Limits (in displayed degree units)"),
+				TRUE, FALSE, TRUE, 300, 0, 1.0, 5.0, 1);
+		gkrellm_alert_trigger_connect(s->alert, s->cb_alert, s->cb_alert_data);
+		}
+	else if (s->type == SENSOR_FAN)
+		{
+		s->alert = gkrellm_alert_create(NULL, s->name,
+				_("Sensor Fan RPM Limits"),
+				FALSE, TRUE, TRUE, 20000, 0, 100, 1000, 0);
+		gkrellm_alert_trigger_connect(s->alert, s->cb_alert, s->cb_alert_data);
+		}
+	else
+		return;
+	}
+
+
 static void
 save_sensors_config(FILE *f)
 	{
@@ -1171,6 +1313,9 @@ save_sensors_config(FILE *f)
 			{
 			fprintf(f, "%s %s %s %f %f %d\n", SENSOR_CONFIG_KEYWORD, s->name,
 				sensor_basename(s->path), s->factor, s->offset, s->enabled);
+			if (s->alert)
+				gkrellm_save_alertconfig(f, s->alert,
+						SENSOR_CONFIG_KEYWORD, sensor_basename(s->path));
 			}
 		}
 	fprintf(f, "%s units_fahrenheit %d\n", SENSOR_CONFIG_KEYWORD,
@@ -1183,7 +1328,7 @@ static void
 load_sensors_config(gchar *arg)
 	{
 	Sensor	*s;
-	gchar	config[32], item[CFG_BUFSIZE];
+	gchar	config[32], item[CFG_BUFSIZE], item1[CFG_BUFSIZE];
 	gchar	name[32], sensor_name[CFG_BUFSIZE];
 	gint	n;
 	gfloat	f	= 1.0,
@@ -1199,6 +1344,17 @@ load_sensors_config(gchar *arg)
 		sscanf(item, "%d", &units_fahrenheit);
 	if (strcmp(config, "volt_display_mode") == 0)
 		sscanf(item, "%d", &display_mode);
+	else if (!strcmp(config, GKRELLM_ALERTCONFIG_KEYWORD))
+		{
+		n = sscanf(item, "%31s %[^\n]", name, item1);
+		s = lookup_sensor_from_basename(name);
+		if (s && n == 2)
+			{
+			if (!s->alert)
+				create_sensor_alert(s);
+			gkrellm_load_alertconfig(&s->alert, item1);
+			}
+		}
 	else if (   sscanf(arg, "%31s %s %f %f %d",
 						name, sensor_name, &f, &o, &e) > 1
 			 && (s = map_sensor_name(name, sensor_name)) != NULL
@@ -1219,13 +1375,56 @@ static GtkWidget	*fahrenheit_button;
 static GtkWidget	*display_mode_button[2];
 
 static void
-apply_sensors_config()
+set_alert_button_state(Sensor *s)
+	{
+	gboolean	activated;
+
+	if (   (s->type == SENSOR_VOLTAGE && !s->enabled)
+		|| (s->type != SENSOR_VOLTAGE && !*(s->name))
+	   )
+		gtk_widget_set_sensitive(s->alert_button, FALSE);
+	else
+		gtk_widget_set_sensitive(s->alert_button, TRUE);
+
+	activated = (!s->alert || !s->alert->activated) ? FALSE : TRUE;
+	gtk_label_set_text(GTK_LABEL(GTK_BIN(s->alert_button)->child),
+			activated ? "*" : "-");
+	}
+
+static void
+fix_temp_alert(Sensor *s)
+	{
+	Alert	*a = s->alert;
+
+	if (!a)
+		return;
+	if (units_fahrenheit)
+		{
+		if (a->high.warn_limit > 0)
+			a->high.warn_limit = a->high.warn_limit * 9.0 / 5.0 + 32.0;
+		if (a->high.alarm_limit > 0)
+			a->high.alarm_limit = a->high.alarm_limit * 9.0 / 5.0 + 32.0;
+		}
+	else
+		{
+		if (a->high.warn_limit > 0)
+			a->high.warn_limit = (a->high.warn_limit - 32.0) * 5.0 / 9.0;
+		if (a->high.alarm_limit > 0)
+			a->high.alarm_limit = (a->high.alarm_limit - 32.0) * 5.0 / 9.0;
+		}
+	gkrellm_alert_window_destroy(&s->alert);
+	}
+
+static void
+apply_sensors_config(void)
 	{
 	GList	*list;
 	Sensor	*s;
 	gchar	*name;
 	gint	i, previous;
 
+	previous = units_fahrenheit;
+	units_fahrenheit = GTK_TOGGLE_BUTTON(fahrenheit_button)->active;
 	for (list = sensor_list; list; list = list->next)
 		{
 		s = (Sensor *) list->data;
@@ -1236,9 +1435,23 @@ apply_sensors_config()
 				{
 				name = voltage_map[s->id].name;
 				gtk_entry_set_text(GTK_ENTRY(s->entry), name);
-				gtk_label_set_text(GTK_LABEL(s->select_label), name);
+				gtk_label_set_text(
+						GTK_LABEL(GTK_BIN(s->select_button)->child), name);
+				}
+			if (gkrellm_dup_string(&s->name, name) && s->alert)
+				{
+				if (s->type == SENSOR_VOLTAGE)
+					{
+					g_free(s->alert->name);
+					s->alert->name = g_strdup_printf("%s %s", s->name,
+							_("Voltage"));
+					}
+				else
+					{
+					gkrellm_alert_destroy(&s->alert);
+					set_alert_button_state(s);
+					}
 				}
-			gkrellm_dup_string(&s->name, name);
 			}
 		if (s->factor_spin_button)
 			s->factor = gtk_spin_button_get_value_as_float(
@@ -1246,8 +1459,13 @@ apply_sensors_config()
 		if (s->offset_spin_button)
 			s->offset = gtk_spin_button_get_value_as_float(
 						GTK_SPIN_BUTTON(s->offset_spin_button));
+		set_alert_button_state(s);
+		if (   s->type == SENSOR_TEMPERATURE
+			&& previous != units_fahrenheit
+			&& s->alert
+		   )
+			fix_temp_alert(s);
 		}
-	units_fahrenheit = GTK_TOGGLE_BUTTON(fahrenheit_button)->active;
 	previous = display_mode;
 	for (i = 0; i < N_DISPLAY_MODES; ++i)
 		if (GTK_TOGGLE_BUTTON(display_mode_button[i])->active)
@@ -1278,7 +1496,7 @@ cb_correction_modified(void)
 static void
 cb_volt_enabled(GtkWidget *button, Sensor *voltage)
 	{
-	voltage->enabled = 1 - voltage->enabled;
+	voltage->enabled = !voltage->enabled;
 	voltages_modified = TRUE;
 	}
 
@@ -1289,10 +1507,26 @@ cb_volt_entry(GtkWidget *entry, Sensor *voltage)
 
 	voltages_modified = TRUE;
 	name = gkrellm_entry_get_text(&entry);
-	if (voltage->select_label)
-		gtk_label_set_text(GTK_LABEL(voltage->select_label), name);
+	if (voltage->select_button)
+		gtk_label_set_text(
+				GTK_LABEL(GTK_BIN(voltage->select_button)->child), name);
 	}
 
+static void
+cb_sensor_alert_config(Alert *alert, Sensor *sensor)
+	{
+	if (gkrellm_config_window_shown())
+		set_alert_button_state(sensor);
+	}
+
+static void
+cb_sensor_set_alert(GtkWidget *entry, Sensor *s)
+	{
+	if (!s->alert)
+		create_sensor_alert(s);
+	gkrellm_alert_config_connect(s->alert, cb_sensor_alert_config, s);
+	gkrellm_alert_config_window(&s->alert);
+	}
 
 static gchar	*sensor_info_text0[]	= 
 	{
@@ -1338,15 +1572,14 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	{
 	GtkWidget	*tabs;
 	GtkWidget	*table;
-	GtkWidget	*frame;
 	GtkWidget	*button;
 	GtkWidget	*text;
-	GtkWidget	*scrolled;
-	GtkWidget	*vbox, *hbox, *box;
+	GtkWidget	*vbox, *vbox1, *hbox, *box;
 	GtkWidget	*label;
 	GtkWidget	*entry;
 	GList		*list;
-	Sensor		*s;
+	Sensor		*sr;
+	gchar		*s, *dirname, *chipname;
 	gint		i, n;
 
 	tabs = gtk_notebook_new();
@@ -1354,10 +1587,10 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);
 
 /* --Setup tab */
-	vbox = gkrellm_create_tab(tabs, _("Setup"));
+	vbox = gkrellm_create_framed_tab(tabs, _("Setup"));
 
-	table = gtk_table_new(2, 4, FALSE);
-	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
+	table = gtk_table_new(2, 5, FALSE);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 0);
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 3);
 
 	label = gtk_label_new(_("Sensor Label"));
@@ -1376,87 +1609,87 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 2, 3, 1, 2);
 	label = gtk_label_new(_("Offset"));
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 3, 4, 1, 2);
+	label = gtk_label_new(_("Alerts"));
+	gtk_table_attach_defaults(GTK_TABLE(table), label, 4, 5, 1, 2);
 
-	scrolled = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
-		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
+	box = gkrellm_scrolled_vbox(vbox, NULL,
+			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
-	box = gtk_vbox_new(FALSE, 3);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled), box);
-	table = gtk_table_new(g_list_length(sensor_list), 4, FALSE);
-	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
+	table = gtk_table_new(g_list_length(sensor_list), 5, FALSE);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 2);
 	gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 2);
 
 	for (i = 0, list = sensor_list; list; list = list->next, ++i)
 		{
-		s = (Sensor *) list->data;
+		sr = (Sensor *) list->data;
 		entry = gtk_entry_new_with_max_length(8);
-		s->entry = entry;
-		gtk_entry_set_text(GTK_ENTRY(entry), s->name);
+		sr->entry = entry;
+		gtk_entry_set_text(GTK_ENTRY(entry), sr->name);
 		gtk_widget_set_usize(entry, 36, 0);
 		gtk_table_attach_defaults(GTK_TABLE(table), entry, 0, 1, i, i + 1);
-		if (s->type == SENSOR_VOLTAGE)
+		if (sr->type == SENSOR_VOLTAGE)
 			gtk_signal_connect(GTK_OBJECT(entry), "changed",
-						(GtkSignalFunc) cb_volt_entry, s);
+						(GtkSignalFunc) cb_volt_entry, sr);
 
-		label = gtk_label_new(sensor_basename(s->path));
+		label = gtk_label_new(sensor_basename(sr->path));
 		gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, i, i + 1);
 
 		box = gtk_hbox_new(TRUE, 0);
 		gtk_table_attach_defaults(GTK_TABLE(table), box, 2, 3, i, i+1);
-		gkrellm_spin_button(box, &s->factor_spin_button, s->factor, -1000.0,
+		gkrellm_spin_button(box, &sr->factor_spin_button, sr->factor, -1000.0,
 					1000.0, 0.01, 1.0, 4, 60, NULL, NULL, FALSE, NULL);
-		gtk_signal_connect(GTK_OBJECT(s->factor_spin_button), "changed",
+		gtk_signal_connect(GTK_OBJECT(sr->factor_spin_button), "changed",
 					(GtkSignalFunc) cb_correction_modified, NULL);
-		if (s->type != SENSOR_FAN)
+		if (sr->type != SENSOR_FAN)
 			{
-			n = (s->type == SENSOR_VOLTAGE) ? 3 : 2;
+			n = (sr->type == SENSOR_VOLTAGE) ? 3 : 2;
 			box = gtk_hbox_new(TRUE, 0);
 			gtk_table_attach_defaults(GTK_TABLE(table), box, 3, 4, i, i+1);
-			gkrellm_spin_button(box, &s->offset_spin_button, s->offset,
+			gkrellm_spin_button(box, &sr->offset_spin_button, sr->offset,
 				-10000.0, 10000.0, 1.0, 5.0, n, 60, NULL, NULL, FALSE, NULL);
-			gtk_signal_connect(GTK_OBJECT(s->offset_spin_button), "changed",
+			gtk_signal_connect(GTK_OBJECT(sr->offset_spin_button), "changed",
 					(GtkSignalFunc) cb_correction_modified, NULL);
 			}
+		box = gtk_hbox_new(FALSE, 0);
+		gtk_table_attach_defaults(GTK_TABLE(table), box, 4, 5, i, i+1);
+		gkrellm_button_connected(box, &sr->alert_button, TRUE, TRUE, 10,
+				cb_sensor_set_alert, sr, "");
+		set_alert_button_state(sr);
 		}
 	gkrellm_check_button(vbox, &fahrenheit_button, units_fahrenheit, FALSE, 2,
 				_("Display fahrenheit"));
 
 /* --Voltages tab */
-	vbox = gkrellm_create_tab(tabs, _("Voltages"));
-	hbox = gtk_hbox_new (FALSE, 0);	
+	vbox = gkrellm_create_framed_tab(tabs, _("Voltages"));
+	hbox = gtk_hbox_new (TRUE, 0);	
 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
-	frame = gtk_frame_new (_("Select"));
-	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 2);
-	box = gtk_vbox_new (FALSE, 2);
-	gtk_container_set_border_width(GTK_CONTAINER(box), 2);
-	gtk_container_add(GTK_CONTAINER(frame), box);
-
+	vbox1 = gkrellm_framed_vbox(hbox, _("Select"), 4, TRUE, 0, 2);
+	vbox1 = gkrellm_scrolled_vbox(vbox1, NULL,
+			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	box = vbox1;
+	dirname = chipname = NULL;
 	for (list = sensor_list; list; list = list->next)
 		{
-		s = (Sensor *) list->data;
-		if (s->type != SENSOR_VOLTAGE)
+		sr = (Sensor *) list->data;
+		if (sr->type != SENSOR_VOLTAGE)
 			continue;
-		button = gtk_check_button_new();
-		label = gtk_label_new(s->name);
-		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-		gtk_container_add(GTK_CONTAINER(button), label);
-
-		gtk_container_add(GTK_CONTAINER(box), button);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), s->enabled);
-		gtk_signal_connect(GTK_OBJECT(button), "toggled",
-				(GtkSignalFunc) cb_volt_enabled, s);
-		s->select_label = label;
+		s = sensor_basename(sr->path);
+		dirname = g_dirname(s);
+		if (*dirname != '.' && (!chipname || strcmp(dirname, chipname)))
+			{
+			chipname = dirname;
+			dirname = NULL;
+			box = gkrellm_framed_vbox(vbox1, chipname, 4, TRUE, 0, 2);
+			}
+		gkrellm_check_button_connected(box, &sr->select_button, sr->enabled,
+					FALSE, FALSE, 0, cb_volt_enabled, sr, sr->name);
 		}
-
-	frame = gtk_frame_new (_("Display Mode"));
-	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 2);
-	box = gtk_vbox_new (FALSE, 2);
-	gtk_container_set_border_width(GTK_CONTAINER(box), 2);
-	gtk_container_add(GTK_CONTAINER(frame), box);
-	insert_expanded_filler(box);
+	if (chipname)
+		g_free(chipname);
+	if (dirname)
+		g_free(dirname);
+	box = gkrellm_framed_vbox(hbox, _("Display Mode"), 4, FALSE, 0, 2);
 
 	button = gtk_radio_button_new_with_label(NULL,
 				_("Normal with labels"));
@@ -1474,24 +1707,17 @@ create_sensors_tab(GtkWidget *tab_vbox)
 	insert_expanded_filler(box);
 
 /* --Info tab */
-	vbox = gkrellm_create_tab(tabs, _("Info"));
+	vbox = gkrellm_create_framed_tab(tabs, _("Info"));
+	text = gkrellm_scrolled_text(vbox, NULL,
+				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 
-	scrolled = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
-					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
-
-	text = gtk_text_new(NULL, NULL);
 	for (i = 0; i < sizeof(sensor_info_text0) / sizeof(gchar *); ++i)
 		gkrellm_add_info_text_string(text, _(sensor_info_text0[i]));
-	if ((i = smp_cpus) > 3)
+	if ((i = n_smp_cpus) > 3)
 		i = 3;
 	gkrellm_add_info_text_string(text, sensor_name_set[i]);
 	for (i = 0; i < sizeof(sensor_info_text1) / sizeof(gchar *); ++i)
 		gkrellm_add_info_text_string(text, _(sensor_info_text1[i]));
-
-	gtk_text_set_editable(GTK_TEXT(text), FALSE);
-	gtk_container_add(GTK_CONTAINER(scrolled), text);
 	}
 
 
@@ -1525,6 +1751,7 @@ init_sensors_monitor(void)
 		return NULL;
 	sensor_list = g_list_sort(sensor_list, (GCompareFunc) strcmp_sensor_path);
     monitor_sensors.name = _(monitor_sensors.name);
+	mon_sensors = &monitor_sensors;
 	return &monitor_sensors;
 	}
 
